/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include "chart/BaseChart.h"
#include <MHDate.h>
#include <MHDayChart.h>
#include <MHDC.h>



/**
*
*/
BaseChart::BaseChart (MHDC* dc, int view, MHDate::DAYS timeType) {
    MHDate start ("19700101");
    MHDate stop;

    aView       = view;
    aDC         = dc;
    aChart      = 0;
    aData       = 0;
    aLayout     = (XTRADER_CHART_LAYOUT) 0;
    aTickWidth  = 0;
    aBGColor    = 0;
    aVSpace     = 0;
    aBSpace     = 0;
    aTimeType   = timeType;
    aStartDate  = start.Get (MHDate::LONG_DATE);
    aStopDate   = stop.Get (MHDate::LONG_DATE);
}



/**
*  Check date, swap dates if start date is greater than stop date
*/
void BaseChart::CheckDate () {
    if (aStartDate > aStopDate) {
        MHString tmp = aStartDate;
        aStartDate   = aStopDate;
        aStopDate    = tmp;
    }
}




/**
*  Copy data from cache series to local vector.
*  Convert time serie if settings is set to week time serie.
*  Clip data also if outside user date range.
*/
void BaseChart::copy (MHVector* v1, MHVector* v2, MHVector* ve1, MHVector* ve2) {
    if (aTimeType ==  MHDate::FRIDAY) {
        MHVector* tmp = new MHVector ();
        MHPrice::Day2Week (v1, tmp, MHDate::FRIDAY);
        MHPrice::Copy (tmp, ve1, aStartDate(), aStopDate());
        if (v2)
            MHPrice::Copy (v2, ve2, aStartDate(), aStopDate());
        tmp->Erase ();
        delete tmp;
    }
    else {
        MHPrice::Copy (v1, ve1, aStartDate(), aStopDate());
        if (v2)
            MHPrice::Copy (v2, ve2, aStartDate(), aStopDate());
    }
}



/**
*  Create new chart view
*  @param MHDC& - The device context
*/
void BaseChart::createView () {
    delete aChart;
    aChart = 0;

    switch (aLayout) {
        case XTRADER_SINGLE_LAYOUT:
            aChart = new MHDayChart (aVSpace, aBSpace, 100);
            break;

        case XTRADER_DOUBLE_LAYOUT:
            aChart = new MHDayChart (aVSpace, aBSpace, 50, 50);
            break;

        case XTRADER_DOUBLE2_LAYOUT:
            aChart = new MHDayChart (aVSpace, aBSpace, 70, 30);
            break;

        case XTRADER_TRIPLE_LAYOUT:
            aChart = new MHDayChart (aVSpace, aBSpace, 34, 33, 33);
            break;

        case XTRADER_TRIPLE2_LAYOUT:
            aChart = new MHDayChart (aVSpace, aBSpace, 50, 25, 25);
            break;

        default:
            aChart = new MHDayChart (aVSpace, aBSpace, 100);
            break;
    }
    aChart->aTickWidth = aTickWidth;
    aChart->aBgColor   = ((aBGColor == 0) ? MHDC::WHITE : aBGColor);
}



/**
* Get align of chart
* @param int   - What chart area
* @return int  - MHDayChart::LEFT or MHDayChart::RIGHT
*/
int BaseChart::getAlign (int win) {
   switch (win) {
       case XTRADER_WINDOW_TOP:
            return MHDayChart::RIGHT;

        case XTRADER_WINDOW_TOP2:
            return MHDayChart::LEFT;

        case XTRADER_WINDOW_MIDDLE:
            return MHDayChart::RIGHT;

        case XTRADER_WINDOW_MIDDLE2:
            return MHDayChart::LEFT;

        case XTRADER_WINDOW_BOTTOM:
            return MHDayChart::RIGHT;

        case XTRADER_WINDOW_BOTTOM2:
            return MHDayChart::LEFT;

        default:
            return MHDayChart::LEFT;
    }
}



/**
* Get area of chart
* @param int   - What chart area
* @return int  - MHDayChart::LEFT or MHDayChart::RIGHT
*/
int BaseChart::getArea (int win) {
    switch (win) {
        case XTRADER_WINDOW_TOP:
        case XTRADER_WINDOW_TOP2:
            return 0;

        case XTRADER_WINDOW_MIDDLE:
        case XTRADER_WINDOW_MIDDLE2:
            return 1;

        case XTRADER_WINDOW_BOTTOM:
        case XTRADER_WINDOW_BOTTOM2:
            return 2;

        default:
            return 0;
    }
}



/**
* Get align of chart
* @param int   - What chart area
* @return int  - MHDayChart::LEFT or MHDayChart::RIGHT
*/
int BaseChart::getArea2 (int win) {
    switch (win) {
        case XTRADER_SINGLE_LAYOUT:
            return 0;
            
        case XTRADER_DOUBLE_LAYOUT:
        case XTRADER_DOUBLE2_LAYOUT:
            return 1;

        case XTRADER_TRIPLE_LAYOUT:
        case XTRADER_TRIPLE2_LAYOUT:
            return 2;

        default:
            return 0;
    }
}



/**
* Set tick width
* @param int   - New tick width
*/
void BaseChart::SetTickWidth (int width) {
    aTickWidth += width;
    
    if (aTickWidth < XTRADER_MIN_LINE_WIDTH)
        aTickWidth = XTRADER_MIN_LINE_WIDTH;
    else if (aTickWidth > XTRADER_MAX_LINE_WIDTH)
        aTickWidth = XTRADER_MAX_LINE_WIDTH;
    Create ();
}

