/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#ifndef BaseChart_h
#define BaseChart_h

#include "report/ReportTypes.h"
#include "Enums.h"
#include <MHDate.h>
#include <MHString.h>

class MHDayChart;
class MHDC;



/**
*  Base class for reports. All child classes must implement Create.
*  Creater of the report must delete the BaseChartFormat object when finished.
*/
class BaseChart {
public:
    MH*                     aData;
    MHDayChart*             aChart;
    MHString                aStartDate;
    MHString                aStopDate;
    ReportTypes             aChoice;

                            BaseChart (MHDC* dc, int view, MHDate::DAYS timeType);
    virtual                 ~BaseChart () {;}

    virtual const char*     Class () {return "BaseChart";}
    virtual XTRADER_DATE    ChangeDate () {return XTRADER_BOTH_DATE;}
    void                    CheckDate ();
    virtual void            Create () = 0;
    virtual void            Reload (const char* message){;}
    void                    SetTickWidth (int width);
   
protected:
    int                     aWindows[XTRADER_LAST_WINDOW + 1][3];
    int                     aLayout;
    int                     aView;
    int                     aTickWidth;
    int                     aVSpace;
    int                     aBSpace;
    int                     aBGColor;
    MHDate::DAYS            aTimeType;
    MHDC*                   aDC;

    void                    copy (MHVector* v1, MHVector* v2, MHVector* ve1, MHVector* ve2);
    void                    createView ();
    int                     getAlign (int win);
    int                     getArea (int win);
    int                     getArea2 (int win);
};

#endif
