/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include "chart/PortfolioChart.h"
#include "db/Account.h"
#include "db/DBFactory.h"
#include "db/Portfolio.h"
#include "db/Portfolio2.h"
#include "db/Price.h"
#include "db/Security.h"
#include "report/PortfolioValue.h"
#include "Progress.h"
#include "Resource.h"
#include <MHDayChart.h>
#include <MHDate.h>
#include <MHDC.h>
#include <MHDebug.h>
#include <MHPrice.h>



/**
*
*/
PortfolioChart::PortfolioChart (MHDC* dc, int view) :
BaseChart (dc, view, MHDate::FRIDAY) {
    MHDate start;

    start.Set (start.GetYear(), 1, 1);
    aStartDate = start.Get (MHDate::LONG_DATE);

    aChoice.aNames.Push (new MHString (STRING(CHARTINDICATOR_NO_CHART)));
    aChoice.aTypes.Push (new MHString ((int)NO_CHART));

    aChoice.aNames.Push (new MHString (STRING(PORTFOLIO_PERFORMANCE)));
    aChoice.aTypes.Push (new MHString ((int)PERFORMANCE_CHART));

    aChoice.aNames.Push (new MHString (STRING(PORTFOLIO_VALUE)));
    aChoice.aTypes.Push (new MHString ((int)VALUE_CHART));

    aChoice.aNames.Push (new MHString (STRING(CHARTINDICATOR_RELATIVE)));
    aChoice.aTypes.Push (new MHString ((int)RELATIVE_CHART));

    Reload ("PrefChartArea");
}



/**
*  Create new charts
*/
void PortfolioChart::Create () {
    DBG ("Create")
    DBG_LOG2 ("aLayout=%d  aTickWidth=%d", aLayout, aTickWidth);

    if (!aDC || !aData) return;
    createView();

    //  Draw as many chart areas as user has selected
    for (int f = XTRADER_DEFAULT_WINDOW; f <= XTRADER_LAST_WINDOW; f++) {
        int align    = getAlign (f);
        int area     = getArea (f);
        int usedArea = getArea2 (aLayout);

        for (int e = 0; e <= 2; e++) {
            int chart = aWindows[f][e];

            if (area > usedArea || chart == NO_CHART) continue;

            switch (chart) {
                case PERFORMANCE_CHART:
                    createPerformance (align, area);
                    break;

                case RELATIVE_CHART:
                    createRelative (align, area);
                    break;

                case VALUE_CHART:
                    createValue (align, area);
                    break;

                default:
                    break;
            }
        }
    }
}




/**
*  Create a performance chart for portfolio
*/
void PortfolioChart::createPerformance (int align, int area) {
    MHVector*       v1   = 0;
    MHVector*       ve1  = new MHVector ();
    MHVector*       ve2  = new MHVector ();
    MHVector*       ve3  = new MHVector ();
    MHVector*       ve4  = new MHVector ();
    MHVector*       ve5  = new MHVector ();
    MHVector*       ve6  = new MHVector ();
    Account*        acc  = (Account*) aData;
    int             id   = DBFactory::GetPrefIntValue (STRING(PREF_CHARTP_RELATIVE), aView, 0, 999999999, 0);
    Security        stmp(id, "", "");
    Security*       sec  = (Security*) DBFactory::LoadSecurity()->Find (&stmp, MHVector::FIND_EXACT, Security::SORT_ID, 0);
    char            buff[100];
    PortfolioValue  value;

    Portfolio2::LoadPortfolio (acc, ve1);
    Portfolio2::Performance (ve1, ve2, ve3, 0, &aStartDate, &aStopDate, &value);
    copy (ve3, 0, ve4, 0);

    if (sec && ve4->Size() > 0) {
        MHPrice* p1 = (MHPrice*) ve4->Get (0);
        v1 = DBFactory::LoadPrice (sec);
        copy (v1, 0, ve5, 0);
        MHPrice::Normalise (ve5, ve6, p1->aHigh);
        if (ve6->Size() > ve4->Size())
            MHPrice::Cut (ve4, ve6);
    }

    #ifdef BUGG
        MHPrice::Save ("dbg_portfolio_chart.txt", ve4);
    #endif

    snprintf (buff, 99, "%s %s", acc->GetAccount()->Get(), STRING(PORTFOLIO_PERFORMANCE));
    aChart->AddData (buff, MHDayChart::LINE_CHART, align, area, ve4);
    if (sec)
        aChart->AddData (sec->GetName()->Get(), MHDayChart::LINE_CHART, align, area, ve6);
    else
        delete ve6;
    aChart->AddDates (aTimeType);
    ve1->Erase ();
    ve2->Erase ();
    ve3->Erase ();
    ve5->Erase ();
    delete ve1;
    delete ve2;
    delete ve3;
    delete ve5;
}



/**
*  Create a relative price chart
*/
void PortfolioChart::createRelative (int align, int area) {
    MHVector*       v1   = 0;
    MHVector*       ve1  = new MHVector ();
    MHVector*       ve2  = new MHVector ();
    int             id   = DBFactory::GetPrefIntValue (STRING(PREF_CHARTP_RELATIVE), aView, 0, 999999999, 0);
    Security        stmp(id, "", "");
    Security*       sec  = (Security*) DBFactory::LoadSecurity()->Find (&stmp, MHVector::FIND_EXACT, Security::SORT_ID, 0);

    if (sec) v1 = DBFactory::LoadPrice (sec);
    copy (v1, 0, ve1, 0);
    MHPrice::Normalise (ve1, ve2, 100);

    if (sec)
        aChart->AddData (sec->GetName()->Get(), MHDayChart::LINE_CHART, align, area, ve2);
    else
        delete ve2;
    aChart->AddDates (aTimeType);
    ve1->Erase ();
    delete ve1;
}



/**
*  Create a value chart for portfolio
*/
void PortfolioChart::createValue (int align, int area) {
    MHVector*       ve1  = new MHVector ();
    MHVector*       ve2  = new MHVector ();
    MHVector*       ve3  = new MHVector ();
    MHVector*       ve4  = new MHVector ();
    Account*        acc  = (Account*) aData;
    PortfolioValue  value;
    char            buff[100];

    Portfolio2::LoadPortfolio (acc, ve1);
    Portfolio2::Performance (ve1, ve2, 0, ve3, &aStartDate, &aStopDate, &value);
    copy (ve3, 0, ve4, 0);

    #ifdef BUGG
        MHPrice::Save ("dbg_portfolio_chart.txt", ve4);
    #endif

    snprintf (buff, 99, "%s %s", acc->GetAccount()->Get(), STRING(PORTFOLIO_VALUE));
    aChart->AddData (buff, MHDayChart::LINE_CHART, align, area, ve4);
    aChart->AddDates (aTimeType);

    ve1->Erase ();
    ve2->Erase ();
    ve3->Erase ();
    delete ve1;
    delete ve2;
    delete ve3;
}



/**
*  Check is the message is for me and create chart again if so
*/
void PortfolioChart::Reload (const char* message) {
    DBG ("Reload")

    if (message == 0 || *message == '\0')
        Create();
    else if (strcmp (message, "Price") == 0)
        Create();
    else if (strcmp (message, "Portfolio") == 0)
        Create();
    else if (strcmp (message, "PrefChartArea") == 0) {
        //  Load all default settings from preference file
        aLayout    = (XTRADER_CHART_LAYOUT) DBFactory::GetPrefIntValue (STRING(PREF_CHARTP_LAYOUT), aView, XTRADER_DEFAULT_LAYOUT, XTRADER_LAST_LAYOUT, XTRADER_DEFAULT_LAYOUT);
        aTickWidth = DBFactory::GetPrefIntValue (STRING(PREF_CHARTP_LINEWIDTH), aView, XTRADER_MIN_LINE_WIDTH, XTRADER_MAX_LINE_WIDTH, XTRADER_DEFAULTP_LINE_WIDTH);
        aBGColor   = DBFactory::GetPrefIntValue (STRING(PREF_CHARTP_BGCOLOR), aView, 0, 255, 0);
        aVSpace    = DBFactory::GetPrefIntValue (STRING(PREF_CHARTP_VSPACE), aView, XTRADER_MIN_VSPACE, XTRADER_MIN_VSPACE, XTRADER_MAX_VSPACE);
        aBSpace    = DBFactory::GetPrefIntValue (STRING(PREF_CHARTP_BSPACE), aView, XTRADER_MIN_BSPACE, XTRADER_MAX_BSPACE, XTRADER_DEFAULT_BSPACE);

        for (int f = XTRADER_DEFAULT_WINDOW; f <= XTRADER_LAST_WINDOW; f++) {
            for (int e = 0; e <= 2; e++) {
                char b[100];
                snprintf (b, 99, "%s_%d_%d_%d", STRING(PREF_CHARTP_WINDOW), aView, f, e + 1);
                aWindows[f][e] = (XTRADER_CHART_WINDOW) DBFactory::GetPrefIntValue (b, NO_CHART, LAST_CHART, NO_CHART);
            }
        }

        DBG_LOG3 ("aLayout=%d  aTickWidth=%d  aTimeType=%d", aLayout, aTickWidth, aTimeType);

        Create();
    }
}
