/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include "chart/PriceChart.h"
#include "db/DBFactory.h"
#include "db/Price.h"
#include "db/Security.h"
#include "Resource.h"
#include "Progress.h"
#include <MHDayChart.h>
#include <MHDate.h>
#include <MHDC.h>
#include <MHDebug.h>
#include <MHPrice.h>



/**
*
*/
PriceChart::PriceChart (MHDC* dc, int view, MHDate::DAYS timeType) :
BaseChart (dc, view, timeType) {
    aChoice.aNames.Push (new MHString (STRING(CHARTINDICATOR_NO_CHART)));
    aChoice.aTypes.Push (new MHString ((int)NO_CHART));

    aChoice.aNames.Push (new MHString (STRING(CHARTINDICATOR_BAR)));
    aChoice.aTypes.Push (new MHString ((int)BAR));

    aChoice.aNames.Push (new MHString (STRING(CHARTINDICATOR_LINE)));
    aChoice.aTypes.Push (new MHString ((int)LINE));

    aChoice.aNames.Push (new MHString (STRING(CHARTINDICATOR_SMA)));
    aChoice.aTypes.Push (new MHString ((int)SMA));

    //aChoice.aNames.Push (new MHString ("SMA_SIGNALS"));
    //aChoice.aTypes.Push (new MHString ((int)SMA_SIGNALS));

    aChoice.aNames.Push (new MHString (STRING(CHARTINDICATOR_ESMA)));
    aChoice.aTypes.Push (new MHString ((int)ESMA));

    //aChoice.aNames.Push (new MHString ("ESMA_SIGNALS"));
    //aChoice.aTypes.Push (new MHString ((int)ESMA_SIGNALS));

    aChoice.aNames.Push (new MHString (STRING(CHARTINDICATOR_MACD)));
    aChoice.aTypes.Push (new MHString ((int)MACD));

    aChoice.aNames.Push (new MHString (STRING(CHARTINDICATOR_MACD_HISTOGRAMME)));
    aChoice.aTypes.Push (new MHString ((int)MACD_HISTOGRAMME));

    aChoice.aNames.Push (new MHString (STRING(CHARTINDICATOR_ATR)));
    aChoice.aTypes.Push (new MHString ((int)ATR));

    aChoice.aNames.Push (new MHString (STRING(CHARTINDICATOR_BOLLINGER)));
    aChoice.aTypes.Push (new MHString ((int)BOLLINGER));

    aChoice.aNames.Push (new MHString (STRING(CHARTINDICATOR_MOMENTUM)));
    aChoice.aTypes.Push (new MHString ((int)MOMENTUM));

    aChoice.aNames.Push (new MHString (STRING(CHARTINDICATOR_RELATIVE)));
    aChoice.aTypes.Push (new MHString ((int)RELATIVE));

    aChoice.aNames.Push (new MHString (STRING(CHARTINDICATOR_RSI)));
    aChoice.aTypes.Push (new MHString ((int)RSI));

    //aChoice.aNames.Push (new MHString ("RSI_SIGNALS"));
    //aChoice.aTypes.Push (new MHString ((int)RSI_SIGNALS));

    aChoice.aNames.Push (new MHString (STRING(CHARTINDICATOR_STDDEV)));
    aChoice.aTypes.Push (new MHString ((int)STDDEV));

    aChoice.aNames.Push (new MHString (STRING(CHARTINDICATOR_STOCHASTICS)));
    aChoice.aTypes.Push (new MHString ((int)STOCHASTICS));

    aChoice.aNames.Push (new MHString (STRING(CHARTINDICATOR_VOLUME)));
    aChoice.aTypes.Push (new MHString ((int)VOLUME));

    Reload ("PrefChartArea");
}



/**
*  Create new charts
*/
void PriceChart::Create () {
    DBG ("Create")
    DBG_LOG2 ("aLayout=%d  aTickWidth=%d", aLayout, aTickWidth);

    if (!aDC || !aData) return;
    createView();

    //  Draw as many chart areas and chart lines as user has selected
    for (int f = XTRADER_DEFAULT_WINDOW; f <= XTRADER_LAST_WINDOW; f++) {
        int align    = getAlign (f);
        int area     = getArea (f);
        int usedArea = getArea2 (aLayout);

        for (int e = 0; e <= 2; e++) {
            int chart = aWindows[f][e];

            if (area > usedArea || chart == NO_CHART) continue;

            switch (chart) {
                case ATR:
                    createAtr (align, area);
                    break;

                case BAR:
                    createBar (align, area);
                    break;

                case BOLLINGER:
                    createBollinger (align, area);
                    break;

                case ESMA:
                    createESMA (align, area);
                    break;

                case ESMA_SIGNALS:
                    createESMASignals (align, area);
                    break;

                case LINE:
                    createLine (align, area);
                    break;

                case MACD:
                    createMACD (align, area);
                    break;

                case MACD_HISTOGRAMME:
                    createMACDHistogramme (align, area);
                    break;

                case MOMENTUM:
                    createMomentum (align, area);
                    break;

                case RELATIVE:
                    createRelative (align, area);
                    break;

                case RSI:
                    createRSI (align, area);
                    break;

                case RSI_SIGNALS:
                    createRSISignals (align, area);
                    break;

                case SMA:
                    createSMA (align, area);
                    break;

                case SMA_SIGNALS:
                    createSMASignals (align, area);
                    break;


                case STOCHASTICS:
                    createStochastics (align, area);
                    break;

                case STDDEV:
                    createStdDev (align, area);
                    break;

                case VOLUME:
                    createVolume (align, area);
                    break;

                default:
                    break;
            }
        }
    }
}




/**
*  Create a atr chart for a security.
*/
void PriceChart::createAtr (int align, int area) {
    int          mvg = DBFactory::GetPrefIntValue (STRING(PREF_IND_ATR), aView, 2, 100, MHPrice::MVG_ATR);
    char         buff[100];
    MHVector*    v1   = 0;
    MHVector*    v2   = 0;
    MHVector*    ve1  = new MHVector ();
    MHVector*    ve2  = new MHVector ();
    MHVector*    ve3  = new MHVector ();
    Security*    sec  = (Security*) aData;

    v1 = DBFactory::LoadPrice (sec);
    v2 = DBFactory::LoadClosedPrice (sec);
    copy (v1, v2, ve1, ve2);

    MHPrice::ATR (ve1, ve3, mvg);

    snprintf (buff, 99, "%s %s (%d)", sec->GetName()->Get(), STRING(CHARTINDICATOR_ATR), mvg);
    aChart->AddData (buff, MHDayChart::LINE_CHART, align, area, ve3, ve2);
    aChart->AddDates (aTimeType);

    ve1->Erase ();
    delete ve1;
}



/**
*  Create a bar chart
*/
void PriceChart::createBar (int align, int area) {
    char         buff[100];
    MHVector*    v1   = 0;
    MHVector*    v2   = 0;
    MHVector*    ve1  = new MHVector ();
    MHVector*    ve2  = new MHVector ();
    Security*    sec  = (Security*) aData;

    v1 = DBFactory::LoadPrice (sec);
    v2 = DBFactory::LoadClosedPrice (sec);
    copy (v1, v2, ve1, ve2);

    snprintf (buff, 99, "%s %s", sec->GetName()->Get(), STRING(CHARTINDICATOR_BAR));
    aChart->AddData (buff, MHDayChart::BAR_CHART, align, area, ve1, ve2);
    aChart->AddDates (aTimeType);
}



/**
*  Create a bollinger chart for a security.
*/
void PriceChart::createBollinger (int align, int area) {
    char         buff[100];
    int          mvg  = DBFactory::GetPrefIntValue (STRING(PREF_IND_BOLLINGER), aView, 2, 100, MHPrice::MVG_BOLLINGER);
    MHVector*    v1   = 0;
    MHVector*    v2   = 0;
    MHVector*    ve1  = new MHVector ();
    MHVector*    ve2  = new MHVector ();
    MHVector*    ve3  = new MHVector ();
    MHVector*    ve4  = new MHVector ();
    MHVector*    ve5  = new MHVector ();
    Security*    sec  = (Security*) aData;

    v1 = DBFactory::LoadPrice (sec);
    v2 = DBFactory::LoadClosedPrice (sec);
    copy (v1, v2, ve1, ve2);
    MHPrice::Bollinger (v1, ve3, ve4, ve5, mvg);

    snprintf (buff, 99, "%s %s (%d)", sec->GetName()->Get(), STRING(CHARTINDICATOR_BOLLINGER), mvg);
    aChart->AddData (buff, MHDayChart::LINE_CHART, align, area, ve3, ve2);
    aChart->AddData (STRING(CHARTINDICATOR_BOLLINGER_UPPER_BAND), MHDayChart::LINE_CHART, align, area, ve4);
    aChart->AddData (STRING(CHARTINDICATOR_BOLLINGER_LOWER_BAND), MHDayChart::LINE_CHART, align, area, ve5);
    aChart->AddDates (aTimeType);

    ve1->Erase ();
    delete ve1;
}



/**
* Create two exponentiel moving average lines
*/
void PriceChart::createESMA (int align, int area) {
    char         buff[100];
    int          mvg1 = DBFactory::GetPrefIntValue (STRING(PREF_IND_ESMA_SHORT), aView, 2, 100, MHPrice::MVG_ESHORT);
    int          mvg2 = DBFactory::GetPrefIntValue (STRING(PREF_IND_ESMA_LONG), aView, 2, 100, MHPrice::MVG_ELONG);
    MHVector*    v1   = 0;
    MHVector*    v2   = 0;
    MHVector*    ve1  = new MHVector ();
    MHVector*    ve2  = new MHVector ();
    MHVector*    ve3  = new MHVector ();
    MHVector*    ve4  = new MHVector ();
    Security*    sec  = (Security*) aData;

    v1 = DBFactory::LoadPrice (sec);
    v2 = DBFactory::LoadClosedPrice (sec);
    copy (v1, v2, ve1, ve2);

    MHPrice::ExponentialMovingAverage (ve1, ve3, mvg1);
    MHPrice::ExponentialMovingAverage (ve1, ve4, mvg2);

    snprintf (buff, 99, "%s %s (%d)", sec->GetName()->Get(), STRING(CHARTINDICATOR_ESMA), mvg1);
    aChart->AddData (buff, MHDayChart::LINE_CHART, align, area, ve3, ve2);

    snprintf (buff, 99, "%s (%d)", STRING(CHARTINDICATOR_ESMA), mvg2);
    aChart->AddData (buff, MHDayChart::LINE_CHART, align, area, ve4);

    aChart->AddDates (aTimeType);
    ve1->Erase ();
    delete ve1;
}



/**
*  Create a line chart
*/
void PriceChart::createLine (int align, int area) {
    MHVector*    v1   = 0;
    MHVector*    v2   = 0;
    MHVector*    ve1  = new MHVector ();
    MHVector*    ve2  = new MHVector ();
    Security*    sec  = (Security*) aData;

    v1 = DBFactory::LoadPrice (sec);
    v2 = DBFactory::LoadClosedPrice (sec);
    copy (v1, v2, ve1, ve2);

    aChart->AddData (sec->GetName()->Get(), MHDayChart::LINE_CHART, align, area, ve1, ve2);
    aChart->AddDates (aTimeType);
}



/**
* Create MACD chart
*/
void PriceChart::createMACD (int align, int area) {
    char         buff[100];
    int          mvg1 = DBFactory::GetPrefIntValue (STRING(PREF_IND_MACD_SHORT), aView, 2, 100, MHPrice::MVG_MACD_SHORT);
    int          mvg2 = DBFactory::GetPrefIntValue (STRING(PREF_IND_MACD_LONG), aView, 2, 100, MHPrice::MVG_MACD_LONG);
    int          mvg3 = DBFactory::GetPrefIntValue (STRING(PREF_IND_MACD_ESMA), aView, 2, 100, MHPrice::MVG_MACD_ESMA);
    MHVector*    v1   = 0;
    MHVector*    v2   = 0;
    MHVector*    ve1  = new MHVector ();
    MHVector*    ve2  = new MHVector ();
    MHVector*    ve3  = new MHVector ();
    MHVector*    ve4  = new MHVector ();
    Security*    sec  = (Security*) aData;

    v1 = DBFactory::LoadPrice (sec);
    v2 = DBFactory::LoadClosedPrice (sec);
    copy (v1, v2, ve1, ve2);

    MHPrice::MACD (ve1, ve3, ve4, mvg1, mvg2, mvg3);

    snprintf (buff, 99, "%s %s (%d, %d)", sec->GetName()->Get(), STRING(CHARTINDICATOR_MACD), mvg1, mvg2);
    aChart->AddData (buff, MHDayChart::LINE_CHART, align, area, ve3, ve2);

    snprintf (buff, 99, "%s %s (%d)", STRING(CHARTINDICATOR_MACD), STRING(CHARTINDICATOR_ESMA), mvg3);
    aChart->AddData (buff, MHDayChart::LINE_CHART, align, area, ve4);

    aChart->AddDates (aTimeType);

    ve1->Erase ();
    delete ve1;
}



/**
* Create MACD chart
*/
void PriceChart::createMACDHistogramme (int align, int area) {
    char         buff[100];
    int          mvg1 = DBFactory::GetPrefIntValue (STRING(PREF_IND_MACD_SHORT), aView, 2, 100, MHPrice::MVG_MACD_SHORT);
    int          mvg2 = DBFactory::GetPrefIntValue (STRING(PREF_IND_MACD_LONG), aView, 2, 100, MHPrice::MVG_MACD_LONG);
    int          mvg3 = DBFactory::GetPrefIntValue (STRING(PREF_IND_MACD_ESMA), aView, 2, 100, MHPrice::MVG_MACD_ESMA);
    MHVector*    v1   = 0;
    MHVector*    v2   = 0;
    MHVector*    ve1  = new MHVector ();
    MHVector*    ve2  = new MHVector ();
    MHVector*    ve3  = new MHVector ();
    MHVector*    ve4  = new MHVector ();
    MHVector*    ve5  = new MHVector ();
    MHVector*    ve6  = new MHVector ();
    Security*    sec  = (Security*) aData;

    v1 = DBFactory::LoadPrice (sec);
    v2 = DBFactory::LoadClosedPrice (sec);
    copy (v1, v2, ve1, ve2);

    MHPrice::MACD (ve1, ve3, ve4, mvg1, mvg2, mvg3);
    MHPrice::Diff (ve3, ve4, ve5, ve6);

    snprintf (buff, 99, "%s %s", sec->GetName()->Get(), STRING(CHARTINDICATOR_MACD_HISTOGRAMME));
    aChart->AddData ("", MHDayChart::GREY_RECTANGLE_CHART, align, area, ve5, ve2);
    aChart->AddData ("", MHDayChart::LINE_CHART_HORIZONTAL_BLACK, align, area, ve6);
    aChart->AddDates (aTimeType);

    ve1->Erase ();
    ve3->Erase ();
    ve4->Erase ();
    delete ve1;
    delete ve3;
    delete ve4;
}



/**
*  Create a momentum chart for a security.
*/
void PriceChart::createMomentum (int align, int area) {
    int          mvg = DBFactory::GetPrefIntValue (STRING(PREF_IND_MOMENTUM), aView, 2, 100, MHPrice::MVG_MOMENTUM);
    char         buff[100];
    MHVector*    v1   = 0;
    MHVector*    v2   = 0;
    MHVector*    ve1  = new MHVector ();
    MHVector*    ve2  = new MHVector ();
    MHVector*    ve3  = new MHVector ();
    MHVector*    ve4  = new MHVector ();
    Security*    sec  = (Security*) aData;

    v1 = DBFactory::LoadPrice (sec);
    v2 = DBFactory::LoadClosedPrice (sec);
    copy (v1, v2, ve1, ve2);
    MHPrice::Momentum (ve1, ve3, ve4, mvg);

    snprintf (buff, 99, "%s %s (%d)", sec->GetName()->Get(), STRING(CHARTINDICATOR_MOMENTUM), mvg);
    aChart->AddData (buff, MHDayChart::LINE_CHART, align, area, ve3, ve2);
    aChart->AddData ("", MHDayChart::LINE_CHART_HORIZONTAL_BLACK, align, area, ve4);
    aChart->AddDates (aTimeType);

    ve1->Erase ();
    delete ve1;
}



/**
*  Create a relative chart where you compare two securities.
*/
void PriceChart::createRelative (int align, int area) {
    int          id   = DBFactory::GetPrefIntValue (STRING(PREF_CHART_RELATIVE), aView, 0, 999999999, 0);
    MHVector*    v1   = 0;
    MHVector*    v2   = 0;
    MHVector*    v3   = 0;
    MHVector*    ve1  = new MHVector ();
    MHVector*    ve2  = new MHVector ();
    MHVector*    ve3  = new MHVector ();
    MHVector*    ve4  = new MHVector ();
    Security*    sec  = (Security*) aData;
    Security     stmp(id, "", "");
    Security*    sec2 = (Security*) DBFactory::LoadSecurity()->Find (&stmp, MHVector::FIND_EXACT, Security::SORT_ID, 0);

    v1 = DBFactory::LoadPrice (sec);
    v2 = DBFactory::LoadClosedPrice (sec);
    copy (v1, v2, ve1, ve2);

    if (sec2) {
        v3 = DBFactory::LoadPrice (sec2);
        copy (v3, 0, ve3, 0);
        MHPrice::Cut (ve1, ve3);
        
        MHPrice* p1 = (MHPrice*) ve3->Get(0);
        if (p1) {
            MHPrice* p2 = (MHPrice*) ve1->Find (p1, MHVector::FIND_EXACT, Price::SORT_DATE, 0);
            if (p2)
                MHPrice::Normalise (ve3, ve4, p2->aClose);
        }
        if (ve4->Size() == 0) {
            p1 = (MHPrice*) ve1->Get(0);
            if (p1)
                MHPrice::Normalise (ve3, ve4, p1->aClose);
            else
                MHPrice::Normalise (ve3, ve4, 100);
        }
        aChart->AddData (sec2->GetName()->Get(), MHDayChart::LINE_CHART, align, area, ve4, ve2);
        aChart->AddDates (aTimeType);

        ve1->Erase ();
        delete ve1;
        ve3->Erase ();
        delete ve3;
    }
    else {
        ve1->Erase ();
        delete ve1;
        ve2->Erase ();
        delete ve2;
        ve3->Erase ();
        delete ve3;
        ve4->Erase ();
        delete ve4;
    }
}



/**
*  Create a chart using the rsi formula
*/
void PriceChart::createRSI (int align, int area) {
    char         buff[100];
    int          mvg  = DBFactory::GetPrefIntValue (STRING(PREF_IND_RSI), aView, 2, 100, MHPrice::MVG_RSI);
    MHVector*    v1   = 0;
    MHVector*    v2   = 0;
    MHVector*    ve1  = new MHVector ();
    MHVector*    ve2  = new MHVector ();
    MHVector*    ve3  = new MHVector ();
    Security*    sec  = (Security*) aData;

    v1 = DBFactory::LoadPrice (sec);
    v2 = DBFactory::LoadClosedPrice (sec);
    copy (v1, v2, ve1, ve2);
    MHPrice::RSI (ve1, ve3, mvg);

    snprintf (buff, 99, "%s %s (%d)", sec->GetName()->Get(), STRING(CHARTINDICATOR_RSI), mvg);
    aChart->AddData (buff, MHDayChart::LINE_CHART, align, area, ve3, ve2);
    aChart->AddDates (aTimeType);

    ve1->Erase ();
    delete ve1;
}



/**
* Create two moving average lines
*/
void PriceChart::createSMA (int align, int area) {
    char         buff[100];
    int          mvg1 = DBFactory::GetPrefIntValue (STRING(PREF_IND_SMA_SHORT), aView, 2, 100, MHPrice::MVG_SHORT);
    int          mvg2 = DBFactory::GetPrefIntValue (STRING(PREF_IND_SMA_LONG), aView, 2, 100, MHPrice::MVG_LONG);
    MHVector*    v1   = 0;
    MHVector*    v2   = 0;
    MHVector*    ve1  = new MHVector ();
    MHVector*    ve2  = new MHVector ();
    MHVector*    ve3  = new MHVector ();
    MHVector*    ve4  = new MHVector ();
    Security*    sec  = (Security*) aData;

    v1 = DBFactory::LoadPrice (sec);
    v2 = DBFactory::LoadClosedPrice (sec);
    copy (v1, v2, ve1, ve2);

    MHPrice::MovingAverage (ve1, ve3, mvg1);
    MHPrice::MovingAverage (ve1, ve4, mvg2);

    snprintf (buff, 99, "%s %s (%d)", sec->GetName()->Get(), STRING(CHARTINDICATOR_SMA), mvg1);
    aChart->AddData (buff, MHDayChart::LINE_CHART, align, area, ve3, ve2);

    snprintf (buff, 99, "%s (%d)", STRING(CHARTINDICATOR_SMA), mvg2);
    aChart->AddData (buff, MHDayChart::LINE_CHART, align, area, ve4);

    aChart->AddDates (aTimeType);
    ve1->Erase ();
    delete ve1;
}



/**
* Create signals for moving average
*/
void PriceChart::createSMASignals (int align, int area) {
    char         buff[100];
    int          mvg1 = DBFactory::GetPrefIntValue (STRING(PREF_IND_SMA_SHORT), aView, 2, 100, MHPrice::MVG_SHORT);
    int          mvg2 = DBFactory::GetPrefIntValue (STRING(PREF_IND_SMA_LONG), aView, 2, 100, MHPrice::MVG_LONG);
    MHVector*    v1   = 0;
    MHVector*    v2   = 0;
    MHVector*    ve1  = new MHVector ();
    MHVector*    ve2  = new MHVector ();
    MHVector*    ve3  = new MHVector ();
    MHVector*    ve4  = new MHVector ();
    MHVector*    ve5  = new MHVector ();
    Security*    sec  = (Security*) aData;

    v1 = DBFactory::LoadPrice (sec);
    v2 = DBFactory::LoadClosedPrice (sec);
    copy (v1, v2, ve1, ve2);

    MHPrice::MovingAverage (ve1, ve3, mvg1);
    MHPrice::MovingAverage (ve1, ve4, mvg2);
    MHPrice::MovingAverageSignals (ve3, ve4, ve5);

    snprintf (buff, 99, "%s", "SIGNALS");
    aChart->AddData (buff, MHDayChart::VERTICAL_CHART, align, area, ve5, ve2);
    aChart->AddDates (aTimeType);
    ve1->Erase ();
    delete ve1;
    ve3->Erase ();
    delete ve3;
    ve4->Erase ();
    delete ve4;
}



/**
*  Create a standard deviation chart for a security.
*/
void PriceChart::createStdDev (int align, int area) {
    char         buff[100];
    int          mvg  = DBFactory::GetPrefIntValue (STRING(PREF_IND_STDDEV), aView, 2, 100, MHPrice::MVG_STD);
    MHVector*    v1   = 0;
    MHVector*    v2   = 0;
    MHVector*    ve1  = new MHVector ();
    MHVector*    ve2  = new MHVector ();
    MHVector*    ve3  = new MHVector ();
    Security*    sec  = (Security*) aData;

    v1 = DBFactory::LoadPrice (sec);
    v2 = DBFactory::LoadClosedPrice (sec);
    copy (v1, v2, ve1, ve2);
    MHPrice::StdDev (ve1, ve3, mvg);

    snprintf (buff, 99, "%s %s (%d)", sec->GetName()->Get(), STRING(CHARTINDICATOR_STDDEV), mvg);
    aChart->AddData (buff, MHDayChart::LINE_CHART, align, area, ve3, ve2);
    aChart->AddDates (aTimeType);

    ve1->Erase ();
    delete ve1;
}



/**
*  Create a chart using the stochastics formula.
*/
void PriceChart::createStochastics (int align, int area) {
    char         buff[100];
    int          mvgk = DBFactory::GetPrefIntValue (STRING(PREF_IND_STOCHASTIC_K), aView, 2, 100, MHPrice::MVG_STOCHASTICS_K);
    int          mvgkk= DBFactory::GetPrefIntValue (STRING(PREF_IND_STOCHASTIC_SMA_K), aView, 1, 100, MHPrice::MVG_STOCHASTICS_SMA_K);
    int          mvgd = DBFactory::GetPrefIntValue (STRING(PREF_IND_STOCHASTIC_SMA_D), aView, 2, 100, MHPrice::MVG_STOCHASTICS_SMA_D);
    MHVector*    v1   = 0;
    MHVector*    v2   = 0;
    MHVector*    ve1  = new MHVector ();
    MHVector*    ve2  = new MHVector ();
    MHVector*    ve3  = new MHVector ();
    MHVector*    ve4  = new MHVector ();
    MHVector*    ve5  = new MHVector ();
    MHVector*    ve6  = new MHVector ();
    Security*    sec  = (Security*) aData;

    v1 = DBFactory::LoadPrice (sec);
    v2 = DBFactory::LoadClosedPrice (sec);
    copy (v1, v2, ve1, ve2);
    MHPrice::Stochastics (ve1, ve3, ve4, ve5, ve6, mvgk, mvgkk, mvgd);

    snprintf (buff, 99, "%s %s (%d,%d)", sec->GetName()->Get(), STRING(CHARTINDICATOR_STOCHASTICS), mvgk, mvgkk);
    aChart->AddData (buff, MHDayChart::LINE_CHART, align, area, ve3, ve2);
    snprintf (buff, 99, "%s %s (%d)", STRING(CHARTINDICATOR_STOCHASTICS), STRING(CHARTINDICATOR_SMA), mvgd);
    aChart->AddData (buff, MHDayChart::LINE_CHART, align, area, ve4);
    aChart->AddData ("", MHDayChart::LINE_CHART_HORIZONTAL_BLACK, align, area, ve5);
    aChart->AddData ("", MHDayChart::LINE_CHART_HORIZONTAL_BLACK, align, area, ve6);
    aChart->AddDates (aTimeType);

    ve1->Erase ();
    delete ve1;
}



/**
*  Create a chart with only volume data
*/
void PriceChart::createVolume (int align, int area) {
    char         buff[100];
    MHVector*    v1   = 0;
    MHVector*    v2   = 0;
    MHVector*    ve1  = new MHVector ();
    MHVector*    ve2  = new MHVector ();
    Security*    sec  = (Security*) aData;

    v1 = DBFactory::LoadPrice (sec);
    v2 = DBFactory::LoadClosedPrice (sec);
    copy (v1, v2, ve1, ve2);

    snprintf (buff, 99, "%s %s", sec->GetName()->Get(), STRING(CHARTINDICATOR_VOLUME));
    aChart->AddData (buff, MHDayChart::VOLUME_CHART, align, area, ve1, ve2);
    aChart->AddDates (aTimeType);
}



/**
*  Check is the message is for me and create chart again if so
*/
void PriceChart::Reload (const char* message) {
    DBG ("Reload")

    if (message == 0 || *message == '\0')
        Create();
    else if (strcmp (message, "Price") == 0)
        Create();
    else if (strcmp (message, "PrefChartArea") == 0) {
        //  Load all default settings from preference file
        aLayout    = (XTRADER_CHART_LAYOUT) DBFactory::GetPrefIntValue (STRING(PREF_CHART_LAYOUT), aView, XTRADER_DEFAULT_LAYOUT, XTRADER_LAST_LAYOUT, XTRADER_DEFAULT_LAYOUT);
        aTickWidth = DBFactory::GetPrefIntValue (STRING(PREF_CHART_LINEWIDTH), aView, XTRADER_MIN_LINE_WIDTH, XTRADER_MAX_LINE_WIDTH, XTRADER_DEFAULT_LINE_WIDTH);
        aBGColor   = DBFactory::GetPrefIntValue (STRING(PREF_CHART_BGCOLOR), aView, 0, 255, 0);
        aVSpace    = DBFactory::GetPrefIntValue (STRING(PREF_CHART_VSPACE), aView, XTRADER_MIN_VSPACE, XTRADER_MAX_VSPACE, XTRADER_DEFAULT_VSPACE);
        aBSpace    = DBFactory::GetPrefIntValue (STRING(PREF_CHART_BSPACE), aView, XTRADER_MIN_BSPACE, XTRADER_MAX_BSPACE, XTRADER_DEFAULT_BSPACE);

        for (int f = XTRADER_DEFAULT_WINDOW; f <= XTRADER_LAST_WINDOW; f++) {
            for (int e = 0; e <= 2; e++) {
                char b[100];
                snprintf (b, 99, "%s_%d_%d_%d", STRING(PREF_CHART_WINDOW), aView, f, e + 1);
                aWindows[f][e] = (XTRADER_CHART_WINDOW) DBFactory::GetPrefIntValue (b, NO_CHART, LAST_CHART, NO_CHART);
            }
        }

        DBG_LOG3 ("aLayout=%d  aTickWidth=%d  aTimeType=%d", aLayout, aTickWidth, aTimeType);

        Create();
    }
}
