/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <db/Account.h>
#include <db/DBFactory.h>
#include <Resource.h>
#include <MHCSVFile.h>



/**
*  Copy constructor
*  @param const Account& - Object to copy
*/
Account::Account (const Account& x) :
MH() {
    aID            = x.aID;
    aAccount       = x.aAccount;
    aBalance       = x.aBalance;
    aCurrency      = x.aCurrency;
}



/**
*  Create new account
*  @param int           - Unique id
*  @param const char*   - Name of account
*  @param double        - Start balance
*  @param const char*   - Currency
*/
Account::Account (int id, const char* account, double balance, const char* currency) :
MH () {
    SetID (id);
    SetAccount (account);
    SetBalance (balance);
    SetCurrency (currency);
}



/**
*  Operator =
*  @param const Account& - Object to copy
*/
Account& Account::operator=(const Account& x) {
    if (this == &x) return *this;

    IsChanged (((Account&)x).IsChanged());
    aID            = x.aID;
    aAccount       = x.aAccount;
    aBalance       = x.aBalance;
    aCurrency      = x.aCurrency;
    return *this;
}



/**
*  Export data to file
*/
void Account::Export (MHVector* ve, FILE* file) {
    Account*    i;
    char        buffer[200];

    for (int f = 0; f < ve->Size(); f++) {
        i = (Account*) (*ve)[f];
        snprintf (buffer, 199, "+a\t%s\t%f\t%s\n", i->aAccount(), i->aBalance, i->aCurrency());
        fwrite (buffer, sizeof(char), strlen (buffer), file);
    }
}



/**
*  Load a vector of Account objects.
*  @param MHVector& - Vector
*/
void Account::Load (MHVector* ve) {
    MHString    name = DBFactory::Database();

    name += "account.db";
    ve->Erase ();
    ve->SetSortType (SORT_NAME);

    try {
        MHCSVFile file (name(), '\t');

        while (file.ReadRow()) {
            if (file.Cols() != 4)
                throw "Wrong number of columns in the file";
            ve->Push (new Account (file.GetInt(0), file.GetString(1), file.GetDouble(2), file.GetString(3)));
        }
    }
    catch (const char* x) {
        ve->Erase ();
        if (MHFile::Exist (name()) == true)
            DBFactory::WriteLog ("Loading account failed file=%s, error=%s", name(), x);
    }
}



/**
*  Save a vector of Account objects.
*  Throws Exception at failure
*  @param MHVector& - Vector
*/
void Account::Save (MHVector* ve) {
    Account*   i;
    MHString   name = DBFactory::Database();

    ve->Sort(SORT_NAME);
    name += "account.db";

    try {
        MHCSVFile file;
        char      buff[400];

        for (int f = 0; f < ve->Size(); f++) {
            i = (Account*) (*ve)[f];
            sprintf (buff, "%d\t%s\t%.2f\t%s", i->GetID(), i->GetAccount()->Get(), i->GetBalance(), i->GetCurrency()->Get());
            file.WriteString (buff);
        }
        file.WriteFile (name());
        MH::ClearChanged (ve);
    }
    catch (const char* x) {
        DBFactory::WriteLog ("Saving accounts failed file=%s, error=%s", name(), x);
        throw STRING(ERROR_SAVE);
    }
}
