/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#ifndef Account_h
#define Account_h

#include <MHString.h>
#include <MHVector.h>



/**
*  Account stores all names of security lists.
*/
class Account : public MH {
public:
    enum {
                            ACCOUNT_LEN         = 20,
                            CURRENCY_LEN        = 4,
    };

    enum SORT {
                            SORT_NAME,
                            SORT_ID,
    };

                            Account (){aBalance = 0;}
                            Account (const Account& x);
                            Account (const char* account) {SetAccount(account); }
                            Account (int id, const char* account, double balance = 0, const char* currency = "");

    Account&                operator=(const Account&);
    bool                    operator==(const Account& x) {return aAccount == x.aAccount;}
    bool                    operator!=(const Account& x) {return aAccount != x.aAccount;}
    bool                    operator>(const Account& x) {return aAccount > x.aAccount;}
    bool                    operator<(const Account& x) {return aAccount < x.aAccount;}

    int                     Compare (const MH* o, int type){if (type == SORT_ID) return compareID ((Account*)o); else return compareName ((Account*)o);}
    static void             Export (MHVector* ve, FILE* file);
    MHString*               GetAccount () {return &aAccount;}
    double                  GetBalance () {return aBalance;}
    MHString*               GetCurrency () {return &aCurrency;}
    int                     GetID () {return aID;}
    static void             Load (MHVector* ve);
    static void             Save (MHVector* ve);
    void                    SetAccount (const char* s) {aAccount = CHECK_STRING(s); aAccount.Shrink(ACCOUNT_LEN);}
    void                    SetBalance (double balance) {aBalance = balance;}
    void                    SetCurrency (const char* s) {aCurrency = CHECK_STRING(s); aCurrency.Shrink(CURRENCY_LEN);}
    void                    SetID (int i) {aID = i;}

private:
    int                     aID;
    MHString                aAccount;
    double                  aBalance;
    MHString                aCurrency;
    MHString                aName;

    int                     compareName (Account* o){return strcasecmp (aAccount(), (*o).aAccount());}
    int                     compareID (Account* o){if (aID < o->aID) return -1; else if (aID > o->aID) return 1; else return 0;}
};

#endif
