/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#ifndef Guard_h
#define Guard_h

#include <MHString.h>
#include <MHVector.h>
#include <db/DBFactory.h>
#include <db/Security.h>



/**
*  Guard is the price guard object.
*/
class Guard : public MH {
public:
    enum {
                            DATE_LEN  = 8,
                            TYPE_LEN  = 1,
    };

    enum TYPE {
                            BUY_HIGH = 0,
                            BUY_LOW,
                            SELL_HIGH,
                            SELL_LOW,
                            NO_TYPES,
                            BUY_MVG,
                            SELL_MVG,
    };

                            Guard ();
                            Guard (const Guard&);
                            Guard (int sid, TYPE type, double value);

    Guard&                  operator=(const Guard&);

    int                     Compare (const MH* o, int type);
    static void             Export (MHVector*, FILE* file);
    MHString*               GetCurrDate () {return &aCurrDate;}
    double                  GetCurrValue () {return aCurrValue;}
    Security*               GetSecurity () {return &aSecurity;}
    int                     GetSecurityID () {return aSecurityID;}
    TYPE                    GetType () {return aType;}
    const char*             GetTypeName ();
    double                  GetValue () {return aValue;}
    static void             Load (MHVector* ve);
    Security*               LoadSecurity();
    static void             LoadSecurities(MHVector* v);
    static void             Save (MHVector* ve);
    void                    SetCurrDate (const char* s) {aCurrDate = CHECK_STRING(s); aCurrDate.Shrink(DATE_LEN);}
    void                    SetCurrValue (double d) {aCurrValue = d;}
    void                    SetSecurity (Security* s) {aSecurityID = s->GetID(); aSecurity = *s;}
    void                    SetSecurityID (int i) {aSecurityID = i;}
    void                    SetType (TYPE c) {aType = c;}
    void                    SetValue (double d) {aValue = d;}

private:
    int                     aSecurityID;
    TYPE                    aType;
    double                  aValue;

    Security                aSecurity;
    MHString                aCurrDate;
    double                  aCurrValue;
};

#endif
