/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#ifndef Portfolio_h
#define Portfolio_h

#include <db/DBFactory.h>
#include <db/Security.h>
#include <MHString.h>
#include <MHVector.h>

class Account;



/**
*  Portfolio stores all investment transactions
*/
class Portfolio : public MH {
public:
    enum {
                            DATE_LEN     = 8,
                            MEMO_LEN     = 50,
    };

    enum EVENT {
                            EV_IN        = 0,
                            EV_OUT,
                            EV_INTEREST,
                            EV_LOAN,
                            EV_FEE,
                            EV_TAX,
                            EV_BUY,
                            EV_SELL,
                            EV_DIVIDEND,
                            EV_TRANSFER,
                            EV_USE_SECURITY = EV_TAX,
    };

                            Portfolio (){aSecurityID = aAccountID = aTransferID = 0; aCourtage = aPrice = aVolume = 0.0; aEvent = EV_IN; }
                            Portfolio (const Portfolio& x);
                            Portfolio (const char* date, EVENT event, int sid, double volume, double price, double courtage, int aid, int tid, const char* memo);
                            Portfolio (const char* date, Account* account, double price, const char* memo){;}

    Portfolio&              operator=(const Portfolio&);

    int                     Compare (const MH* o, int type){char a[22]; char b[22];sprintf (a, "%s%d", aDate(), aEvent);sprintf (b, "%s%d", ((Portfolio*)o)->GetDate()->Get(), ((Portfolio*)o)->GetEvent()); return strcasecmp (a, b);}
    static void             Export (Account* account, MHVector* ve, FILE* file);
    int                     GetAccountID () {return aAccountID;}
    int                     GetOldAccountID () {return aOldAccountID;}
    double                  GetCourtage () {return aCourtage;}
    MHString*               GetDate () {return &aDate;}
    EVENT                   GetEvent () {return aEvent;}
    static const char*      GetEventName (int event);
    MHString*               GetMemo () {return &aMemo;}
    double                  GetPrice () {return aPrice;}
    Security*               GetSecurity () {return &aSecurity;}
    int                     GetSecurityID () {return aSecurityID;}
    int                     GetTransferID () {return aTransferID;}
    int                     GetOldTransferID () {return aTransferID;}
    double                  GetVolume () {return aVolume;}
    static void             Load (Account* account, MHVector* ve);
    Security*               LoadSecurity();
    static void             LoadSecurities(MHVector* v);
    static void             Save (Account* account, MHVector* ve);
    void                    SetAccountID (int accountID) {if (!aOldAccountID) aOldAccountID = aAccountID; aAccountID = accountID;}
    void                    SetCourtage (double courtage) {aCourtage = courtage;}
    void                    SetDate (const char* s) {aDate = CHECK_STRING(s); aDate.Shrink(DATE_LEN);}
    void                    SetEvent (EVENT event) {aEvent = event;}
    void                    SetMemo (const char* s) {aMemo = CHECK_STRING(s); aMemo.Shrink(MEMO_LEN);}
    void                    SetPrice (double price) {aPrice = price;}
    void                    SetSecurityID (int i) {aSecurityID = i;}
    void                    SetTransferID (int transferID) {if (!aOldTransferID) aOldTransferID = aTransferID; aTransferID = transferID;}
    void                    SetVolume (double volume) {aVolume = volume;}

protected:
    MHString                aDate;
    EVENT                   aEvent;
    int                     aSecurityID;
    double                  aVolume;
    double                  aPrice;
    double                  aCourtage;
    int                     aAccountID;
    int                     aTransferID;
    int                     aOldAccountID;
    int                     aOldTransferID;
    MHString                aMemo;
    Security                aSecurity;
};

#endif
