/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#ifndef Portfolio2_h
#define Portfolio2_h

#include "db/Portfolio.h"
#include <MHString.h>
#include <MHVector.h>

class PortfolioValue;



/**
*  Extends Portfolio.
*  Add methods to get buy and sell prices and to set and get prices.
*/
class Portfolio2 : public Portfolio {
public:
    enum {
                            NAME_LEN     = 30,
                            VALUE        = 1,
                            BUY_VALUE,
                            BUY_VALUE2,
                            SELL_VALUE,
                            DIV_VALUE,
    };

                            Portfolio2 () {aCurrPrice = 0.0; aDivVolume = 0.0; aDivPrice = 0.0;}
                            Portfolio2 (const Portfolio& x) : Portfolio(x) {aCurrPrice = aDivPrice = aDivVolume = 0.0;}
                            Portfolio2 (const Portfolio2&);

    Portfolio2&             operator= (const Portfolio2&);

    void                    AddNewPrice (Portfolio2* r2);
    static void             AddNewTrade (const char* startdate, Portfolio2* r2, PortfolioValue* value, MHVector* in);
    double                  BuyValue() {return value (BUY_VALUE);}
    double                  BuyValue2() {return value (BUY_VALUE2);}
    double                  CashValue();
    int                     Compare (const MH* o){char a[40]; char b[40]; sprintf (a, "%-20s%s", aName(), aDate()); sprintf (b, "%-20s%s", ((Portfolio2*)o)->GetName()->Get(), ((Portfolio2*)o)->GetDate()->Get()); return strcasecmp (a, b);}
    double                  DivValue () {return value (DIV_VALUE);}
    double                  GetCurrPrice () {return aCurrPrice;}
    MHString*               GetCurrPriceDate () {return &aCurrPriceDate;}
    double                  GetDivPrice () {return aDivPrice;}
    double                  GetDivVolume () {return aDivVolume;}
    MHString*               GetName () {return &aName;}
    static void             GetBuySell (MHVector* in, MHString* stopDate, MHVector* out);
    static void             GroupTrades (MHVector* in, MHVector* out);
    bool                    LoadBuyPrice (const char* date) {return loadPrice (date, false);}
    static void             LoadBuyPrices (const char* date, MHVector* in) {loadPrices(date, in, false);}
    bool                    LoadCurrPrice (const char* date) {return loadPrice(date, true);}
    static void             LoadCurrPrices (const char* date, MHVector* in) {loadPrices(date, in, true);}
    static void             LoadPortfolio (Account* account, MHVector* out);
    static bool             Performance (MHVector* trades, MHVector* buyList, MHVector* chartp, MHVector* chartv, MHString* start, MHString* stop, PortfolioValue* value);
    static void             Reset (MHVector* trades);
    double                  SellValue() {return value (SELL_VALUE);}
    void                    SetCurrPrice (double d) {aCurrPrice = d;}
    void                    SetCurrPriceDate (const char* s) {aCurrPriceDate = CHECK_STRING(s); aCurrPriceDate.Shrink(DATE_LEN);}
    void                    SetDivPrice (double d) {aDivPrice = d;}
    void                    SetDivVolume (double d) {aDivVolume = d;}
    double                  Value () {return value (VALUE);}

private:
    double                  aCurrPrice;
    MHString                aCurrPriceDate;
    double                  aDivVolume;
    double                  aDivPrice;
    MHString                aName;

    bool                    loadPrice (const char* date, bool current);
    static void             loadPrices (const char* date, MHVector* in, bool current);
    double                  value(int type);
};

#endif
