/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <db/Pref.h>
#include <db/DBFactory.h>
#include <Resource.h>
#include <MHCSVFile.h>



/**
*  Create new object.
*  @param const Pref& - Object to copy
*/
Pref::Pref(const Pref& x) :
MH() {
    IsChanged (((Pref&)x).IsChanged());
    aName  = x.aName;
    aValue = x.aValue;
}



/**
*  Operator =
*  @param const Pref& - Object to copy
*/
Pref& Pref::operator=(const Pref& x) {
    if (this == &x) return *this;

    IsChanged (((Pref&)x).IsChanged());
    aName  = x.aName;
    aValue = x.aValue;

    return *this;
}



/**
*  Export data to file
*/
void Pref::Export (MHVector* ve, FILE* file) {
    Pref*  i;
    char   buffer[200];

    for (int f = 0; f < ve->Size(); f++) {
        i = (Pref*) (*ve)[f];
        snprintf (buffer, 199, "+i\t%s\t%s\n", i->aName(), i->aValue());
        fwrite (buffer, sizeof(char), strlen (buffer), file);
    }
}



/**
*  Load a vector of Pref objects.
*  @param MHVector& - Vector
*/
void Pref::Load (MHVector* ve) {
    MHString    name = DBFactory::Database();

    name += DBFactory::GetWorkSpaceName();
    name += ".ws";
    ve->Erase ();

    try {
        MHCSVFile file (name(), '\t');

        while (file.ReadRow()) {
            if (file.Cols() != 2)
                throw "Wrong number of columns in the file";
            ve->Push (new Pref (file.GetString(0), file.GetString(1)));
        }
    }
    catch (const char* x) {
        ve->Erase ();
        if (MHFile::Exist (name()) == true)
            DBFactory::WriteLog ("Loading preference failed file=%s, error=%s", name(), x);
    }
}



/**
*  Save a vector of Pref objects.
*  Throws Exception at failure
*  @param MHVector& - Vector
*/
void Pref::Save (MHVector* ve) {
    Pref*       i;
    MHString    name = DBFactory::Database();

    name += DBFactory::GetWorkSpaceName();
    name += ".ws";
    ve->Sort();

    try {
        MHCSVFile file;
        char      buff[400];

        for (int f = 0; f < ve->Size(); f++) {
            i = (Pref*) (*ve)[f];
            sprintf (buff, "%s\t%s", i->aName(), i->aValue());
            file.WriteString (buff);
        }
        file.WriteFile (name());
        MH::ClearChanged (ve);
    }
    catch (const char* x) {
        DBFactory::WriteLog ("Saving preference failed file=%s, error=%s", name(), x);
        throw STRING(ERROR_SAVE);
    }
}
