/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#ifndef Pref_h
#define Pref_h

#include <MHString.h>
#include <MHVector.h>



/**
*  Pref is the parameter and ini settings object.
*/
class Pref : public MH {
public:
    enum {
                            NAME_LEN  = 20,
    };

                            Pref () : MH() {;}
                            Pref (const Pref&);
                            Pref (const char* pName, const char* pValue = "") : MH () {SetName(pName); SetValue(pValue); }
                            Pref (const char* pName, double value) : MH() {SetName(pName); SetValue(value); }
                            Pref (const char* pName, int value) : MH() {SetName(pName); SetValue(value); }
                            ~Pref () {;}

    Pref&                   operator=(const Pref&);
    bool                    operator==(const Pref& x) {return aName == x.aName;}
    bool                    operator!=(const Pref& x) {return aName != x.aName;}
    bool                    operator>(const Pref& x) {return aName > x.aName;}
    bool                    operator<(const Pref& x) {return aName < x.aName;}

    int                     Compare (const MH* o, int type){return strcasecmp (aName(), ((Pref*)o)->GetName()->Get());}
    static void             Export (MHVector* ve, FILE* file);
    static void             Load (MHVector* ve);
    MHString*               GetName() {return &aName;}
    MHString*               GetValue() {return &aValue;}
    static void             Save (MHVector* ve);
    void                    SetName(const char* s) {aName = CHECK_STRING(s); aName.Shrink(NAME_LEN);}
    void                    SetValue(const char* s) {aValue = CHECK_STRING(s);}
    void                    SetValue(double d) {aValue = d;}
    void                    SetValue(int i) {aValue = i;}

private:
    MHString                aName;
    MHString                aValue;
};

#endif
