/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <db/Security.h>
#include <db/DBFactory.h>
#include <Resource.h>
#include <MHCSVFile.h>



/**
*  Create a new security object
*  @param const char*    - Code of object
*  @param const char*    - Name of object
*/
Security::Security () :
MH() {
    aID       = 0;
    aNoShares = 0;
}




/**
*  Create a new security object
*  @param const Security& - Object to copy
*/
Security::Security(const Security& x) :
MH() {
    aID       = x.aID;
    aCode     = x.aCode;
    aName     = x.aName;
    aMarket   = x.aMarket;
    aSector   = x.aSector;
    aNoShares = x.aNoShares;
}



/**
*  Create a new security object
*  @param int            - Id number
*  @param const char*    - Code of object
*  @param const char*    - Name of object
*  @param const char*    - Market for this security
*  @param const char*    - Sector for this security
*  @param double         - Number of shares
*/
Security::Security (int id, const char* pCode, const char* pName, const char* pMarket, const char* pSector, double nShares) :
MH() {
    SetID (id);
    SetCode (pCode);
    SetName (pName);
    SetMarket (pMarket);
    SetSector (pSector);
    SetNoShares (nShares);
}




/**
*  Operator =
*  @param const Security& - Object to copy
*/
Security& Security::operator=(const Security& x) {
    if (this == &x) return *this;

    IsChanged (((Security&)x).IsChanged());
    aID       = x.aID;
    aCode     = x.aCode;
    aName     = x.aName;
    aMarket   = x.aMarket;
    aSector   = x.aSector;
    aNoShares = x.aNoShares;

    return *this;
}



/**
*  Export data to file
*/
void Security::Export (MHVector* ve, FILE* file) {
    Security*   i;
    char        buffer[200];

    for (int f = 0; f < ve->Size(); f++) {
        i = (Security*) (*ve)[f];
        snprintf (buffer, 199, "+s\t%s\t%s\t%s\t%s\t%f\n", i->aCode(), i->aName(), i->aMarket(), i->aSector(), i->aNoShares);
        fwrite (buffer, sizeof(char), strlen (buffer), file);
    }
}



/**
*  Load a vector of Security objects.
*  @param MHVector& - Vector
*/
void Security::Load (MHVector* ve) {
    MHString    name = DBFactory::Database();

    name += "security.db";
    ve->Erase ();
    ve->SetSortType (SORT_ID);

    try {
        MHCSVFile file (name(), '\t');

        while (file.ReadRow()) {
            if (file.Cols() != 6)
                throw "Wrong number of columns in the file";
            ve->Push (new Security (file.GetInt(0), file.GetString(1), file.GetString(2), file.GetString(3), file.GetString(4), file.GetDouble(5)));
        }
    }
    catch (const char* x) {
        ve->Erase ();
        if (MHFile::Exist (name()) == true)
            DBFactory::WriteLog ("Loading security failed file=%s, error=%s", name(), x);
    }
}



/**
*  Save a vector of Security objects.
*  Throws Exception at failure
*  @param MHVector& - Vector
*/
void Security::Save (MHVector* ve) {
    Security*   i;
    MHString    name = DBFactory::Database();

    name += "security.db";
    ve->Sort(SORT_ID);

    try {
        MHCSVFile file;
        char      buff[400];

        for (int f = 0; f < ve->Size(); f++) {
            i = (Security*) (*ve)[f];
            sprintf (buff, "%d\t%s\t%s\t%s\t%s\t%.2f", i->GetID(), i->GetCode()->Get(), i->GetName()->Get(), i->GetMarket()->Get(), i->GetSector()->Get(), i->GetNoShares());
            file.WriteString (buff);
        }
        file.WriteFile (name());
        MH::ClearChanged (ve);
    }
    catch (const char* x) {
        DBFactory::WriteLog ("Saving security failed file=%s, error=%s", name(), x);
        throw STRING(ERROR_SAVE);
    }
}
