/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#ifndef Security_h
#define Security_h

#include <MHString.h>
#include <MHVector.h>



/**
*  Security stores security information
*/
class Security : public MH {
public:
    enum {
                            SORT_NAME     = 1,
                            SORT_CODE,
                            SORT_ID,
                            CODE_LEN      = 20,
                            NAME_LEN      = 30,
                            MARKET_LEN    = 20,
                            SECTOR_LEN    = 20,
    };

                            Security ();
                            Security (const Security&);
                            Security (int id, const char* pCode = 0, const char* pName = 0, const char* pMarket = 0, const char* pSector = 0, double nShares = 0);

    Security&               operator=(const Security&);
    bool                    operator==(const Security& x) {return aID == x.aID;}
    bool                    operator!=(const Security& x) {return aID != x.aID;}
    bool                    operator>(const Security& x) {return aID > x.aID;}
    bool                    operator<(const Security& x) {return aID < x.aID;}

    int                     Compare (const MH* o, int type) {if (type == SORT_NAME) return compareName((Security*)o); else if (type == SORT_ID) return compareID ((Security*)o); else return compareCode((Security*)o);}
    static void             Export (MHVector* ve, FILE* file);
    MHString*               GetCode () {return &aCode;}
    int                     GetID () {return aID;}
    MHString*               GetMarket () {return &aMarket;}
    MHString*               GetName () {return &aName;}
    double                  GetNoShares () {return aNoShares;}
    MHString*               GetSector () {return &aSector;}
    static void             Load (MHVector* ve);
    static void             Save (MHVector* ve);
    void                    SetCode (const char* s) {aCode = CHECK_STRING(s); aCode.Shrink (CODE_LEN);}
    void                    SetID (int i) {aID = i;}
    void                    SetMarket (const char* s) {aMarket = CHECK_STRING(s); aMarket.Shrink (MARKET_LEN);}
    void                    SetName (const char* s) {aName = CHECK_STRING(s); aName.Shrink (NAME_LEN);}
    void                    SetNoShares (double d) {aNoShares = d;}
    void                    SetSector (const char* s) {aSector = CHECK_STRING(s); aSector.Shrink (SECTOR_LEN);}
    static bool             Update (Security* security, MHVector* ve);

private:
    int                     aID;
    MHString                aCode;
    MHString                aName;
    MHString                aMarket;
    MHString                aSector;
    double                  aNoShares;

    int                     compareCode (Security* o) {return strcasecmp (aCode(), o->GetCode()->Get());}
    int                     compareID (Security* o)   {if (aID < o->aID) return -1; else if (aID > o->aID) return 1; else return 0;}
    int                     compareName (Security* o) {return strcasecmp (aName(), o->GetName()->Get());}
};

#endif

