/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <db/Set.h>
#include <db/DBFactory.h>
#include <Resource.h>
#include <MHCSVFile.h>



/**
*  Operator =
*  @param const Set& - Object to copy
*/
Set& Set::operator=(const Set& x) {
    if (this == &x) return *this;

    IsChanged (((Set&)x).IsChanged());
    aID  = x.aID;
    aSet = x.aSet;

    return *this;
}



/**
*  Export data to file
*/
void Set::Export (MHVector* ve, FILE* file) {
    Set*  i;
    char  buffer[200];

    for (int f = 0; f < ve->Size(); f++) {
        i = (Set*) (*ve)[f];
        snprintf (buffer, 199, "+e\t%s\n", i->aSet());
        fwrite (buffer, sizeof(char), strlen (buffer), file);
    }
}



/**
*  Load a vector of Set objects.
*  @param MHVector& - Vector
*/
void Set::Load (MHVector* ve) {
    MHString    name = DBFactory::Database();

    name += "set.db";
    ve->Erase ();

    try {
        MHCSVFile file (name(), '\t');

        while (file.ReadRow()) {
            if (file.Cols() != 2)
                throw "Wrong number of columns in the file";
            ve->Push (new Set (file.GetInt(0), file.GetString(1)));
        }
    }
    catch (const char* x) {
        ve->Erase ();
        if (MHFile::Exist (name()) == true)
            DBFactory::WriteLog ("Loading set failed file=%s, error=%s", name(), x);
    }
}



/**
*  Save a vector of Set objects.
*  Throws Exception at failure
*  @param MHVector& - Vector
*/
void Set::Save (MHVector* ve) {
    Set*        i;
    MHString    name = DBFactory::Database();

    name += "set.db";
    ve->Sort();

    try {
        MHCSVFile file;
        char      buff[400];

        for (int f = 0; f < ve->Size(); f++) {
            i = (Set*) (*ve)[f];
            sprintf (buff, "%d\t%s", i->GetID(), i->GetSet()->Get());
            file.WriteString (buff);
        }
        file.WriteFile (name());
        MH::ClearChanged (ve);
    }
    catch (const char* x) {
        DBFactory::WriteLog ("Saving set failed file=%s, error=%s", name(), x);
        throw STRING(ERROR_SAVE);
    }
}
