/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <db/SetItem.h>
#include <db/Set.h>
#include <Resource.h>
#include <MHCSVFile.h>



/**
*  Operator =
*  @param const SetItem& - Object to copy
*/
SetItem& SetItem::operator=(const SetItem& x) {
    if (this == &x) return *this;

    IsChanged (((SetItem&)x).IsChanged());
    aSecurityID = x.aSecurityID;
    aSecurity   = x.aSecurity;
    return *this;
}



/**
*
*/
int SetItem::compareID (SetItem* o) {
    if (aSecurityID < ((SetItem*)o)->aSecurityID)
        return -1;
    else if (aSecurityID > ((SetItem*)o)->aSecurityID)
        return 1;
    else
        return 0;
}



/**
*
*/
int SetItem::compareSecurity (SetItem* o) {
    if (aSecurity.GetID() == 0)
        LoadSecurity();

    if (o->GetSecurity()->GetID() == 0)
        o->LoadSecurity();

    return aSecurity.Compare (o->GetSecurity(), Security::SORT_NAME);
}



/**
*  Export data to file
*/
void SetItem::Export (Set* set, MHVector* ve, FILE* file) {
    SetItem*    i;
    Security*   s;
    char        buffer[200];

    for (int f = 0; f < ve->Size(); f++) {
        i = (SetItem*) (*ve)[f];
        s = i->LoadSecurity();
        if (s && s->GetCode()->Size() > 0) {
            snprintf (buffer, 199, "+t\t%s\t%s\n", set->GetSet()->Get(), s->GetCode()->Get());
            fwrite (buffer, sizeof(char), strlen (buffer), file);
        }
    }
}



/**
*  Load a vector of SetItem objects.
*  @param const char* - Set name
*  @param MHVector&   - Vector
*/
void SetItem::Load (Set* set, MHVector* ve) {
    MHString    name = DBFactory::Database();

    name += set->GetID();
    name += ".t";
    ve->Erase ();
    ve->SetSortType (SORT_ID);

    try {
        MHCSVFile file (name(), '\t');

        while (file.ReadRow()) {
            if (file.Cols() != 1)
                throw "Wrong number of columns in the file";
            SetItem* tmp = new SetItem (file.GetInt(0));
        if (tmp->LoadSecurity()->GetID() > 0)
                ve->Push (tmp);
            else
                delete tmp;
        }
    }
    catch (const char* x) {
        ve->Erase ();
        if (MHFile::Exist (name()) == true)
            DBFactory::WriteLog ("Loading setitems failed file=%s, error=%s", name(), x);
    }
}



/**
*  Load security for this set item
*  @return Security* - Security object, if not found the id is 0 for this security
*/
Security* SetItem::LoadSecurity () {
    aSecurity.SetID(0);
    aSecurity.SetName("");

    MHVector* v = DBFactory::LoadSecurity();
    Security  s1(aSecurityID);
    Security* s2 = (Security*) v->Find (&s1, MHVector::FIND_EXACT, Security::SORT_ID, 0);
    if (s2)
        aSecurity = *s2;
    return &aSecurity;
}



/**
*  Load security for this set item
*  @return Security* - Security object, if not found the id is 0 for this security
*/
void SetItem::LoadSecurities (MHVector* v) {
    int f;

    for (f = 0; f < v->Size(); f++)
        ((SetItem*)(*v)[f])->LoadSecurity();

    for (f = 0; f < v->Size(); f++) {
        SetItem*  i = (SetItem*) (*v)[f];
        Security* s = i->LoadSecurity ();

        if (s->GetID() == 0)
            v->Erase(f);
    }
}


/**
*  Remove setitem file from disk
*  @param Set* - The set which setitems should be removed from
*/
void SetItem::Remove (Set* set) {
    MHString   name = DBFactory::Database();

    name += set->GetID();
    name += ".t";
    remove (name());
}



/**
*  Save a vector of SetItem objects.
*  Throws Exception at failure
*  @param Set*    - Set name
*  @param MHVector* - Vector
*/
void SetItem::Save (Set* set, MHVector* ve) {
    SetItem*   i;
    MHString   name = DBFactory::Database();

    name += set->GetID();
    name += ".t";
    ve->Sort (SORT_ID);

    try {
        MHCSVFile file;
        char      buff[400];

        for (int f = 0; f < ve->Size(); f++) {
            i = (SetItem*) (*ve)[f];
            sprintf (buff, "%d", i->GetSecurityID());
            file.WriteString (buff);
        }
        file.WriteFile (name());
        MH::ClearChanged (ve);
    }
    catch (const char* x) {
        DBFactory::WriteLog ("Saving setitems failed file=%s, error=%s", name(), x);
        throw STRING(ERROR_SAVE);
    }
}
