/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <Resource.h>
#include <gui/XTrader.h>
#include <dlg/DateDlg.h>
#include <MHDate.h>
#include <FL/fl_ask.h>



/**
*  Create date dialog and get date from user.
*  @param const char* - Startdate
*  @param const char* - Stopdate
*/
DateDlg::DateDlg (const char* startDate, const char* stopDate) :
Fl_Dialog (330, 110, "SELECT DATE") {
    Fl_Window* o = this;

    o->box(FL_UP_BOX);
    { Fl_Input* o = aStart = new Fl_Input(10, 25, 200, 25, "STARTDATE");
      o->type(2);
      o->labelfont(1);
      o->align(FL_ALIGN_TOP_LEFT);
    }
    { Fl_Input* o = aStop = new Fl_Input(10, 75, 200, 25, "STOPDATE");
      o->type(2);
      o->labelfont(1);
      o->align(FL_ALIGN_TOP_LEFT);
    }
    aOk = new Fl_Return_Button(220, 25, 100, 25, "OK");
    aCancel = new Fl_Button(220, 60, 100, 25, "CANCEL");
    o->end();

    aCancel->callback (Fl_Dialog::cb1_, this);
    aOk->callback (Fl_Dialog::cb2_, this);

    label (STRING(DLG_DATE));
    aStart->label (STRING(START_DATE));
    aStop->label (STRING(STOP_DATE));
    aCancel->label (STRING(CANCEL));
    aOk->label (STRING(OK));

    if (startDate)
        aStart->value (startDate);
    else
        aStart->deactivate ();

    if (stopDate)
        aStop->value (stopDate);
    else
        aStop->deactivate ();

    XTrader::FixButton (aOk);
    XTrader::FixButton (aCancel);
}



/**
*  Quit dialog with ok button, check input date format.
*  Must be valid to exit dialog box
*/
void DateDlg::cb2 () {
    if (checkDate (aStart) == true && checkDate (aStop) == true || checkDate2 (aStart, aStop) == true)
        aRetVal = 1;
}



/**
*  Check date if valid
*  If input widget is disbled no test is done but return value is ok
*  @return true - If date is ok
*/
bool DateDlg::checkDate (Fl_Input* input) {
    if (input->active() && MHDate::ValidDate (input->value()) == false) {
        label (STRING(ERROR_DATE));
        Fl::focus (input);
        fl_beep (FL_BEEP_ERROR);
        redraw ();
        return false;
    }

    return true;
}



/**
*  Check that start date is less than stop date
*  @return true - If date is ok
*/
bool DateDlg::checkDate2 (Fl_Input* input, Fl_Input* input2) {
    if (input->active() && input->active() && strcmp (input->value(), input2->value()) >= 0)
        return false;

    return true;
}

