/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <Resource.h>
#include <db/DBFactory.h>
#include <dlg/ExportDlg.h>
#include <dlg/InfoDlg.h>
#include <dlg/ProgressDlg.h>
#include <gui/XTrader.h>
#include <help/Export.h>
#include <MHDate.h>
#include <MHDebug.h>
#include <FL/fl_draw.h>
#include <FL/fl_ask.h>
#include <FL/Fl_File_Chooser.h>



/**
*  Create export dialog.
*/
ExportDlg::ExportDlg () :
Fl_Dialog (370, 265, "EXPORT DATABASE TO FILE") {
    Fl_Window* o = this;

    o->box(FL_UP_BOX);
    { Fl_Input* o = aFile = new Fl_Input(10, 25, 245, 25, "EXPORT TO FILE");
      o->labelfont(1);
      o->align(FL_ALIGN_TOP_LEFT);
    }
    { Fl_Box* o = aSelectLabel = new Fl_Box(10, 75, 245, 180, "SELECT DATA");
      o->box(FL_ENGRAVED_FRAME);
      o->labelfont(1);
      o->align(FL_ALIGN_TOP_LEFT);
    }
    { Fl_Check_Button* o = aSecurity = new Fl_Check_Button(15, 80, 145, 25, "SECURITIES");
      o->down_box(FL_DOWN_BOX);
    }
    { Fl_Check_Button* o = aSecurityList = new Fl_Check_Button(15, 105, 145, 25, "SECURITY LISTS");
      o->down_box(FL_DOWN_BOX);
    }
    { Fl_Check_Button* o = aPrice = new Fl_Check_Button(15, 130, 145, 25, "PRICE DATA");
      o->down_box(FL_DOWN_BOX);
    }
    { Fl_Check_Button* o = aAccount = new Fl_Check_Button(15, 155, 145, 25, "ACCOUNTS");
      o->down_box(FL_DOWN_BOX);
    }
    { Fl_Check_Button* o = aTransaction = new Fl_Check_Button(15, 180, 145, 25, "TRANSACTIONS");
      o->down_box(FL_DOWN_BOX);
    }
    { Fl_Check_Button* o = aGuard = new Fl_Check_Button(15, 205, 145, 25, "GUARD");
      o->down_box(FL_DOWN_BOX);
    }
    { Fl_Check_Button* o = aSettings = new Fl_Check_Button(15, 230, 145, 25, "SETTINGS");
      o->down_box(FL_DOWN_BOX);
    }
    aExport = new Fl_Return_Button(265, 25, 100, 25, "&EXPORT");
    aBrowse = new Fl_Button(265, 60, 100, 25, "&BROWSE...");
    aHelp = new Fl_Button(265, 95, 100, 25, "&HELP");
    aClose = new Fl_Button(265, 130, 100, 25, "&CLOSE");
    o->end();

    aExport->callback (Fl_Dialog::cb2_, this);
    aBrowse->callback (Fl_Dialog::cb3_, this);
    aHelp->callback (Fl_Dialog::cb4_, this);
    aClose->callback (Fl_Dialog::cb1_, this);

    label (STRING(DLG_EXPORT));
    aFile->label (STRING(RESULT_FILE));
    aSelectLabel->label (STRING(SELECT_TABLES));
    aSecurity->label (STRING(SECURITY));
    aSecurityList->label (STRING(SECURITY_LIST));
    aPrice->label (STRING(PRICE));
    aAccount->label (STRING(ACCOUNT));
    aTransaction->label (STRING(TRANSACTIONS));
    aGuard->label (STRING(PRICE_GUARD));
    aSettings->label (STRING(CURRENT_WORKSPACE));

    aExport->label (STRING(EXPORT));
    aBrowse->label (STRING(BROWSE));
    aHelp->label (STRING(HELP));
    aClose->label (STRING(CLOSE_));

    aSecurity->labelsize (XTrader::FontSize());
    aSecurityList->labelsize (XTrader::FontSize());
    aPrice->labelsize (XTrader::FontSize());
    aAccount->labelsize (XTrader::FontSize());
    aTransaction->labelsize (XTrader::FontSize());
    aGuard->labelsize (XTrader::FontSize());
    aSettings->labelsize (XTrader::FontSize());

    XTrader::FixButton (aExport);
    XTrader::FixButton (aBrowse);
    XTrader::FixButton (aHelp);
    XTrader::FixButton (aClose);
}



/**
*  Export database to file
*/
void ExportDlg::cb1 () {
    aRetVal = 0;
}



/**
*  Export database to file
*/
void ExportDlg::cb2 () {
    DBG ("cbExport")

    if (aFile->value () == 0 || *aFile->value () == '\0') 
        fl_alert (STRING(ERROR_MISSING_FILE));
    else {
        XTrader::StartWork (STRING(INFO_EXPORT));

        try {
            DBFactory::Export (aFile->value(), aAccount->value() == '\1', aTransaction->value() == '\1', aSecurityList->value() == '\1', aSecurity->value() == '\1', aPrice->value() == '\1', aGuard->value() == '\1', aSettings->value() == '\1');
            fl_beep (FL_BEEP_MESSAGE);
        }
        catch (const char* e) {
            fl_beep (FL_BEEP_ERROR);
            label (e);
        }
        XTrader::StopWork ("");
    }
}



/**
*  Get filename
*/
void ExportDlg::cb3 () {
    char* file = fl_file_chooser (STRING(DLG_SELECT_WRITE_FILE), "*.*", "");
    if (file && *file) aFile->value (file);
}



/**
*  Show help
*/
void ExportDlg::cb4 () {
    InfoDlg dlg (STRING(HELP_EXPORT), aaExport);
    dlg.show_modal (this);
}
