/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <dlg/ProgressDlg.h>
#include <gui/XTrader.h>
#include <Progress.h>
#include <Resource.h>
#include <MHDate.h>
#include <MHDebug.h>
#include <MHUtil.h>
#include <FL/fl_draw.h>
#include <FL/fl_ask.h>



/**
*  Create about dialog box
*  @param const char* - Info message
*  @param int         - 0 to have a no cancel button, 1 to disable, default value
*/
ProgressDlg::ProgressDlg (const char* message, int disableCancel, int disableprogress) :
Fl_Dialog (410, 95, "XTRADER") {
    Fl_Window* o = this;

    o->box(FL_UP_BOX);
    aProgress = new Fl_Progress(5, 30, 400, 25);
    aCancel = new Fl_Button(5, 65, 400, 25, "CANCEL");
    { Fl_Box* o = aInfo = new Fl_Box(5, 5, 400, 25);
      o->labelfont(1);
      o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    }
    { Fl_Box* o = aInfo2 = new Fl_Box(5, 30, 400, 25);
      o->labelfont(1);
      o->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    }
    o->end();

    aCancel->callback (Fl_Dialog::cb1_, this);


    label (STRING(DLG_PROGRESS));
    aCancel->label (STRING(CANCEL));
    XTrader::FixButton (aCancel);

    if (message && *message) aInfo->label (message);
    if (disableCancel) aCancel->deactivate ();
    if (disableprogress) {
        aProgress->size (0, 0);
        aInfo2->activate ();
    }
    aProgress->value (0);
    aDate  = new MHDate ();
    aStart = MHUtil::Time();
    aDiff  = 0;
}




/**
*  Cancel work
*/
void ProgressDlg::cb1() {
    DBG ("cbCancel")
    xtrader_cancel();
}



/**
*  Update progress
*  @param double      - 0-100 percent
*  @param const char* - Info message
*/
void ProgressDlg::Value (double v, const char* message) {
    if (aProgress->w() > 0 && ((int)v) > ((int)aProgress->value())) {
        snprintf (aBuffer, 99, "%.0f%%", v);
        aProgress->label (aBuffer);
        aProgress->value ((float)v);
        if (message && *message) aInfo->label (message);
        redraw ();
        Fl::check ();
    }
    else if (aProgress->w() == 0) {
        int sec = int (MHUtil::Time() - aStart);
        if (aDiff != sec) {
            aDiff = sec;
            int m = (int)aDiff / 60;
            int s = aDiff - (m * 60);
            snprintf (aBuffer2, 99, "00:%02d:%02d", m, s);
            aInfo2->label (aBuffer2);
            aInfo2->redraw ();
            redraw ();
            Fl::check ();
        }
    }
}
