/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include "dlg/SetupDlg.h"
#include "db/DBFactory.h"
#include "db/Pref.h"
#include "table/PrefDownloadData.h"
#include "table/PrefGeneralData.h"
#include "table/PrefPortfolioChartData.h"
#include "table/PrefPriceChartData.h"
#include "table/PrefReportData.h"
#include "table/PrefToolbarData.h"
#include "report/ReportTypes.h"
#include "Enums.h"
#include "Resource.h"
#include <MHCSVFile.h>
#include <MHString.h>
#include <MHVector.h>
#include <fl/Fl_Table.h>
#include <stdio.h>



/**
*  Create a list selection dialog
*  @param MHVector* - Vector with selection strings
*/
SetupDlg::SetupDlg (int layout) :
Fl_Dialog (780, 500, "XTRADER") {
    Fl_Window* o = this;

    o->box(FL_UP_BOX);
    { Fl_Browser* o = aPreference = new Fl_Browser(5, 20, 190, 445, "PREFERENCE TYPE");
      o->type(2);
      o->box(FL_DOWN_BOX);
      o->labelfont(1);
      o->align(FL_ALIGN_TOP_LEFT);
    }
    { Fl_Group* o = aSettings = new Fl_Group(200, 20, 570, 445, "SETTINGS");
      o->box(FL_DOWN_BOX);
      o->labelfont(1);
      o->align(FL_ALIGN_TOP_LEFT);
      o->end();
    }
    aClose = new Fl_Button(5, 470, 765, 25, "CLOSE");
    o->end();

    aClose->callback (Fl_Dialog::cb1_, this);
    aPreference->callback (Fl_Dialog::cb2_, this);

    aTable = new Fl_Table (this, 0, aSettings->x(), aSettings->y(), aSettings->w(), aSettings->h());
    aSettings->add (aTable);
    this->label (STRING(DLG_SETUP));
    aPreference->label (STRING(PREFERENCE_TYPE));
    aSettings->label ("");

    if (DBFactory::GetPrefIntValue (STRING(PREF_WRITEPROTECTED), 0, 1000, 0) == 0)
        aClose->label (STRING(CLOSE_));
    else
        aClose->label (STRING(CLOSE_NOT_SAVE));

    aPreference->add (STRING(GENERAL));
    aPreference->add (STRING(TOOLBAR));
    aPreference->add (STRING(REPORT));
    aPreference->add (STRING(PRICE_CHART_LAYOUT_1));
    aPreference->add (STRING(PRICE_CHART_LAYOUT_2));
    aPreference->add (STRING(PRICE_CHART_LAYOUT_3));
    aPreference->add (STRING(PORTFOLIO_CHART_LAYOUT_1));
    aPreference->add (STRING(PORTFOLIO_CHART_LAYOUT_2));
    aPreference->add (STRING(PORTFOLIO_CHART_LAYOUT_3));
    aPreference->add (STRING(DOWNLOAD));

    aPreference->textsize (XTrader::FontSize());
    aPreference->labelsize (XTrader::FontSize());
    XTrader::FixButton (aClose);
    aSettings->labelsize (XTrader::FontSize());

    switch (layout) {
        case 0:
            aPreference->value(4);
            break;

        case 1:
            aPreference->value(5);
            break;

        case 2:
            aPreference->value(6);
            break;

        case 3:
            aPreference->value(7);
            break;

        case 4:
            aPreference->value(8);
            break;

        case 5:
            aPreference->value(9);
            break;
    }

    cb2 ();
    aTable->resize (aTable->x(), aTable->y(), aTable->w(), aTable->h());
    resize (x(), y(), 640, 480);
}



/**
*/
SetupDlg::~SetupDlg () {
    aSettings->remove (aTable);
    delete aTable;
}



/**
*  Save data then close dialog
*/
void SetupDlg::cb1 () {
    try {
        if (DBFactory::GetPrefIntValue (STRING(PREF_WRITEPROTECTED), 0, 1000, 0) == 0) {
            Pref::Save (DBFactory::LoadPref ());
            XTrader::GetApp()->BuildToolbar();
        }
        if (DBFactory::GetPrefIntValue (STRING(PREF_COMPRESS), XTRADER_NO, XTRADER_YES, XTRADER_NO) == XTRADER_YES)
            MHCSVFile::Compress();
        else
            MHCSVFile::Uncompress();
        aRetVal = 0;
    }
    catch (const char* message) {
        fl_beep (FL_BEEP_ERROR);
        fl_alert (message);
    }
}



/**
*  User has selected new type of settings to edit
*/
void SetupDlg::cb2 () {
    switch (aPreference->value()) {
        case 1:
            aTable->data (new PrefGeneralData ());
            break;

        case 2:
            aTable->data (new PrefToolbarData ());
            break;

        case 3:
            aTable->data (new PrefReportData ());
            break;

        case 4:
            aTable->data (new PrefPriceChartData (0));
            break;

        case 5:
            aTable->data (new PrefPriceChartData (1));
            break;

        case 6:
            aTable->data (new PrefPriceChartData (2));
            break;

        case 7:
            aTable->data (new PrefPortfolioChartData (0));
            break;

        case 8:
            aTable->data (new PrefPortfolioChartData (1));
            break;

        case 9:
            aTable->data (new PrefPortfolioChartData (2));
            break;

        case 10:
            aTable->data (new PrefDownloadData ());
            break;
    }
    Fl_Table_Data* data = aTable->data ();
    if (data) {
        data->show_label (false);
        //data->stripe (fl_rgb_color(127, 255, 255));
    }
    aTable->move_cursor(-999, -999);
}
