/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <dlg/WorkSpaceDlg.h>
#include <db/DBFactory.h>
#include <gui/XTrader.h>
#include <report/ReportTypes.h>
#include <Resource.h>
#include <MHString.h>
#include <MHVector.h>
#include <FL/fl_ask.h>



/**
*  Create a list selection dialog
*  @param MHVector* - Vector with selection strings
*/
WorkSpaceDlg::WorkSpaceDlg () :
Fl_Dialog (370, 260, "XTrader") {
    Fl_Window* o = this;

    o->box(FL_UP_BOX);
    { Fl_Browser* o = aBrowser = new Fl_Browser(5, 10, 250, 240);
      o->type(2);
      o->labelfont(1);
      o->align(FL_ALIGN_TOP_LEFT);
    }
    aNew = new Fl_Button(265, 45, 100, 25, "NEW");
    aDelete = new Fl_Button(265, 80, 100, 25, "DELETE");
    aCancel = new Fl_Button(265, 150, 100, 25, "CANCEL");
    aLoad = new Fl_Return_Button(265, 10, 100, 25, "LOAD");
    aRename = new Fl_Button(265, 115, 100, 25, "RENAME");
    o->end();

    aCancel->callback (Fl_Dialog::cb1_, this);
    aDelete->callback (Fl_Dialog::cb2_, this);
    aNew->callback (Fl_Dialog::cb4_, this);
    aLoad->callback (Fl_Dialog::cb3_, this);
    aRename->callback (Fl_Dialog::cb5_, this);

    label (STRING(DLG_WORKSPACE));
    aCancel->label (STRING(CANCEL));
    aDelete->label (STRING(DELETE_));
    aRename->label (STRING(RENAME));
    aLoad->label (STRING(LOAD));
    aNew->label (STRING(NEW));

    aBrowser->textsize (XTrader::FontSize());
    aBrowser->labelsize (XTrader::FontSize());
    XTrader::FixButton (aCancel);
    XTrader::FixButton (aDelete);
    XTrader::FixButton (aLoad);
    XTrader::FixButton (aNew);
    XTrader::FixButton (aRename);

    loadFiles ();
}



/**
*
*/
WorkSpaceDlg::~WorkSpaceDlg () {
    aFiles.Erase ();
}



/**
*  Erase work space from disk
*/
void WorkSpaceDlg::cb2 () {
    if (aBrowser->value () == 0 && aBrowser->value () < aFiles.Size()) return;
    MHString* file = (MHString*) aFiles[aBrowser->value () - 1];
    MHString  s    = DBFactory::Database();

    s += *file;
    s += ".ws";
    ::remove (s());
    loadFiles ();
}



/**
*  Load existing workspace
*/
void WorkSpaceDlg::cb3 () {
    if (aBrowser->value () == 0 && aBrowser->value () < aFiles.Size()) return;
    MHString* file = (MHString*) aFiles[aBrowser->value () - 1];

    if (XTrader::GetApp()->Close()) {
        DBFactory::SetWorkSpace (file->Get());
        XTrader::GetApp()->LoadWorkSpace();
        aRetVal = 1;
    }
}



/**
*  Create new empty preference file
*/
void WorkSpaceDlg::cb4 () {
    const char* file = fl_input (STRING(INFO_NEW_WS));

    if (file && *file) {
        MHString s = DBFactory::Database();
        s += file;
        s += ".ws";
        FILE* f = fopen (s(), "wb");
        fclose (f);
        loadFiles ();
    }
}



/**
*
*/
void WorkSpaceDlg::cb5 () {
    if (aBrowser->value () == 0 && aBrowser->value () < aFiles.Size()) return;
    MHString* file = (MHString*) aFiles[aBrowser->value () - 1];

    const char* file_new = fl_input (STRING(INFO_RENAME_WS));

    if (file_new) {
        MHString  s1 = DBFactory::Database();
        s1 += *file;
        s1 += ".ws";

        MHString s2 = DBFactory::Database();
        s2 += file_new;
        s2 += ".ws";
        rename (s1(), s2());
        loadFiles ();
    }
}


/**
*
*/
void WorkSpaceDlg::loadFiles () {
    dirent** list;
    int      listSize, i;

    listSize = fl_filename_list (DBFactory::Database(), &list);

    aBrowser->clear ();
    aFiles.Erase ();
    for (i = 0; i < listSize; i++) {
        MHString s = list[i]->d_name;
        MHVector v;

        s.Split ('.', &v);
        if (v.Size() == 2) {
            MHString* s1 = (MHString*) v[0];
            MHString* s2 = (MHString*) v[1];

            if (*s2 == "ws") {
                aBrowser->add (s1->Get());
                aFiles.Push (new MHString (s1->Get()));
            }
        }
        v.Erase ();
    }

    for (i = listSize; i > 0;) {
        free((void*)(list[--i]));
    }

    free ((void*)list);
}
