/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include "download/BaseDownload.h"
#include "db/DBFactory.h"
#include "db/Price.h"
#include "db/Security.h"
#include "db/Set.h"
#include "db/SetItem.h"
#include "Enums.h"
#include <MHDate.h>
#include <MHUtil.h>
#include <pinet.h>

USING_PTYPES



/**
*
*/
BaseDownload::BaseDownload () {
    aBuffer = new char[BUFFER_SIZE + 2];
}



/**
*
*/
BaseDownload::~BaseDownload () {
    delete aBuffer;
}



/**
*  !!! What to do with blocking requests?
*/
const char* BaseDownload::getData (const char* server, const char* url) {
    pt::ipstream* stream = 0;
    aBuffer[0] = '\0';
    aStatus = XTRADER_WORKING;

    try {
        pt::ipaddress adress = pt::phostbyname (server);

        if (adress == pt::ipnone) {
            DBFactory::WriteLog ("Error contacting %s%s", server, url);
            aStatus = XTRADER_ERROR_NETWORK;
        }
        else {
            char    tmp[256];
            int     index = 0;
            MHDate  date;

            stream = new pt::ipstream (adress, 80);

            snprintf (tmp, 255, "GET %s", url);
            stream->open();
            stream->putline(tmp);
            stream->flush();
            double start = MHUtil::Time ();

            while (!stream->get_eof() && index < BUFFER_SIZE) {
                if (stream->waitfor(0) == true)
                    aBuffer[index++] = stream->get();

                if (aStatus == XTRADER_CANCEL) {
                    aBuffer[0] = '\0';
                    break;
                }
                else if ((MHUtil::Time() - start) > 30) {
                    aBuffer[0] = '\0';
                    aStatus = XTRADER_ERROR_NETWORK;
                    break;
                }
            }
            aBuffer[index++] = '\0';
            stream->close();
            delete stream;
            stream = 0;
        }
    }
    catch (pt::estream* e) {
        DBFactory::WriteLog ("Error reading from %s%s (%s)", server, url, pt::pconst(e->get_message()));
        delete e;
        if (stream)
            delete stream;
        aStatus = XTRADER_ERROR_NETWORK;
        aBuffer[0] = '\0';
    }
    return aBuffer;
}
