/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is di stributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/


#include "download/DownloadThread.h"
#include "download/BaseDownload.h"
#include "db/DBFactory.h"
#include "db/Price.h"
#include "db/Security.h"
#include "Progress.h"
#include "Enums.h"
#include <MHFile.h>

USING_PTYPES



/**
*
*/
DownloadThread::DownloadThread () :
thread (false) {
    aStatus = XTRADER_WORKING;
}



/**
*
*/
void DownloadThread::execute () {
    int         rows    = 0;
    FILE*       file    = 0;
    Security*   sec     = 0;
    MHVector*   prices  = 0;

    aStatus = XTRADER_WORKING;
    DBFactory::WriteLog ("Starting download");
    xtrader_reset (aSecs->Size());
    MHFile::Delete (aFileName);
    file = fopen (aFileName, "w");

    if (file == 0) {
        aStatus = XTRADER_ERROR_OPEN_FILE;
        return;
    }

    for (int f = 0; f < aSecs->Size(); f++) {
        try {
            sec    = (Security*) (*aSecs)[f];
            prices = aLoad->Load (aDownloadType, aStart, aStop, sec);

            Price::Export (sec, prices, file);
            rows += prices->Size();
            prices->Erase ();
            delete prices;
            prices = 0;

            xtrader_add(1);
            if (xtrader_abort())
                aStatus = XTRADER_CANCEL;
            if (aStatus == XTRADER_CANCEL) {
                break;
            }
            else if (aLoad->aStatus == XTRADER_ERROR_NETWORK) {
                aStatus = XTRADER_ERROR_NETWORK;
                break;
            }
            else if (f == (aSecs->Size() - 1))
                aStatus = XTRADER_DONE;
        }
        catch (...) {
            if (prices) {
                prices->Erase ();
                delete prices;
                prices = 0;
            }
        }
    }
    DBFactory::WriteLog ("%d rows downloaded", rows);
    fclose (file);
}
