/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include "gui/ChartView.h"
#include "gui/XTrader.h"
#include "db/Account.h"
#include "db/DBFactory.h"
#include "db/Security.h"
#include "db/Set.h"
#include "db/SetItem.h"
#include "dlg/DateDlg.h"
#include "dlg/SetupDlg.h"
#include "chart/PortfolioChart.h"
#include "chart/PriceChart.h"
#include "report/PortfolioReport.h"
#include "report/PriceReport.h"
#include "table/ReportData.h"
#include "Resource.h"
#include <MHDate.h>
#include <MHDebug.h>
#include <MHPrice.h>
#include <MHVector.h>
#include <FL/Fl.h>
#include <FL/fl_ask.h>
#include <FL/fl_draw.h>
#include <FL/Fl_Button.h>
#include <FL/Fl_File_Chooser.h>
#include <fl/Fl_Layout.h>
#include <FL/Fl_Hold_Browser.h>
#include <fl/Fl_DC.h>



/**
*
*/
ChartView::ChartView (const char* workSpace, int type, int x, int y, int w, int h, const char* label) :
BaseView (workSpace, type, x, y, w, h, label) {
    aView       = 0;
    aDCLayout   = new Fl_BorderLayout (0, 0, 0, 0);
    aDCSlider   = new Fl_Slider (0, 0, 0, 22);
    aDC         = new Fl_DC (0, 0, 0, 0, 0, aDCSlider);
    aTimeType   = MHDate::WEEKDAY;

    aDCSlider->type (FL_HOR_NICE_SLIDER);
    aDC->SetFontSize (XTrader::FontSize());
    aDC->color (FL_RED);

    aDCLayout->add (Fl_Layout::CENTER, aDC);
    aDCLayout->add (Fl_Layout::SOUTH, aDCSlider);
    aEditLayout->addRight (aDCLayout);
    createToolbar (STRING(SETTINGS), STRING(DATE2), STRING(DAY), STRING(WEEK), STRING(LAYOUT1), STRING(LAYOUT2), STRING(LAYOUT3), STRING(RELOAD), STRING(PREV), STRING(NEXT), STRING(CLOSE_), 0);
    createBrowser ();
    loadBrowserData1 ();
    loadBrowserData2 ();
    enableButtons ();

    if (aBrowser2)
        Fl::focus (aBrowser2);
    else if (aBrowser1)
        Fl::focus (aBrowser1);
}



/**
*  Destroy view
*/
ChartView::~ChartView () {
}



/**
*  Check if view can be closed
*/
bool ChartView::CanClose () {
    return true;
}



/**
*  Callback for browser1, create portfolio chart os load securitites into browser2
*/
void ChartView::cbBrowser1 () {
    aMem1 = aMem2 = 0;

    try {
        switch (aType) {
            case PORTFOLIO: {
                if (aBrowser1->value () == 0 || aBrowser1->value () > aList1->Size()) return;
                if (CanClose() == false) return;
                Account*  acc  = (Account*) (*aList1)[aBrowser1->value() - 1];
                MHDate    date;
                MHDate    date2 (date.GetYear(), 1, 1);

                XTrader::StartWork (STRING(INFO_CREATE_REPORT), 1);
                PortfolioChart* chart = new PortfolioChart (aDC, aView);
                if (aStartDate < aStopDate && aStartDate != "") {
                    chart->aStartDate = aStartDate;
                    chart->aStopDate  = aStopDate;
                }
                else {
                    aStartDate = chart->aStartDate;
                    aStopDate  = chart->aStopDate;
                }
                chart->aData = acc;
                chart->Create ();
                aDC->AddChart (chart->aChart);
                aDC->aChart->Init (aDC);
                delete chart;
                XTrader::StopWork ("");
                redraw ();
                aMem1 = acc;
                break;
            }

            case SET_SETITEM: {
                if (CanClose() == false) return;
                loadBrowserData2 ();
                break;
            }
        }
    }
    catch (const char* e) {
        fl_beep (FL_BEEP_ERROR);
        fl_alert (e);
    }

    enableButtons ();
}



/**
*  Callback for browser2, use selected security and create chart
*/
void ChartView::cbBrowser2 () {
    DBG ("ChartView::cbBrowser2");
    aMem2 = 0;

    try {
        switch (aType) {
            case SET_SETITEM: {
                if (aBrowser2->value () == 0 || aBrowser2->value () > aList2->Size()) return;
                if (CanClose() == false) return;
                SetItem*  item = (SetItem*) (*aList2)[aBrowser2->value() - 1];
                MHDate    date;
                MHDate    date2 (date.GetYear(), 1, 1);

                PriceChart* chart = new PriceChart (aDC, aView, aTimeType);
                if (aStartDate < aStopDate && aStartDate != "") {
                    chart->aStartDate = aStartDate;
                    chart->aStopDate  = aStopDate;
                }
                else {
                    aStartDate = chart->aStartDate;
                    aStopDate  = chart->aStopDate;
                }
                chart->aData = item->GetSecurity();
                chart->Create ();
                aDC->AddChart (chart->aChart);
                aDC->aChart->Init (aDC);
                delete chart;
                redraw ();
                aMem2 = item;
                break;
            }
        }
    }
    catch (const char* e) {
        fl_beep (FL_BEEP_ERROR);
        fl_alert (e);
    }

    enableButtons ();
}



/**
*  Callback for buttons
*/
void ChartView::cbButton (Fl_Button* button) {
    try {
        if (strcmp (button->label(), STRING(CLOSE_)) == 0)
            XTrader::Close (this);
        else if (strcmp (button->label(), STRING(DAY)) == 0) {
            aTimeType = MHDate::WEEKDAY;
            reload ();
        }
        else if (strcmp (button->label(), STRING(DATE2)) == 0) {
            DateDlg dlg (aStartDate(), aStopDate());

            if (dlg.show_modal (XTrader::GetApp()) == 1) {
                aStartDate  = dlg.StartDate();
                aStopDate   = dlg.StopDate();
                reload ();
            }
        }
        else if (strcmp (button->label(), STRING(LAYOUT1)) == 0) {
            aView = 0;
            reload ();
        }
        else if (strcmp (button->label(), STRING(LAYOUT2)) == 0) {
            aView = 1;
            reload ();
        }
        else if (strcmp (button->label(), STRING(LAYOUT3)) == 0) {
            aView = 2;
            reload ();
        }
        else if (strcmp (button->label(), STRING(NEXT)) == 0) {
            aBrowser2->value (aBrowser2->value() + 1);
            reload ();
        }
        else if (strcmp (button->label(), STRING(PREV)) == 0) {
            aBrowser2->value (aBrowser2->value() - 1);
            reload ();
        }
        else if (strcmp (button->label(), STRING(SETTINGS)) == 0) {
            SetupDlg dlg (aType == SET_SETITEM ? (0 + aView) : (3 + aView));
            dlg.show_modal (XTrader::GetApp());
            reload ();
        }
        else if (strcmp (button->label(), STRING(RELOAD)) == 0) {
            reload ();
        }
        else if (strcmp (button->label(), STRING(WEEK)) == 0) {
            aTimeType = MHDate::FRIDAY;
            reload ();
        }
    }
    catch (const char* x) {
        fl_alert (x);
    }

    enableButtons ();
}



/**
*  Enable/disable buttons
*/
void ChartView::enableButtons () {
    disableButtons ();

    if (aDC->aChart) {
        if (aType == PORTFOLIO)
            ;
        else if (aTimeType == MHDate::WEEKDAY)
            enableButton (STRING(WEEK));
        else
            enableButton (STRING(DAY));
        enableButton (STRING(RELOAD));

        if (aBrowser2) {
            if (aBrowser2->value () > 0 && aBrowser2->value () < aList2->Size())  enableButton (STRING(NEXT));
            if (aBrowser2->value () > 1  && aBrowser2->value () <= aList2->Size())  enableButton (STRING(PREV));
        }
    }

    if (aView != 0) enableButton (STRING(LAYOUT1));
    if (aView != 1) enableButton (STRING(LAYOUT2));
    if (aView != 2) enableButton (STRING(LAYOUT3));
    enableButton (STRING(SETTINGS));
    enableButton (STRING(DATE2));
    enableButton (STRING(CLOSE_));

    aToolbar->redraw ();
}



/**
*
*/
int ChartView::handle (int event) {
    if (event == FL_MOVE) {
        char      message[200];
        MHString  label;
        double    lv;
        double    rv;
        char      lc = ' ';
        char      rc = ' ';
        int       lp = 0;
        int       rp = 0;

        if (aDC->aLeft.aOpen < 1.0) {
            lv = aDC->aLeft.aClose;
            lp = 2;
        }
        else if (aDC->aLeft.aOpen < 10.0) {
            lv = aDC->aLeft.aClose;
            lp = 1;
        }
        else if (aDC->aLeft.aOpen < 1000.0) {
            lv = aDC->aLeft.aClose;
            lp = 0;
        }
        else if (aDC->aLeft.aOpen < 1000000.0) {
            lv = aDC->aLeft.aClose / 1000.0;
            lc = '\'';
            lp = 0;
        }
        else {
            lv = aDC->aLeft.aClose / 1000000.0;
            lc = '\"';
            lp = 1;
        }

        if (aDC->aRight.aOpen < 1.0) {
            rv = aDC->aRight.aClose;
            rp = 2;
        }
        else if (aDC->aRight.aOpen < 10.0) {
            rv = aDC->aRight.aClose;
            rp = 1;
        }
        else if (aDC->aRight.aOpen < 1000.0) {
            rv = aDC->aRight.aClose;
            rp = 0;
        }
        else if (aDC->aRight.aOpen < 1000000.0) {
            rv = aDC->aRight.aClose / 1000.0;
            rc = '\'';
            rp = 0;
        }
        else {
            rv = aDC->aRight.aClose / 1000000.0;
            rc = '\"';
            rp = 1;
        }

        if (aDC->aLeft.aVol > 0 && aDC->aRight.aVol < 0) {
            snprintf (message, 199, "%8s  %12.*f%c", aDC->aLeft.aDate, lp, lv, lc);
            XTrader::SetMessage (message);
        }
        else if (aDC->aLeft.aVol < 0 && aDC->aRight.aVol > 0) {
            snprintf (message, 199, "%8s  %12.*f%c", aDC->aRight.aDate, rp, rv, rc);
            XTrader::SetMessage (message);
        }
        else if (aDC->aLeft.aVol > 0 && aDC->aRight.aVol > 0) {
            snprintf (message, 199, "%8s  %12.*f%c  %12.*f%c", aDC->aLeft.aDate, lp, lv, lc, rp, rv, rc);
            XTrader::SetMessage (message);
        }
        else
            XTrader::SetMessage ("");
    }
    return Fl_Group::handle(event);

}



/**
*  reload data
*/
void ChartView::reload () {
    if (aType == SET_SETITEM)
        cbBrowser2 ();
    else if (aType == PORTFOLIO)
        cbBrowser1 ();
}
