/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#ifndef XTrader_h
#define XTrader_h

#include <FL/Fl.h>
#include <FL/fl_ask.h>
#include <FL/fl_draw.h>
#include <FL/Fl_Box.h>
#include <FL/Fl_Double_Window.h>
#include <FL/Fl_Tabs.h>
#include <gui/BaseView.h>
#include <MHDebug.h>
#include <MHMessage.h>
#include <MHString.h>

class Fl_Layout;
class Fl_Menu_Bar;
struct Fl_Menu_Item;



/**
*
*/
class Tabs : public Fl_Tabs {
public:
                            Tabs (int x, int y, int w, int h) : Fl_Tabs(x, y, w, h, 0) {aOff = false;}
    void                    off (bool o) {aOff = o;}
    void                    resize (int x, int y, int w, int h) {
                                DBG ("Tabs::resize")
                                Fl_Widget::resize(x, y, w, h);
                                if (aOff == true) return;
                                for (int f = 0; f < this->children(); f++) {
                                    DBG_LOG1 ("%d", f)
                                    BaseView* v = (BaseView*) this->child (f);
                                    //if (this->w() != v->w() || (this->h() - 20) != v->h())
                                        v->resize (this->x(), this->y() + 20, this->w(), this->h() - 20);
                                    DBG_LOG4 ("view       %4d, %4d, %4d, %4d", v->x(),  v->y(),  v->w(),  v->h())
                                }
                            }
private:
    bool                    aOff;
};



/**
*  XTrader is main window object and manages the menu,
*  toolbar, statusbar and the different tab views
*/
class XTrader : public Fl_Double_Window {
public:
                            XTrader (int w, int h);
                            ~XTrader ();

    void                    BuildToolbar ();
    void                    cbClose ();
    static void             cbClose_ (Fl_Double_Window* w, void* data) {XTrader::GetApp()->cbClose ();}
    void                    cbMenu (int id);
    static void             cbMenu_ (Fl_Menu_Bar* w, void* data) {XTrader::GetApp()->cbMenu (int(data));}
    bool                    Close ();
    static void             Close (BaseView* view) {XTrader::aaApp->close (view);}
    static void             FixButton (Fl_Button* b);
    static int              FontSize ();
    static XTrader*         GetApp () {return aaApp;}
    int                     handle (int event);
    void                    LoadWorkSpace ();
    void                    resize (int x, int y, int w, int h);
    void                    SavePos ();
    void                    SaveWorkSpace ();
    void                    Show (int argc, char** argv);
    static void             SendMessage (const MH* sender, const char* m1 = 0, const char* m2 = 0, const char* m3 = 0, void* m4 = 0) {XTrader::aaApp->aListeners->Send (sender, m1, m2, m3, m4);}
    static void             SetError (const char* message) {STRNCPY (XTrader::aaApp->aBuffer, message, 199); XTrader::aaApp->aStatus->label (XTrader::aaApp->aBuffer); fl_beep (FL_BEEP_ERROR); XTrader::aaApp->aStatus->redraw(); Fl::flush();}
    static void             SetInfo (const char* message) {STRNCPY (XTrader::aaApp->aBuffer, message, 199); XTrader::aaApp->aStatus->label (XTrader::aaApp->aBuffer); fl_beep (FL_BEEP_MESSAGE); XTrader::aaApp->aStatus->redraw(); Fl::flush();}
    static void             SetMessage (const char* message) {STRNCPY (XTrader::aaApp->aBuffer, message, 199); XTrader::aaApp->aStatus->label (XTrader::aaApp->aBuffer); XTrader::aaApp->aStatus->redraw(); Fl::flush();}
    static void             StartWork (const char* message, int showtime = 0);
    static void             StopWork (const char* message);


protected:
    MHMessage*              aListeners;
    BaseView*               aLastClosedView;
    Tabs*                   aTabs;
    Fl_Menu_Bar*            aMenu;
    Fl_BorderLayout*        aLayout;
    Fl_Box*                 aStatus;
    Fl_BorderLayout*        aTop;
    Fl_HorizontalLayout*    aToolbar;
    static XTrader*         aaApp;
    char                    aBuffer[200];
    MHString                aLabel;
    bool                    aInitSize;
    int                     aX, aY, aW, aH;

    void                    close (BaseView* view);
    void                    createMenu ();
    BaseView*               createView (const char* view);
    bool                    findView (const char* label);
    Fl_Menu_Item*           getMenuItem (const char* label);
};

#endif
