/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <db/DBFactory.h>
#include <db/Pref.h>
#include <gui/XTrader.h>
#include <Resource.h>
#include <MHDate.h>
#include <MHDebug.h>
#include <MHFile.h>
#include <FL/Fl.h>
#include <FL/fl_ask.h>
#include <FL/fl_file_chooser.h>
#include <FL/Fl_Preferences.h>



/**
*  Start application here
*/
int main (int argc, char **argv) {
    DBG_START
    DBG_OPTIONS ("T=1,L=1,F=1,O=tmp/dbg_xtrader.txt")
    //DBG_WATCH ("XTrader::resize,BaseView::resize,Tabs::resize,MHChartDay::calcSize,")
    DBG_WATCH ("MHChartDay::drawArea,MHChartDay::calcSize")

    char            workspace[1000];
    char            database[2000];
    const char*     dir;
    XTrader*        trader = 0;
    Fl_Preferences  pref (Fl_Preferences::USER, "xtrader.sourceforge.net", "XTrader");

    Resource::Init ();

    #ifdef BUGG
        const char* s = STRING(_LAST_);
        if (s == 0 || strcmp(s, "LAST") != 0) {
            printf ("_LAST_=%s\n", s);
            exit(1);
        }
    #endif

    pref.get ("database", database, "", 1999);
    pref.get ("workspace", workspace, STRING(DEFAULT_WORKSPACE), 999);

    if (strlen (database) > 0 && database[strlen(database) - 1] == '/')
        database[strlen (database) - 1] = '\0';
    if (*database == '\0' || MHFile::IsDir (database) == false)
        dir = fl_dir_chooser (STRING(DLG_SELECT_DB), "", 0);
    else
        dir = database;

    if (dir) {
        try {
            DBFactory::Start (dir);
            DBFactory::SetWorkSpace (workspace);
            DBFactory::SetCacheSize (DBFactory::GetPrefIntValue (STRING(PREF_CACHE_SIZE), 5, 100, 25));
            DBFactory::SetCacheTimeout (DBFactory::GetPrefIntValue (STRING(PREF_CACHE_TIMEOUT), 60, 3600, 300));

            trader = new XTrader (0, 0);
            trader->Show(argc, argv);
            trader->LoadWorkSpace ();
            Fl::run();
            pref.set ("database", DBFactory::Database());
            pref.set ("workspace", DBFactory::GetWorkSpaceName());
            DBFactory::Stop ();
        }
        catch (const char* message) {
            fl_beep (FL_BEEP_ERROR);
            fl_alert (message);
        }
    }

    delete trader;
    DBG_PRINT
    DBG_STOP

    #ifdef BUGG
        fl_message ("Data objects in memory=%d", MH::GetReference());
    #endif
    return 0;
}
