/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#ifndef BaseReport_h
#define BaseReport_h

#include <report/Report.h>
#include <report/ReportTypes.h>
#include <MHString.h>
#include <MHVector.h>



/**
*  Base class for reports. All child classes must implement Create.
*/
class BaseReport {
public:
    enum DATE {
                            NO_DATE = 0,
                            START_DATE,
                            STOP_DATE,
                            BOTH_DATE,

    };

                            BaseReport (Report* report, int type = -1);
    virtual                 ~BaseReport () {delete aData; delete aReport;}

    virtual const char*     Class () {return "BaseReport";}
    virtual DATE            ChangeDate () {return NO_DATE;}
    void                    CheckDate ();
    virtual ReportTypes*    Choice () {return &aChoice;}
    virtual void            Create () = 0;
    MH*                     GetData () {return aData;}
    static const char*      GetPrefKey (int type) {return "";}
    Report*                 GetReport () {return aReport;}
    int                     GetReportType () {return aReportType;}
    MHString*               GetStartDate () {return &aStartDate;}
    MHString*               GetStopDate () {return &aStopDate;}
    virtual void            Reload (const char* message){;}
    void                    SetData (MH* data) {delete aData; aData = data;}
    void                    SetReportType (int type);
    void                    SetStartDate (const char* startDate) {aStartDate = startDate;}
    void                    SetStopDate (const char* stopDate) {aStopDate = stopDate;}

protected:
    ReportTypes             aChoice;
    Report*                 aReport;
    MH*                     aData;
    MHString                aStartDate;
    MHString                aStopDate;
    int                     aReportType;
};

#endif
