/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include "report/PortfolioReport.h"
#include "report/PortfolioValue.h"
#include "db/Account.h"
#include "db/Portfolio.h"
#include "db/Portfolio2.h"
#include "Progress.h"
#include "Resource.h"
#include <MHDate.h>
#include <MHDebug.h>
#include <MHString.h>
#include <MHVector.h>



/**
*
*/
PortfolioReport::PortfolioReport (Report* report) :
BaseReport (report) {
    aChoice.aNames.Push (new MHString (STRING(PERFORMANCE_REPORT)));
    aChoice.aTypes.Push (new MHString ((int)PERFORMANCE));

    aChoice.aNames.Push (new MHString (STRING(VALUE_REPORT)));
    aChoice.aTypes.Push (new MHString ((int)VALUE));

    MHDate date;
    date.Set (date.GetYear(), 1, 1);
    SetStartDate (date.Get (MHDate::LONG_DATE));
}



/**
*  Delete all objects.
*/
PortfolioReport::~PortfolioReport () {
    aPortfolio.Erase ();
}



/**
*  Create new price report
*  @param StringVect& - Vector with report strings
*/
void PortfolioReport::Create () {
    DBG_TIME ("PortfolioReport::Create")

    aReport->Erase ();
    aPortfolio.Erase ();
    Portfolio2::LoadPortfolio ((Account*)aData, &aPortfolio);

    switch (aReportType) {
        case PERFORMANCE:
            createPerformance ();
            break;

        case VALUE:
            createValue ();
            break;

        default:
            break;
    }
}



/**
*  Create a net worth report.
*  @param StringVect& - The result vector
*/
void PortfolioReport::createPerformance () {
    DBG ("PortfolioReport::createPerformance")

    Account*        account = (Account*) aData;
    PortfolioValue  value;
    MHVector        buy;
    MHVector        group;

    //  Get portfolio performance first and then current prices for objects that are active in the portfolio
    if (!Portfolio2::Performance (&aPortfolio, &buy, 0, 0, &aStartDate, &aStopDate, &value)) return;
    Portfolio2::GroupTrades (&buy, &group);
    Portfolio2::LoadCurrPrices (aStopDate(), &group);

    aReport->aTitle = new MHString (200, "%s %s %s - %s", STRING(PERFORMANCE_REPORT), account->GetAccount()->Get(), aStartDate(), aStopDate());
    aReport->aHeader.Push (new MHString (30, "%s", STRING(NAME)));
    aReport->aHeader.Push (new MHString (30, "%s", STRING(VOLUME)));
    aReport->aHeader.Push (new MHString (30, "%s", STRING(BUY)));
    aReport->aHeader.Push (new MHString (30, "%s", STRING(CURRENT)));
    aReport->aHeader.Push (new MHString (30, "%s", STRING(DIVIDEND)));
    aReport->aHeader.Push (new MHString (30, "%s", STRING(CHANGE)));
    aReport->aHeader.Push (new MHString (30, "%s", STRING(CHANGE_PROC)));
    aReport->aHeader.Push (new MHString (30, "%s", STRING(TOTAL)));

    aReport->aAlign.Push (new MHString (5, "%c", Report::LEFT));
    aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT));
    aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT));
    aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT));
    aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT | Report::NUMBER));
    aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT | Report::NUMBER));
    aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT | Report::NUMBER));
    aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT));

    for (int f = 0; f < group.Size(); f++) {
        Portfolio2* p2 = (Portfolio2*) group[f];

        MHVector* row = new MHVector ();
        p2->LoadSecurity ();

        row->Push (new MHString (30, "%s", (p2->GetSecurity()) ? p2->GetSecurity()->GetName()->Get() : "STRING(UNKNOWN_NAME)"));
        row->Push (new MHString (30, "%.0f", p2->GetVolume()));
        row->Push (new MHString (30, "%.0f", p2->BuyValue2()));
        row->Push (new MHString (30, "%.0f", p2->Value()));
        row->Push (new MHString (30, "%.0f", p2->DivValue()));
        row->Push (new MHString (30, "%.0f", p2->Value() - p2->BuyValue2()));
        row->Push (new MHString (30, "%.1f%%", MHUtil::Change (p2->BuyValue2(), p2->Value())));
        row->Push (new MHString (30, "%.1f%%", MHUtil::PercentOf (p2->Value(), value.aCurrCash + value.aCurrValue)));

        aReport->aRows.Push (row);
    }

    MHVector* row = new MHVector ();
    row->Push (new MHString (30, "%s", STRING(PORTFOLIO)));
    row->Push (new MHString (30, " "));
    row->Push (new MHString (30, "%.0f", value.aBuyValue2));
    row->Push (new MHString (30, "%.0f", value.aCurrValue));
    row->Push (new MHString (30, "%.0f", value.aDivValue));
    row->Push (new MHString (30, "%.0f", value.aCurrValue - value.aBuyValue2));
    row->Push (new MHString (30, "%.1f%%", MHUtil::Change (value.aBuyValue2, value.aCurrValue)));
    row->Push (new MHString (30, "%.1f%%", MHUtil::PercentOf (value.aCurrValue, value.aCurrCash + value.aCurrValue)));
    aReport->aRows.Push (row);

    row = new MHVector ();
    row->Push (new MHString (30, "%s", STRING(CASH)));
    row->Push (new MHString (30, " "));
    row->Push (new MHString (30, " "));
    row->Push (new MHString (30, "%.0f", value.aCurrCash));
    row->Push (new MHString (30, " "));
    row->Push (new MHString (30, " "));
    row->Push (new MHString (30, " "));
    row->Push (new MHString (30, "%.1f%%", MHUtil::PercentOf (value.aCurrCash, value.aCurrValue + value.aCurrCash)));
    aReport->aRows.Push (row);

    row = new MHVector ();
    row->Push (new MHString (30, "%s", STRING(VALUE)));
    row->Push (new MHString (30, " "));
    row->Push (new MHString (30, " "));
    row->Push (new MHString (30, "%.0f", value.aCurrCash + value.aCurrValue));
    row->Push (new MHString (30, " "));
    row->Push (new MHString (30, "%.0f", value.aCurrCash + value.aCurrValue - value.aNetCash));
    row->Push (new MHString (30, " "));
    row->Push (new MHString (30, "%.1f%%", MHUtil::Change (value.aNetCash, value.aCurrCash + value.aCurrValue)));
    aReport->aRows.Push (row);

    row = new MHVector ();
    row->Push (new MHString (30, "%s", STRING(SHARE_VALUE)));
    row->Push (new MHString (30, " "));
    row->Push (new MHString (30, " "));
    row->Push (new MHString (30, " "));
    row->Push (new MHString (30, " "));
    row->Push (new MHString (30, " "));
    row->Push (new MHString (30, " "));
    row->Push (new MHString (30, "%.1f%%", value.aSharePrice - 100));
    aReport->aRows.Push (row);

    buy.Erase ();
    group.Erase ();
}



/**
*  Create a list with prices from current portfolio
*  @param int         - Type of report (SMS=code price)
*  @param StringVect& -
*/
void PortfolioReport::createValue () {
    DBG ("PortfolioReport::createValue")

    Account* account = (Account*) aData;
    MHVector buy;

    Portfolio2::GroupTrades (&aPortfolio, &buy);
    Portfolio2::LoadCurrPrices (aStopDate(), &buy);

    aReport->aTitle = new MHString (200, "%s %s %s", STRING(VALUE_REPORT), account->GetAccount()->Get(), aStopDate());
    aReport->aHeader.Push (new MHString (30, "%s", STRING(NAME)));
    aReport->aHeader.Push (new MHString (30, "%s", STRING(VOLUME)));
    aReport->aHeader.Push (new MHString (30, "%s", STRING(BUY_VALUE)));
    aReport->aHeader.Push (new MHString (30, "%s", STRING(DATE)));
    aReport->aHeader.Push (new MHString (30, "%s", STRING(LAST)));
    aReport->aHeader.Push (new MHString (30, "%s", STRING(VALUE)));
    aReport->aHeader.Push (new MHString (30, "%s", STRING(CHANGE)));
    aReport->aHeader.Push (new MHString (30, "%s", STRING(CHANGE_PROC)));

    aReport->aAlign.Push (new MHString (5, "%c", Report::LEFT));
    aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT));
    aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT));
    aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT));
    aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT));
    aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT));
    aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT | Report::NUMBER));
    aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT | Report::NUMBER));

    for (int f = 0; f < buy.Size(); f++) {
        Portfolio2* p2  = (Portfolio2*) buy[f];

        if (p2->GetSecurityID() > 0) {
            MHVector* row = new MHVector ();
            p2->LoadSecurity ();
            row->Push (new MHString (30, "%s", (p2->GetSecurity()) ? p2->GetSecurity()->GetName()->Get() : "STRING(UNKNOWN_NAME)"));
            row->Push (new MHString (30, "%.0f", p2->GetVolume()));
            row->Push (new MHString (30, "%.0f", p2->BuyValue2()));
            row->Push (new MHString (30, "%s"  , p2->GetCurrPriceDate()->Get()));
            row->Push (new MHString (30, "%.2f", p2->GetCurrPrice()));
            row->Push (new MHString (30, "%.0f", p2->Value()));
            row->Push (new MHString (30, "%.0f", p2->Value() - (p2->GetPrice() * p2->GetVolume())));
            row->Push (new MHString (30, "%.1f%%", MHUtil::Change (p2->GetPrice() * p2->GetVolume(), p2->Value())));
            aReport->aRows.Push (row);
        }
    }
    buy.Erase ();
}



/**
*  Check is the message is for me and create report again if so
*/
void PortfolioReport::Reload (const char* message) {
    if (*message == '\0' || strcmp (message, "Portfolio") == 0 || strcmp (message, "Price") == 0) {
        Create();
    }
}
