/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <report/PriceReport.h>
#include <db/DBFactory.h>
#include <db/Price.h>
#include <db/Security.h>
#include <db/Set.h>
#include <db/SetItem.h>
#include <Resource.h>
#include <Progress.h>
#include <MHPrice.h>
#include <MHDate.h>
#include <MHDebug.h>



/**
*
*/
PriceReport::PriceReport (Report* report) :
BaseReport (report) {
    aChoice.aNames.Push (new MHString (STRING(PRICE_LIST)));
    aChoice.aTypes.Push (new MHString ((int)QUOTES));
    aChoice.aNames.Push (new MHString (STRING(PRICE_CHANGE_REPORT)));
    aChoice.aTypes.Push (new MHString ((int)CHANGE));
}



/**
*  Can report change date
*  @return DATE - DATE flag
*/
BaseReport::DATE PriceReport::ChangeDate () {
    switch (aReportType) {
        case CHANGE:
            return STOP_DATE;

        case QUOTES:
        case QUOTES_SHORT:
            return BOTH_DATE;

        default:
            return NO_DATE;
    }
}



/**
*  Create new price report
*  @param StringVect& - Vector with report strings
*/
void PriceReport::Create () {
    aReport->Erase ();

    switch (aReportType) {
        case QUOTES:
            createQuoteList (false);
            break;

        case QUOTES_SHORT:
            createQuoteList (true);
            break;

        case CHANGE:
            createChangeList (false);
            break;
    }
}



/**
*  Create a price list, with price changes
*  @param MHVector*  - Vector with report strings
*/
void PriceReport::createChangeList (bool shortReport) {
    DBG ("PriceReport::createChangeList")

    int             f;
    Security*       security;
    Set*            set = (Set*) aData;
    SetItem*        setItem;
    MHVector        setItems;
    MHVector*       prices;
    MHString        year;
    MHDate          date1 (aStopDate());

    //  Set right day of week for all 1 day, 5 days and since start of year for chnages
    if (date1.GetWeekday() == MHDate::SATURDAY || date1.GetWeekday() == MHDate::SUNDAY)
        date1.GotoWeekday (MHDate::FRIDAY);
    date1.Get (MHDate::LONG_DATE, aStopDate);

    //  Create title
    if (shortReport == false) {
        aReport->aTitle = new MHString (100, "%s - %s", STRING(PRICE_CHANGE_REPORT), aStopDate());

        aReport->aHeader.Push (new MHString (50, "%s", STRING(NAME)));
        aReport->aHeader.Push (new MHString (50, "%s", STRING(LAST)));
        aReport->aHeader.Push (new MHString (50, "%s", STRING(CLOSE_)));
        aReport->aHeader.Push (new MHString (50, "%s", STRING(PREV)));
        aReport->aHeader.Push (new MHString (50, "%s", STRING(CLOSE_)));
        aReport->aHeader.Push (new MHString (50, "%s", STRING(CHANGE)));
        aReport->aHeader.Push (new MHString (50, "%s", STRING(CHANGE_PROC)));

        aReport->aHeader.Push (new MHString (50, "%s", STRING(WEEK)));
        aReport->aHeader.Push (new MHString (50, "%s", STRING(CLOSE_)));
        aReport->aHeader.Push (new MHString (50, "%s", STRING(CHANGE)));
        aReport->aHeader.Push (new MHString (50, "%s", STRING(CHANGE_PROC)));

        aReport->aHeader.Push (new MHString (50, "%s", STRING(YEAR)));
        aReport->aHeader.Push (new MHString (50, "%s", STRING(CLOSE_)));
        aReport->aHeader.Push (new MHString (50, "%s", STRING(CHANGE)));
        aReport->aHeader.Push (new MHString (50, "%s", STRING(CHANGE_PROC)));

        aReport->aAlign.Push (new MHString (5, "%c", Report::LEFT));
        aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT));
        aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT | Report::NUMBER));
        aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT));
        aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT | Report::NUMBER));
        aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT | Report::NUMBER));
        aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT | Report::NUMBER));

        aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT));
        aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT | Report::NUMBER));
        aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT | Report::NUMBER));
        aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT | Report::NUMBER));

        aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT));
        aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT | Report::NUMBER));
        aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT | Report::NUMBER));
        aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT | Report::NUMBER));
    }
    else {
        aReport->aTitle = new MHString (50, "%s %s", set->GetSet()->Get(), aStopDate());
    }

    //  Load set and prices and create one row for every security
    DBFactory::LoadSetItem (set, &setItems);

    xtrader_reset (setItems.Size());
    SetItem::LoadSecurities (&setItems);
    setItems.Sort (SetItem::SORT_SECURITY);

    Price* p1 = 0;
    Price* p2 = 0;
    Price* p3 = 0;
    Price* p4 = 0;
    MHPrice p11 (aStopDate(), 0, 0, 0, 0, 0), p22, p33, p44;
    for (f = 0; f < setItems.Size(); f++) {
        setItem  = (SetItem*) setItems[f];
        security = setItem->LoadSecurity();

        if (security) {
            prices = DBFactory::LoadPrice (security);

            if (prices->Size() > 0) {
                p1 = p2 = p3 = p4 = 0;
                p1 = (Price*) prices->Find (&p11, MHVector::FIND_BEFORE, Price::SORT_DATE, 0);
                if (p1) {
                    DBG_LOG2 ("searchdate=%s, founddate=%s", p11.aDate, p1->aDate)
                    MHDate tmp (p1->aDate);
                    tmp.AddDays (-1);
                    p22.SetDate (tmp.Get (MHDate::LONG_DATE));

                    tmp.AddDays (-4);
                    p33.SetDate (tmp.Get (MHDate::LONG_DATE));

                    MHDate tmp2 (tmp.GetYear(), 1, 1);
                    p44.SetDate (tmp2.Get (MHDate::LONG_DATE));

                    p2 = (Price*) prices->Find (&p22, MHVector::FIND_BEFORE, Price::SORT_DATE, 0);
                    p3 = (Price*) prices->Find (&p33, MHVector::FIND_BEFORE, Price::SORT_DATE, 0);
                    p4 = (Price*) prices->Find (&p44, MHVector::FIND_BEFORE, Price::SORT_DATE, 0);
                }
                aReport->aRows.Push (createChangeRow (shortReport, security->GetName()->Get(), p1, p2 ,p3, p4));

            }
        }

        xtrader_add(1);
        if (xtrader_abort()) break;
    }

    setItems.Erase ();
}



/**
*  Create one price change row
*  @param bool        - True to make a "small" report
*  @param const char* - Security name
*  @param Price*      - Current
*  @param Price*      - 1 day
*  @param Price*      - Week gao
*  @param Price*      - Since start of this year
*  @return MHVector*  - New report row
*/
MHVector* PriceReport::createChangeRow (bool shortReport, const char* name, Price* p1, Price* p2, Price* p3, Price* p4) {
    MHVector* row = new MHVector ();

    if (shortReport == false) {
        row->Push (new MHString (50, "%s", name));

        if (p1) {
            row->Push (new MHString (50, "%s", p1->aDate));
            row->Push (new MHString (50, "%.2f", p1->aClose));
        }
        else {
            row->Push (new MHString (""));
            row->Push (new MHString (""));
        }

        if (p2) {
            row->Push (new MHString (50, "%s", p2->aDate));
            row->Push (new MHString (50, "%.2f", p2->aClose));
        }
        else {
            row->Push (new MHString (""));
            row->Push (new MHString (""));
        }

        //  Create change columns
        if (p1 && p2) {
            row->Push (new MHString (50, "%.2f", p1->aClose - p2->aClose));
            row->Push (new MHString (50, "%.2f", MHUtil::Change (p2->aClose, p1->aClose)));
        }
        else {
            row->Push (new MHString (""));
            row->Push (new MHString (""));
        }

        //  Create week change columns
        if (p3) {
            row->Push (new MHString (50, "%s", p3->aDate));
            row->Push (new MHString (50, "%.2f", p3->aClose));
        }
        else {
            row->Push (new MHString (""));
            row->Push (new MHString (""));
        }

        if (p1 && p3) {
            row->Push (new MHString (50, "%.2f", p1->aClose - p3->aClose));
            row->Push (new MHString (50, "%.2f", MHUtil::Change (p3->aClose, p1->aClose)));
        }
        else {
            row->Push (new MHString (""));
            row->Push (new MHString (""));
        }

        //  Create year change columns
        if (p4) {
            row->Push (new MHString (50, "%s", p4->aDate));
            row->Push (new MHString (50, "%.2f", p4->aClose));
        }
        else {
            row->Push (new MHString (""));
            row->Push (new MHString (""));
        }

        if (p1 && p4) {
            row->Push (new MHString (50, "%.2f", p1->aClose - p4->aClose));
            row->Push (new MHString (50, "%.2f", MHUtil::Change (p4->aClose, p1->aClose)));
        }
        else {
            row->Push (new MHString (""));
            row->Push (new MHString (""));
        }
    }
    else {
    }

    return row;
}



/**
*  Create a price list
*/
void PriceReport::createQuoteList (bool shortReport) {
    int             f, i;
    Price*          price;
    Security*       security;
    Set*            set = (Set*) aData;
    SetItem*        setItem;
    MHVector        dates;
    MHVector        closed;
    MHVector        setItems;
    MHVector*       prices;
    MHVector*       row;

    if (aStopDate == aStartDate) {
        MHDate newdate (aStopDate());
        newdate.AddDays (-DBFactory::GetPrefIntValue (STRING(PREF_PRICEREPORT1), 2, 1000, PriceReport::QUOTES_DEFAULT));
        newdate.Get (MHDate::LONG_DATE, aStartDate);
    }

    //  Create title
    if (shortReport == false) {
        aReport->aTitle = new MHString (100, "%s %s %s - %s", STRING(PRICE_LIST), set->GetSet()->Get(), aStartDate(), aStopDate());
        aReport->aHeader.Push (new MHString (50, "%s", STRING(NAME)));
        aReport->aAlign.Push (new MHString (5, "%c", Report::LEFT));
        MHDate::CreateTimeSerie (aStartDate(), aStopDate(), MHDate::WEEKDAY, &closed, &dates);

        //  Create dates to use as a header
        for (f = dates.Size() - 1; f >= 0; f--) {
            MHString* date = (MHString*) dates[f];
            aReport->aHeader.Push (new MHString (14, "%s", date->Get()));
            aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT));
            aReport->aAlign.Push (new MHString (5, "%c", Report::RIGHT));
        }
    }
    else {
        MHDate::CreateTimeSerie (aStopDate(), aStopDate(), MHDate::WEEKDAY, &closed, &dates);
        aReport->aTitle = new MHString (50, "%s %s", set->GetSet()->Get(), aStopDate());
    }

    //  Load set and prices and create one row for every security
    DBFactory::LoadSetItem (set, &setItems);

    xtrader_reset (setItems.Size());
    SetItem::LoadSecurities (&setItems);
    setItems.Sort (SetItem::SORT_SECURITY);

    for (f = 0; f < setItems.Size(); f++) {
        setItem  = (SetItem*) setItems[f];
        security = setItem->LoadSecurity();
        row      = new MHVector ();

        if (security) {
            prices = DBFactory::LoadPrice (security);

            if (shortReport == false)
                row->Push (new MHString (50, "%s", security->GetName()->Get()));
            else
                row->Push (new MHString (50, "%s ", security->GetCode()->Get()));

            for (i = dates.Size() - 1; i >= 0; i--) {
                MHString* date  = (MHString*) dates[i];
                MHPrice p1(date->Get(), 0, 0, 0, 0, 0);
                price = (Price*) prices->Find (&p1, 0, 0, 0);

                if (price)
                    row->Push (new MHString (20, "%.2f", price->aClose));
                else
                    row->Push (new MHString ());
            }
        }
        aReport->aRows.Push (row);
        xtrader_add(1);
        if (xtrader_abort()) break;
    }

    setItems.Erase ();
    dates.Erase ();
}



/**
*  Check is the message is for me and create report again if so
*/
void PriceReport::Reload (const char* message) {
    if (*message == '\0' || strcmp (message, "Price") == 0 || strcmp (message, "Security") == 0) {
        Create();
    }
}

