/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <table/AccountData.h>
#include <db/Account.h>
#include <db/DBFactory.h>
#include <Currency.h>
#include <Resource.h>
#include <FL/Fl.h>



/**
*
*/
AccountData::AccountData () :
BaseData () {
    aVector = new MHVector ();
    Account::Load (aVector);
    aVector->Sort (Account::SORT_NAME);

    aChoice2 = MHUtil::Allocate (CURRENCY_LIST_SIZE, 50);

    for (int f = 0; f < CURRENCY_LIST_SIZE; f++)
        STRNCPY (aChoice2[f], Currency[f][1], 49)
}



/**
*
*/
AccountData::~AccountData () {
    aVector->Erase ();
    delete aVector;
    MHUtil::Delete (aChoice2, CURRENCY_LIST_SIZE);
}



/**
*
*/
void AccountData::add () {
    insert (aVector->Size());
}



/**
*
*/
Fl_Align AccountData::align (int row, int col) {
    switch (col) {
        case 1:
            return FL_ALIGN_RIGHT;

        default:
            return FL_ALIGN_LEFT;
    }
}



/**
*
*/
const char** AccountData::choice (int row, int col, int& size, int& selected, char* label) {
    size     = 2;
    selected = 0;

    switch (col) {
        case 2: {
            Account* a = (Account*) aVector->Get (row);
            size     = CURRENCY_LIST_SIZE;
            selected = 0;

            for (int f = 0; a && f < CURRENCY_LIST_SIZE; f++) {
                if (strncasecmp (aChoice2[f], a->GetCurrency()->Get(), 3) == 0) {
                    selected = f;
                    break;
                }
            }
            strncpy (label, STRING(SELECT_CURRENCY), 99);
            return (char const**) aChoice2;
        }
    }

    return 0;
}



/**
*
*/
FL_TABLE_EDITOR AccountData::editor_type (int row, int col, bool force_custom) {
    switch (col) {
        case 1:
            return FL_FLOAT_EDITOR;

        case 2:
            return FL_DLG_LIST_EDITOR;

        default:
            return FL_TEXT_EDITOR;
    }
}



/**
*
*/
void AccountData::erase (int row) {
    if (row >= 0 && row < aVector->Size()) {
        aVector->Erase (row);
        dirty (true);
    }
}



/**
*
*/
void AccountData::insert (int row) {
    Account* a1 = new Account (DBFactory::GetUniqueID(), "", 0, "");
    Account* a2 = (Account*) aVector->Search (a1, MHVector::FIND_EXACT, Account::SORT_NAME, 0);

    if (a2) {
        delete a1;
        XTrader::SetError (STRING(ERROR_DATA_ERROR));
        return;
    }
    aVector->Insert (a1, row);
    dirty (true);
}



/**
*
*/
int AccountData::save () {
    Account::Save (aVector);
    DBFactory::ClearAccount();
    aVector->Sort (Account::SORT_NAME);
    dirty (false);
    XTrader::SetInfo (STRING(DATA_SAVED));
    XTrader::SendMessage (0, "Account", "changed");
    return -1;
}



/**
*
*/
void AccountData::sort (int col, bool ascending) {
}



/**
*
*/
bool AccountData::value (const char* data, int row, int col) {
    Account* a = (Account*) aVector->Get (row);

    if (a) {
        switch (col) {
            case 0: {
                if (strcmp (a->GetAccount()->Get(), data) == 0) return true;
                Account a2(0, data);
                if (aVector->Search (&a2, MHVector::FIND_EXACT, Account::SORT_NAME, 0) != 0)
                    return false;
                a->SetAccount (data);
                break;
            }

            case 1:
                a->SetBalance (atof (data));
                break;

            case 2:
                for (int f = 0; f < CURRENCY_LIST_SIZE; f++) {
                    if (strcmp (Currency[f][1], data) == 0) {
                        if (strcmp (a->GetCurrency()->Get(), Currency[f][0]) == 0) return true;
                        a->SetCurrency (Currency[f][0]);
                    }
                }
                break;

        }
    }

    dirty (true);
    return true;
}



/**
*
*/
const char* AccountData::value (int row, int col) {
    if (row == FL_LABEL_ROW)
        return "";
    else if (row == FL_HEADER_ROW) {
        switch (col) {
            case 0:
                return STRING(ACCOUNT);

            case 1:
                return STRING(BALANCE);

            case 2:
                return STRING(CURRENCY);
        }
    }
    else {
        Account* a = (Account*) aVector->Get (row);
        if (!a) return "";

        switch (col) {
            case 0:
                return a->GetAccount()->Get();

            case 1:
                snprintf (aBuff, 30, "%.2f", a->GetBalance());
                return aBuff;

            case 2:
                return a->GetCurrency()->Get();
        }
    }

    return "";
}



/**
* Return prefered column width
*/
int AccountData::width (int col) {
    return XTrader::FontSize() * 8;
}
