/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <table/GuardData.h>
#include <db/Guard.h>
#include <db/Security.h>
#include <db/DBFactory.h>
#include <Resource.h>
#include <FL/Fl.h>



/**
*
*/
GuardData::GuardData () :
BaseData () {
    show_label(false);

    aVector         = new MHVector ();
    aVectorSecurity = new MHVector ();

    Guard::Load (aVector);
    Guard::LoadSecurities (aVector);
    reload ("Security");

    aChoice2 = MHUtil::Allocate (4, 31);
    STRNCPY (aChoice2[0], STRING(SIG_BUY_HIGH),  30)
    STRNCPY (aChoice2[1], STRING(SIG_BUY_LOW),   30)
    STRNCPY (aChoice2[2], STRING(SIG_SELL_HIGH), 30)
    STRNCPY (aChoice2[3], STRING(SIG_SELL_LOW),  30)
    //STRNCPY (aChoice2[2], STRING(SIG_BUY_MVG), 30)
    //STRNCPY (aChoice2[5], STRING(SIG_SELL_MVG), 30)
}



/**
*
*/
GuardData::~GuardData () {
    if (aVectorSecurity->Size() > 0)
        MHUtil::Delete (aChoice, aVectorSecurity->Size());
    MHUtil::Delete (aChoice2, 4);
    aVector->Erase ();
    aVectorSecurity->Erase ();
    delete aVector;
    delete aVectorSecurity;
}



/**
*
*/
void GuardData::add () {
    insert (aVector->Size());
}



/**
*
*/
Fl_Align GuardData::align (int row, int col) {
    switch (col) {
        case 2:
            return FL_ALIGN_RIGHT;

        default:
            return FL_ALIGN_LEFT;
    }
}



/**
*
*/
const char** GuardData::choice (int row, int col, int& size, int& selected, char* label) {
    Guard*   g = (Guard*) aVector->Get (row);
    selected = size = 0;

    switch (col) {
        case 0: {
            size = aVectorSecurity->Size();
            for (int f = 0; f < aVectorSecurity->Size(); f++) {
                Security* s = (Security*) (*aVectorSecurity)[f];
                if (g->GetSecurityID() == s->GetID()) {
                    selected = f;
                    break;
                }
            }
            strncpy (label, STRING(SELECT_SECURITY), 99);
            return (char const**) aChoice;
        }

        case 1:
            size = 4;
            if (g->GetType() >= 0 && g->GetType() < Guard::NO_TYPES)
                selected = g->GetType();
            return (char const**) aChoice2;

        default:
            return 0;
    }
}



/**
*
*/
FL_TABLE_EDITOR GuardData::editor_type (int row, int col, bool force_custom) {
    switch (col) {
        case 0:
            return FL_DLG_LIST_EDITOR;

        case 1:
            return FL_LIST_EDITOR;

        default:
            return FL_FLOAT_EDITOR;
    }
}



/**
*
*/
void GuardData::erase (int row) {
    if (row >= 0 && row < aVector->Size()) {
        aVector->Erase (row);
        dirty (true);
    }
}



/**
*
*/
void GuardData::insert (int row) {
    Guard* g1 = new Guard (DBFactory::GetUniqueID(), Guard::BUY_HIGH, 0.0);
    aVector->Insert (g1, row);
    dirty (true);
}



/**
*
*/
void GuardData::reload (const char* message) {
    if (strcmp (message, "Security") == 0) {
        if (aVectorSecurity->Size() > 0)
            MHUtil::Delete (aChoice, aVectorSecurity->Size());
        aVectorSecurity->Erase ();
        Security::Load (aVectorSecurity);
        aVectorSecurity->Sort (Security::SORT_NAME);

        aChoice = MHUtil::Allocate (aVectorSecurity->Size(), 31);

        for (int f = 0; f < aVectorSecurity->Size(); f++) {
            Security* s = (Security*) (*aVectorSecurity)[f];
            STRNCPY (aChoice[f], s->GetName()->Get(), 30)
        }
    }
}



/**
*
*/
int GuardData::save () {
    Guard::Save (aVector);
    DBFactory::ClearGuard();
    dirty (false);
    XTrader::SetInfo (STRING(DATA_SAVED));
    XTrader::SendMessage (0, "Guard", "changed");
    return -1;
}



/**
*
*/
void GuardData::sort (int col, bool ascending) {
}



/**
*
*/
bool GuardData::value (const char* data, int row, int col) {
    Guard* g = (Guard*) aVector->Get (row);

    if (g) {
        switch (col) {
            case 0: {
                for (int f = 0; f < aVectorSecurity->Size(); f++) {
                    Security* s = (Security*) (*aVectorSecurity)[f];

                    if (strcmp (s->GetName()->Get(), data) == 0) {
                        g->SetSecurity (s);
                        break;
                    }
                }
                break;
            }

            case 1: {
                for (int f = 0; f < 4; f++) {
                    if (strcmp (aChoice2[f], data) == 0) {
                        g->SetType ((Guard::TYPE) f);
                        break;
                    }
                }
                break;
            }

            case 2:
                g->SetValue (atof (data));
                break;
        }
    }

    dirty (true);
    return true;
}



/**
*
*/
const char* GuardData::value (int row, int col) {
    if (row == FL_LABEL_ROW)
        return "";
    else if (row == FL_HEADER_ROW) {
        switch (col) {
            case 0:
                return STRING(SECURITY);

            case 1:
                return STRING(SIGNAL);

            case 2:
                return STRING(LIMIT);
        }
    }
    else {
        Guard* g = (Guard*) aVector->Get (row);
        if (!g) return "";

        switch (col) {
            case 0:
                return g->GetSecurity()->GetName()->Get();

            case 1:
                return g->GetTypeName();

            case 2:
                snprintf (aBuff, 30, "%.2f", g->GetValue());
                return aBuff;
        }
    }

    return "";
}



/**
*
*/
int GuardData::width (int col) {
    switch (col) {
        case 0:
        case 1:
            return XTrader::FontSize() * 8;

        default:
            return XTrader::FontSize() * 6;
    }
}
