/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <table/PrefGeneralData.h>
#include <db/DBFactory.h>
#include <db/Pref.h>
#include <report/PriceReport.h>
#include <Enums.h>
#include <Resource.h>
#include <FL/Fl.h>



/**
*
*/
PrefGeneralData::PrefGeneralData () :
BaseData () {
    aChoiceYesNo = MHUtil::Allocate (2, 30);
    aChoiceTheme = MHUtil::Allocate (XTRADER_LAST_THEME, 30);
    aChoiceFont  = MHUtil::Allocate (XTRADER_LAST_FONT, 30);

    STRNCPY (aChoiceYesNo[0], STRING(YES), 29)
    STRNCPY (aChoiceYesNo[1], STRING(NO), 29)

    STRNCPY (aChoiceTheme[0], STRING(THEME_PLASTIC), 29)
    STRNCPY (aChoiceTheme[1], STRING(THEME_FLTK), 29)

    STRNCPY (aChoiceFont[0], "10", 29)
    STRNCPY (aChoiceFont[1], "12", 29)
    STRNCPY (aChoiceFont[2], "14", 29)
}



/**
*
*/
PrefGeneralData::~PrefGeneralData () {
    MHUtil::Delete (aChoiceYesNo, 2);
    MHUtil::Delete (aChoiceTheme, XTRADER_LAST_THEME);
    MHUtil::Delete (aChoiceFont,  XTRADER_LAST_FONT);
}



/**
*
*/
Fl_Align PrefGeneralData::align (int row, int col) {
    return FL_ALIGN_LEFT;
}



/**
*
*/
const char** PrefGeneralData::choice (int row, int col, int& size, int& selected, char* label) {
    size = selected = 0;

    if (row == 0) {
        size     = XTRADER_LAST_THEME;
        selected = DBFactory::GetPrefIntValue (STRING(PREF_THEME), 0, XTRADER_LAST_THEME - 1, 0);
        return (const char**) aChoiceTheme;
    }
    else if (row == 2) {
        size     = 3;
        selected = DBFactory::GetPrefIntValue (STRING(PREF_FONTSIZE), 0, XTRADER_LAST_FONT, 1);
        return (const char**) aChoiceFont;
    }
    else if (row == 3) {
        size     = 2;
        selected = (DBFactory::GetPrefIntValue (STRING(PREF_AUTORELOAD), XTRADER_NO, XTRADER_YES, XTRADER_YES) == XTRADER_NO) ? 1 : 0;
        return (const char**) aChoiceYesNo;
    }
    else if (row == 6) {
        size     = 2;
        selected = (DBFactory::GetPrefIntValue (STRING(PREF_COMPRESS), XTRADER_NO, XTRADER_YES, XTRADER_NO) == XTRADER_NO) ? 1 : 0;
        return (const char**) aChoiceYesNo;
    }
    return 0;
}



/**
*
*/
FL_TABLE_EDITOR PrefGeneralData::editor_type (int row, int col, bool force_custom) {
    if (col == 1) {
        switch (row) {
            case 0:
            case 2:
            case 3:
            case 6:
                return FL_LIST_EDITOR;

            case 1:
                return FL_DLG_COLOR_EDITOR;

            default:
                return FL_INTEGER_EDITOR;
        }
    }
    return (FL_TABLE_EDITOR)0;
}



/**
*
*/
bool PrefGeneralData::value (const char* data, int row, int col) {
    if (row == 0) {
        for (int f = 0; f < XTRADER_LAST_THEME; f++) {
            if (strcmp (data, aChoiceTheme[f]) == 0) {
                DBFactory::SetPrefIntValue (STRING(PREF_THEME), 0, XTRADER_LAST_THEME - 1, 0, f);
                return true;
            }
        }
    }
    else if (row == 1)
        DBFactory::SetPrefIntValue (STRING(PREF_BCOLOR), 0, 255, 0, atoi (data));
    else if (row == 2) {
        if (strcmp (data, "10") == 0)
            DBFactory::SetPrefIntValue (STRING(PREF_FONTSIZE), 0, XTRADER_LAST_FONT, 1, 0);
        else if (strcmp (data, "14") == 0)
            DBFactory::SetPrefIntValue (STRING(PREF_FONTSIZE), 0, XTRADER_LAST_FONT, 1, 2);
        else
            DBFactory::SetPrefIntValue (STRING(PREF_FONTSIZE), 0, XTRADER_LAST_FONT, 1, 1);
    }
    else if (row == 3)
        DBFactory::SetPrefIntValue (STRING(PREF_AUTORELOAD), XTRADER_NO, XTRADER_YES, XTRADER_YES, (strcmp(data, STRING(YES)) == 0) ? XTRADER_YES : XTRADER_NO);
    else if (row == 4)
        DBFactory::SetPrefIntValue (STRING(PREF_CACHE_SIZE), 5, 100, 25, atoi (data));
    else if (row == 5)
        DBFactory::SetPrefIntValue (STRING(PREF_CACHE_TIMEOUT), 60, 3600, 300, atoi (data));
    else if (row == 6)
        DBFactory::SetPrefIntValue (STRING(PREF_COMPRESS), XTRADER_NO, XTRADER_YES, XTRADER_NO, (strcmp(data, STRING(YES)) == 0) ? XTRADER_YES : XTRADER_NO);
    return true;
}



/**
*
*/
const char* PrefGeneralData::value (int row, int col) {
    *aBuffer = '\0';

    if (row == FL_LABEL_ROW)
        return "";
    else if (row == FL_HEADER_ROW) {
        switch (col) {
            case 0:
                return STRING(PROPERTY);

            case 1:
                return STRING(VALUE);
        }
    }
    else {
        switch (row) {
            case 0:
                //  Theme
                if (col == 0)
                    snprintf (aBuffer, 999, "%s", STRING(THEME));
                else
                    return aChoiceTheme[DBFactory::GetPrefIntValue (STRING(PREF_THEME), 0, XTRADER_LAST_THEME - 1, 0)];
                break;

            case 1:
                // Button Color
                if (col == 0)
                    snprintf (aBuffer, 999, "%s", STRING(BUTTON_COLOR));
                else
                    snprintf (aBuffer, 999, "%d", DBFactory::GetPrefIntValue (STRING(PREF_BCOLOR), 0, 255, 0));
                break;

            case 2:
                // Font size
                if (col == 0)
                    snprintf (aBuffer, 999, "%s", STRING(FONT_SIZE));
                else {
                    switch (DBFactory::GetPrefIntValue (STRING(PREF_FONTSIZE), 0, XTRADER_LAST_FONT, XTRADER_FONT12)) {
                        case 0:
                            return "10";
                        case 2:
                            return "14";
                        default:
                            return "12";
                    }
                }
                break;

            case 3:
                // Should data be autoreloaded when changes are sent out to all views
                if (col == 0)
                    snprintf (aBuffer, 999, "%s", STRING(AUTORELOAD));
                else {
                    if (DBFactory::GetPrefIntValue (STRING(PREF_AUTORELOAD), XTRADER_NO, XTRADER_YES, XTRADER_YES) == XTRADER_YES)
                        return STRING(YES);
                    else
                        return STRING(NO);
                }
                break;

            case 4:
                // Cache size, no of objects
                if (col == 0)
                    snprintf (aBuffer, 999, "%s (5 - 100)", STRING(CACHE_SIZE));
                else
                    snprintf (aBuffer, 999, "%d", DBFactory::GetPrefIntValue (STRING(PREF_CACHE_SIZE), 5, 100, 25));
                break;

            case 5:
                // Cache size, no of objects
                if (col == 0)
                    snprintf (aBuffer, 999, "%s (60 - 3600)", STRING(CACHE_TIMEOUT));
                else
                    snprintf (aBuffer, 999, "%d", DBFactory::GetPrefIntValue (STRING(PREF_CACHE_TIMEOUT), 60, 3600, 300));
                break;

            case 6:
                // Compress database
                if (col == 0)
                    snprintf (aBuffer, 999, "%s", STRING(COMPRESS_DB));
                else {
                    if (DBFactory::GetPrefIntValue (STRING(PREF_COMPRESS), XTRADER_NO, XTRADER_YES, XTRADER_NO) == XTRADER_YES)
                        return STRING(YES);
                    else
                        return STRING(NO);
                }
                break;
        }
    }

    return aBuffer;
}



/**
*
*/
int PrefGeneralData::width (int col) {
    if (col == 0)
        return 230;
    else
        return 320;
}
