/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include "table/PrefPortfolioChartData.h"
#include "db/DBFactory.h"
#include "db/Pref.h"
#include "db/Security.h"
#include "chart/PortfolioChart.h"
#include "Enums.h"
#include "Resource.h"
#include <MHPrice.h>
#include <FL/Fl.h>
#include <math.h>



/**
*  Edit portfolio chart options
*/
PrefPortfolioChartData::PrefPortfolioChartData (int view) :
BaseData (),
aModels (0, -1) {
    int f;

    PortfolioChart p (0, -1);
    ReportTypes*   types = &p.aChoice;

    aView           = view;
    aChoiceLayout   = MHUtil::Allocate (5, 40);
    aChoiceModels   = MHUtil::Allocate (aModels.aChoice.aNames.Size(), 30);
    aChoiceSecurity = 0;
    aVectorSecurity = new MHVector ();

    STRNCPY (aChoiceLayout[0], STRING(LAYOUT_1), 29)
    STRNCPY (aChoiceLayout[1], STRING(LAYOUT_2EQUAL), 29)
    STRNCPY (aChoiceLayout[2], STRING(LAYOUT_2), 29)
    STRNCPY (aChoiceLayout[3], STRING(LAYOUT_3EQUAL), 29)
    STRNCPY (aChoiceLayout[4], STRING(LAYOUT_3), 29)

    for (f = 0; f < aModels.aChoice.aNames.Size(); f++) {
        MHString* s = (MHString*) types->aNames[f];
        STRNCPY (aChoiceModels[f], s->Get(), 29)
    }
    reload ("Security");
}



/**
*
*/
PrefPortfolioChartData::~PrefPortfolioChartData () {
    MHUtil::Delete (aChoiceLayout, 5);
    MHUtil::Delete (aChoiceModels,  aModels.aChoice.aNames.Size());
    if (aChoiceSecurity) MHUtil::Delete (aChoiceSecurity, aVectorSecurity->Size() + 1);

    aVectorSecurity->Erase ();
    delete aVectorSecurity;
}



/**
*
*/
Fl_Align PrefPortfolioChartData::align (int row, int col) {
    return FL_ALIGN_LEFT;
}



/**
*
*/
const char** PrefPortfolioChartData::choice (int row, int col, int& size, int& selected, char* label) {
    size = selected = 0;

    if (row == 0) {
        snprintf (aBuffer, 99, "%s_%d", STRING(PREF_CHARTP_LAYOUT), aView);
        size     = 5;
        selected = DBFactory::GetPrefIntValue (aBuffer, XTRADER_DEFAULT_LAYOUT, XTRADER_LAST_LAYOUT, XTRADER_DEFAULT_LAYOUT);
        return (const char**) aChoiceLayout;
    }
    else if (row >= 1 && row <= 6) {
        setPreference (row, col);
        size     = aModels.aChoice.aNames.Size();
        selected = 0;
        int tmp  = DBFactory::GetPrefIntValue (aBuffer, PortfolioChart::NO_CHART, PortfolioChart::LAST_CHART, PortfolioChart::NO_CHART);
        for (int f = 0; f < aModels.aChoice.aTypes.Size(); f++) {
            int selected2 = ((MHString*)aModels.aChoice.aTypes[f])->ToInt();
            if (tmp == selected2) {
                selected = f;
                break;
            }
        }
        return (const char**) aChoiceModels;
    }
    else if (row == 11) {
        int sec = DBFactory::GetPrefIntValue (STRING(PREF_CHARTP_RELATIVE), aView, 0, 999999999, 0);

        size     = aVectorSecurity->Size();
        selected = 0;

        for (int f = 0; f < aVectorSecurity->Size(); f++) {
            Security* s = (Security*) (*aVectorSecurity)[f];
            if (s->GetID() == sec) {
                selected = f;
                break;
            }
        }
        strncpy (label, STRING(SELECT_SECURITY), 99);
        return (const char**) aChoiceSecurity;
    }
    return 0;
}



/**
*
*/
FL_TABLE_EDITOR PrefPortfolioChartData::editor_type (int row, int col, bool force_custom) {
    if (col == 1) {
        switch (row) {
            case 7:
            case 8:
            case 9:
                return FL_INTEGER_EDITOR;

            case 10:
                return FL_DLG_COLOR_EDITOR;

            case 11:
                return FL_DLG_LIST_EDITOR;

            default:
                return FL_LIST_EDITOR;
        }
    }
    return (FL_TABLE_EDITOR)0;
}



/**
*
*/
void PrefPortfolioChartData::reload (const char* message) {
    if (strcmp (message, "Security") == 0) {
        if (aChoiceSecurity) MHUtil::Delete (aChoiceSecurity, aVectorSecurity->Size() + 1);
        aVectorSecurity->Erase ();
        Security::Load (aVectorSecurity);
        aVectorSecurity->Sort (Security::SORT_NAME);

        aChoiceSecurity = MHUtil::Allocate (aVectorSecurity->Size() + 1, 31);
        for (int f = 0; f < aVectorSecurity->Size(); f++) {
            Security* s = (Security*) (*aVectorSecurity)[f];
            STRNCPY (aChoiceSecurity[f], s->GetName()->Get(), 30)
        }
    }
}



/**
* Create label for chart area
* @param int - Row
*/
void PrefPortfolioChartData::setLabel (int row) {
    if (row == 1)
        snprintf (aBuffer, 999, "%s", STRING(CHART_AREA_TOP));
    else if (row == 2)
        snprintf (aBuffer, 999, "%s", STRING(CHART_AREA_TOP2));
    else if (row == 3)
        snprintf (aBuffer, 999, "%s", STRING(CHART_AREA_MIDDLE));
    else if (row == 4)
        snprintf (aBuffer, 999, "%s", STRING(CHART_AREA_MIDDLE2));
    else if (row == 5)
        snprintf (aBuffer, 999, "%s", STRING(CHART_AREA_BOTTOM));
    else if (row == 6)
        snprintf (aBuffer, 999, "%s", STRING(CHART_AREA_BOTTOM2));
}



/**
* Create the window key to get or set chart type value for window area
* @param int - Row
* @param int - Column, (chart num really, every left/right area has max two different charts)
*/
void PrefPortfolioChartData::setPreference (int row, int col) {
    if (row == 1)
        snprintf (aBuffer, 99, "%s_%d_%d_%d", STRING(PREF_CHARTP_WINDOW), aView, XTRADER_WINDOW_TOP, col);
    else if (row == 2)
        snprintf (aBuffer, 99, "%s_%d_%d_%d", STRING(PREF_CHARTP_WINDOW), aView, XTRADER_WINDOW_TOP2, col);
    else if (row == 3)
        snprintf (aBuffer, 99, "%s_%d_%d_%d", STRING(PREF_CHARTP_WINDOW), aView, XTRADER_WINDOW_MIDDLE, col);
    else if (row == 4)
        snprintf (aBuffer, 99, "%s_%d_%d_%d", STRING(PREF_CHARTP_WINDOW), aView, XTRADER_WINDOW_MIDDLE2, col);
    else if (row == 5)
        snprintf (aBuffer, 99, "%s_%d_%d_%d", STRING(PREF_CHARTP_WINDOW), aView, XTRADER_WINDOW_BOTTOM, col);
    else if (row == 6)
        snprintf (aBuffer, 99, "%s_%d_%d_%d", STRING(PREF_CHARTP_WINDOW), aView, XTRADER_WINDOW_BOTTOM2, col);
}



/**
*
*/
bool PrefPortfolioChartData::value (const char* data, int row, int col) {
    if (row == 0) {
        for (int f = 0; f <= XTRADER_LAST_LAYOUT; f++) {
            if (strcmp (data, aChoiceLayout[f]) == 0) {
                DBFactory::SetPrefIntValue (STRING(PREF_CHARTP_LAYOUT), aView, XTRADER_DEFAULT_LAYOUT, XTRADER_LAST_LAYOUT, XTRADER_DEFAULT_LAYOUT, f);
                break;
            }
        }
    }
    else if (row >= 1 && row <= 6) {
        setPreference (row, col);

        for (int f = 0; f < aModels.aChoice.aNames.Size(); f++) {
            MHString* s = (MHString*) aModels.aChoice.aNames[f];
            if (strcmp (data, s->Get()) == 0) {
                DBFactory::SetPrefIntValue (aBuffer, PortfolioChart::NO_CHART, PortfolioChart::LAST_CHART, PortfolioChart::NO_CHART, ((MHString*)aModels.aChoice.aTypes[f])->ToInt());
                break;
            }
        }
    }
    else if (row == 7)
        DBFactory::SetPrefIntValue (STRING(PREF_CHARTP_VSPACE), aView, XTRADER_MIN_VSPACE, XTRADER_MAX_VSPACE, XTRADER_DEFAULT_VSPACE, atoi (data));
    else if (row == 8)
        DBFactory::SetPrefIntValue (STRING(PREF_CHARTP_BSPACE), aView, XTRADER_MIN_BSPACE, XTRADER_MAX_BSPACE, XTRADER_DEFAULT_BSPACE, atoi (data));
    else if (row == 9)
        DBFactory::SetPrefIntValue (STRING(PREF_CHARTP_LINEWIDTH), aView, XTRADER_MIN_LINE_WIDTH, XTRADER_MAX_LINE_WIDTH, XTRADER_DEFAULTP_LINE_WIDTH, atoi (data));
    else if (row == 10)
        DBFactory::SetPrefIntValue (STRING(PREF_CHARTP_BGCOLOR), aView, 0, 255, 0, atoi (data));
    else if (row == 11) {
        for (int f = 0; f < aVectorSecurity->Size(); f++) {
            Security* s = (Security*) (*aVectorSecurity)[f];

            if (strcmp (s->GetName()->Get(), data) == 0) {
                DBFactory::SetPrefIntValue (STRING(PREF_CHARTP_RELATIVE), aView, 0, 999999999, 0, s->GetID());
                break;
            }
        }
    }

    return true;
}



/**
*
*/
const char* PrefPortfolioChartData::value (int row, int col) {
    *aBuffer = '\0';

    if (row == FL_LABEL_ROW)
        return "";
    else if (row == FL_HEADER_ROW) {
        switch (col) {
            case 0:
                return STRING(PROPERTY);

            case 1:
                return STRING(VALUE);
        }
    }
    else {
        if (row == 0) {
            if (col == 0)
                return STRING(CHART_LAYOUT);
            else {
                snprintf (aBuffer, 99, "%s_%d", STRING(PREF_CHARTP_LAYOUT), aView);
                return aChoiceLayout[DBFactory::GetPrefIntValue (aBuffer, XTRADER_DEFAULT_LAYOUT, XTRADER_LAST_LAYOUT, XTRADER_DEFAULT_LAYOUT)];
            }
        }
        else if (row >= 1 && row <= 6) {
            if (col == 0) {
                setLabel (row);
                return aBuffer;
            }
            else {
                setPreference (row, col);
                return aChoiceModels[DBFactory::GetPrefIntValue (aBuffer, PortfolioChart::NO_CHART, PortfolioChart::LAST_CHART, PortfolioChart::NO_CHART)];
            }
        }
        else if (row == 7) {
            if (col == 0)
                snprintf (aBuffer, 99, "%s (0 - 20)", STRING(CHART_VERTICAL_SPACE));
            else if (col == 1) {
                snprintf (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_CHARTP_VSPACE), aView, XTRADER_MIN_VSPACE, XTRADER_MAX_VSPACE, XTRADER_DEFAULT_VSPACE));
            }
        }
        else if (row == 8) {
            if (col == 0)
                snprintf (aBuffer, 99, "%s (0 - 20)", STRING(CHART_BORDER_SPACE));
            else if (col == 1) {
                snprintf (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_CHARTP_BSPACE), aView, XTRADER_MIN_BSPACE, XTRADER_MAX_BSPACE, XTRADER_DEFAULT_BSPACE));
            }
        }
        else if (row == 9) {
            if (col == 0)
                return STRING(INFO_TICK_WIDTH);
            else
                snprintf (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_CHARTP_LINEWIDTH), aView, XTRADER_MIN_LINE_WIDTH, XTRADER_MAX_LINE_WIDTH, XTRADER_DEFAULTP_LINE_WIDTH));
        }
        else if (row == 10) {
            if (col == 0)
                snprintf (aBuffer, 999, "%s", STRING(CANVAS_COLOR));
            else
                snprintf (aBuffer, 999, "%d", DBFactory::GetPrefIntValue (STRING(PREF_CHARTP_BGCOLOR), aView, 0, 255, 0));
        }
        else if (row == 11) {
            if (col == 0)
                return STRING(CHARTINDICATOR_RELATIVE);
            else {
                int sec = DBFactory::GetPrefIntValue (STRING(PREF_CHARTP_RELATIVE), aView, 0, 999999999, 0);

                for (int f = 0; f < aVectorSecurity->Size(); f++) {
                    Security* s = (Security*) (*aVectorSecurity)[f];
                    if (s->GetID() == sec) return aChoiceSecurity[f];
                }
                return "";
            }
        }
    }

    return aBuffer;
}



/**
*
*/
int PrefPortfolioChartData::width (int col) {
    if (col == 0)
        return 230;
    else
        return 320;
}
