/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include "table/PrefPriceChartData.h"
#include "db/DBFactory.h"
#include "db/Pref.h"
#include "db/Security.h"
#include "Enums.h"
#include "Resource.h"
#include <MHPrice.h>
#include <FL/Fl.h>
#include <math.h>

#define START_MODELS 5



/**
* Settings for the security charts
* @param int - View number to change settings for
*/
PrefPriceChartData::PrefPriceChartData (int view) :
BaseData (),
aModels (0, -1, MHDate::WEEKDAY) {
    int f;

    aView           = view;
    aChoiceLayout   = MHUtil::Allocate (5, 40);
    aChoiceModels   = MHUtil::Allocate (aModels.aChoice.aNames.Size(), 30);
    aChoiceSecurity = 0;
    aVectorSecurity = new MHVector ();

    STRNCPY (aChoiceLayout[0], STRING(LAYOUT_1), 29)
    STRNCPY (aChoiceLayout[1], STRING(LAYOUT_2EQUAL), 29)
    STRNCPY (aChoiceLayout[2], STRING(LAYOUT_2), 29)
    STRNCPY (aChoiceLayout[3], STRING(LAYOUT_3EQUAL), 29)
    STRNCPY (aChoiceLayout[4], STRING(LAYOUT_3), 29)

    for (f = 0; f < aModels.aChoice.aNames.Size(); f++) {
        MHString* s = (MHString*) aModels.aChoice.aNames[f];
        STRNCPY (aChoiceModels[f], s->Get(), 29)
    }
    reload ("Security");
}



/**
*
*/
PrefPriceChartData::~PrefPriceChartData () {
    MHUtil::Delete (aChoiceLayout, 5);
    MHUtil::Delete (aChoiceModels,  aModels.aChoice.aNames.Size());
    if (aChoiceSecurity) MHUtil::Delete (aChoiceSecurity, aVectorSecurity->Size() + 1);

    aVectorSecurity->Erase ();
    delete aVectorSecurity;
}



/**
*
*/
Fl_Align PrefPriceChartData::align (int row, int col) {
    return FL_ALIGN_LEFT;
}



/**
*
*/
const char** PrefPriceChartData::choice (int row, int col, int& size, int& selected, char* label) {
    size = selected = 0;

    if (row == 0) {
        snprintf (aBuffer, 99, "%s_%d", STRING(PREF_CHART_LAYOUT), aView);
        size     = 5;
        selected = DBFactory::GetPrefIntValue (aBuffer, XTRADER_DEFAULT_LAYOUT, XTRADER_LAST_LAYOUT, XTRADER_DEFAULT_LAYOUT);
        return (const char**) aChoiceLayout;
    }
    else if (row >= START_MODELS && row < START_MODELS + 6) {
        setPreference (row, col);
        size     = aModels.aChoice.aNames.Size();
        selected = 0;
        int tmp  = DBFactory::GetPrefIntValue (aBuffer, PriceChart::NO_CHART, PriceChart::LAST_CHART, PriceChart::NO_CHART);
        for (int f = 0; f < aModels.aChoice.aTypes.Size(); f++) {
            int selected2 = ((MHString*)aModels.aChoice.aTypes[f])->ToInt();
            if (tmp == selected2) {
                selected = f;
                break;
            }
        }
        return (const char**) aChoiceModels;
    }
    else if (row == START_MODELS + 21) {
        int sec = DBFactory::GetPrefIntValue (STRING(PREF_CHART_RELATIVE), aView, 0, 999999999, 0);

        size     = aVectorSecurity->Size();
        selected = 0;

        for (int f = 0; f < aVectorSecurity->Size(); f++) {
            Security* s = (Security*) (*aVectorSecurity)[f];
            if (s->GetID() == sec) {
                selected = f;
                break;
            }
        }
        strncpy (label, STRING(SELECT_SECURITY), 99);
        return (const char**) aChoiceSecurity;
    }
    return 0;
}



/**
*
*/
FL_TABLE_EDITOR PrefPriceChartData::editor_type (int row, int col, bool force_custom) {
    if (col == 1) {
        switch (row) {
            case 1:
            case 2:
            case 3:
            case START_MODELS + 6:
            case START_MODELS + 7:
            case START_MODELS + 8:
            case START_MODELS + 9:
            case START_MODELS + 10:
            case START_MODELS + 11:
            case START_MODELS + 12:
            case START_MODELS + 13:
            case START_MODELS + 14:
            case START_MODELS + 15:
            case START_MODELS + 16:
            case START_MODELS + 17:
            case START_MODELS + 18:
            case START_MODELS + 19:
            case START_MODELS + 20:
                return FL_INTEGER_EDITOR;

            case 4:
                return FL_DLG_COLOR_EDITOR;

            case 0:
            case START_MODELS:
            case START_MODELS + 1:
            case START_MODELS + 2:
            case START_MODELS + 3:
            case START_MODELS + 4:
            case START_MODELS + 5:
                return FL_LIST_EDITOR;

            case START_MODELS + 21:
                return FL_DLG_LIST_EDITOR;

            default:
                return (FL_TABLE_EDITOR)0;
        }
    }
    else if (col == 2) {
        switch (row) {
            case START_MODELS:
            case START_MODELS + 1:
            case START_MODELS + 2:
            case START_MODELS + 3:
            case START_MODELS + 4:
            case START_MODELS + 5:
                return FL_LIST_EDITOR;

            default:
                return (FL_TABLE_EDITOR)0;
        }
    }
    return (FL_TABLE_EDITOR)0;
}



/**
*
*/
void PrefPriceChartData::reload (const char* message) {
    if (strcmp (message, "Security") == 0) {
        if (aChoiceSecurity) MHUtil::Delete (aChoiceSecurity, aVectorSecurity->Size() + 1);
        aVectorSecurity->Erase ();
        Security::Load (aVectorSecurity);
        aVectorSecurity->Sort (Security::SORT_NAME);

        aChoiceSecurity = MHUtil::Allocate (aVectorSecurity->Size() + 1, 31);
        for (int f = 0; f < aVectorSecurity->Size(); f++) {
            Security* s = (Security*) (*aVectorSecurity)[f];
            STRNCPY (aChoiceSecurity[f], s->GetName()->Get(), 30)
        }
    }
}



/**
* Create label for chart area
* @param int - Row
*/
void PrefPriceChartData::setLabel (int row) {
    if (row == START_MODELS)
        snprintf (aBuffer, 999, "%s", STRING(CHART_AREA_TOP));
    else if (row == START_MODELS + 1)
        snprintf (aBuffer, 999, "%s", STRING(CHART_AREA_TOP2));
    else if (row == START_MODELS + 2)
        snprintf (aBuffer, 999, "%s", STRING(CHART_AREA_MIDDLE));
    else if (row == START_MODELS + 3)
        snprintf (aBuffer, 999, "%s", STRING(CHART_AREA_MIDDLE2));
    else if (row == START_MODELS + 4)
        snprintf (aBuffer, 999, "%s", STRING(CHART_AREA_BOTTOM));
    else if (row == START_MODELS + 5)
        snprintf (aBuffer, 999, "%s", STRING(CHART_AREA_BOTTOM2));
}



/**
* Create the window key to get or set chart type value for window area
* @param int - Row
* @param int - Column, (chart num really, every left/right area has max two different charts)
*/
void PrefPriceChartData::setPreference (int row, int col) {
    if (row == START_MODELS)
        snprintf (aBuffer, 99, "%s_%d_%d_%d", STRING(PREF_CHART_WINDOW), aView, XTRADER_WINDOW_TOP, col);
    else if (row == START_MODELS + 1)
        snprintf (aBuffer, 99, "%s_%d_%d_%d", STRING(PREF_CHART_WINDOW), aView, XTRADER_WINDOW_TOP2, col);
    else if (row == START_MODELS + 2)
        snprintf (aBuffer, 99, "%s_%d_%d_%d", STRING(PREF_CHART_WINDOW), aView, XTRADER_WINDOW_MIDDLE, col);
    else if (row == START_MODELS + 3)
        snprintf (aBuffer, 99, "%s_%d_%d_%d", STRING(PREF_CHART_WINDOW), aView, XTRADER_WINDOW_MIDDLE2, col);
    else if (row == START_MODELS + 4)
        snprintf (aBuffer, 99, "%s_%d_%d_%d", STRING(PREF_CHART_WINDOW), aView, XTRADER_WINDOW_BOTTOM, col);
    else if (row == START_MODELS + 5)
        snprintf (aBuffer, 99, "%s_%d_%d_%d", STRING(PREF_CHART_WINDOW), aView, XTRADER_WINDOW_BOTTOM2, col);
}



/**
*
*/
bool PrefPriceChartData::value (const char* data, int row, int col) {
    if (row == 0) {
        for (int f = 0; f <= XTRADER_LAST_LAYOUT; f++) {
            if (strcmp (data, aChoiceLayout[f]) == 0) {
                DBFactory::SetPrefIntValue (STRING(PREF_CHART_LAYOUT), aView, XTRADER_DEFAULT_LAYOUT, XTRADER_LAST_LAYOUT, XTRADER_DEFAULT_LAYOUT, f);
                break;
            }
        }
    }
    else if (row == 1)
        DBFactory::SetPrefIntValue (STRING(PREF_CHART_VSPACE), aView, XTRADER_MIN_VSPACE, XTRADER_MAX_VSPACE, XTRADER_DEFAULT_VSPACE, atoi (data));
    else if (row == 2)
        DBFactory::SetPrefIntValue (STRING(PREF_CHART_BSPACE), aView, XTRADER_MIN_BSPACE, XTRADER_MAX_BSPACE, XTRADER_DEFAULT_BSPACE, atoi (data));
    else if (row == 3)
        DBFactory::SetPrefIntValue (STRING(PREF_CHART_LINEWIDTH), aView, XTRADER_MIN_LINE_WIDTH, XTRADER_MAX_LINE_WIDTH, XTRADER_DEFAULT_LINE_WIDTH, atoi (data));
    else if (row == 4)
        DBFactory::SetPrefIntValue (STRING(PREF_CHART_BGCOLOR), aView, 0, 255, 0, atoi (data));
    else if (row >= START_MODELS && row < (START_MODELS + 6)) {
        setPreference (row, col);

        for (int f = 0; f < aModels.aChoice.aNames.Size(); f++) {
            MHString* s = (MHString*) aModels.aChoice.aNames[f];
            if (strcmp (data, s->Get()) == 0) {
                DBFactory::SetPrefIntValue (aBuffer, PriceChart::NO_CHART, PriceChart::LAST_CHART, PriceChart::NO_CHART, ((MHString*)aModels.aChoice.aTypes[f])->ToInt());
                break;
            }
        }
    }
    else if (row == START_MODELS + 6)
        DBFactory::SetPrefIntValue (STRING(PREF_IND_SMA_SHORT), aView, 2, 100, MHPrice::MVG_SHORT, atoi (data));
    else if (row == START_MODELS + 7)
        DBFactory::SetPrefIntValue (STRING(PREF_IND_SMA_LONG), aView, 2, 100, MHPrice::MVG_LONG, atoi (data));

    else if (row == START_MODELS + 8)
        DBFactory::SetPrefIntValue (STRING(PREF_IND_ESMA_SHORT), aView, 2, 100, MHPrice::MVG_ESHORT, atoi (data));
    else if (row == START_MODELS + 9)
        DBFactory::SetPrefIntValue (STRING(PREF_IND_ESMA_LONG), aView, 2, 100, MHPrice::MVG_ELONG, atoi (data));

    else if (row == START_MODELS + 10)
        DBFactory::SetPrefIntValue (STRING(PREF_IND_MACD_SHORT), aView, 2, 100, MHPrice::MVG_MACD_SHORT, atoi (data));
    else if (row == START_MODELS + 11)
        DBFactory::SetPrefIntValue (STRING(PREF_IND_MACD_LONG), aView, 2, 100, MHPrice::MVG_MACD_LONG, atoi (data));
    else if (row == START_MODELS + 12)
        DBFactory::SetPrefIntValue (STRING(PREF_IND_MACD_ESMA), aView, 2, 100, MHPrice::MVG_MACD_ESMA, atoi (data));

    else if (row == START_MODELS + 13)
        DBFactory::SetPrefIntValue (STRING(PREF_IND_ATR), aView, 2, 100, MHPrice::MVG_ATR, atoi (data));
    else if (row == START_MODELS + 14)
        DBFactory::SetPrefIntValue (STRING(PREF_IND_BOLLINGER), aView, 2, 100, MHPrice::MVG_BOLLINGER, atoi (data));
    else if (row == START_MODELS + 15)
        DBFactory::SetPrefIntValue (STRING(PREF_IND_MOMENTUM), aView, 2, 100, MHPrice::MVG_MOMENTUM, atoi (data));
    else if (row == START_MODELS + 16)
        DBFactory::SetPrefIntValue (STRING(PREF_IND_RSI), aView, 2, 100, MHPrice::MVG_RSI, atoi (data));
    else if (row == START_MODELS + 17)
        DBFactory::SetPrefIntValue (STRING(PREF_IND_STDDEV), aView, 2, 100, MHPrice::MVG_STD, atoi (data));
    else if (row == START_MODELS + 18)
        DBFactory::SetPrefIntValue (STRING(PREF_IND_STOCHASTIC_K), aView, 2, 100, MHPrice::MVG_STOCHASTICS_K, atoi (data));
    else if (row == START_MODELS + 19)
        DBFactory::SetPrefIntValue (STRING(PREF_IND_STOCHASTIC_SMA_K), aView, 1, 100, MHPrice::MVG_STOCHASTICS_SMA_K, atoi (data));
    else if (row == START_MODELS + 20)
        DBFactory::SetPrefIntValue (STRING(PREF_IND_STOCHASTIC_SMA_D), aView, 2, 100, MHPrice::MVG_STOCHASTICS_SMA_D, atoi (data));

    else if (row == START_MODELS + 21) {
        for (int f = 0; f < aVectorSecurity->Size(); f++) {
            Security* s = (Security*) (*aVectorSecurity)[f];

            if (strcmp (s->GetName()->Get(), data) == 0) {
                DBFactory::SetPrefIntValue (STRING(PREF_CHART_RELATIVE), aView, 0, 999999999, 0, s->GetID());
                break;
            }
        }
    }
    return true;
}



/**
*
*/
const char* PrefPriceChartData::value (int row, int col) {
    *aBuffer = '\0';

    if (row == FL_LABEL_ROW)
        return "";
    else if (row == FL_HEADER_ROW) {
        switch (col) {
            case 0:
                return STRING(PROPERTY);

            case 1:
            case 2:
                return STRING(VALUE);
        }
    }
    else {
        if (row == 0) {
            if (col == 0)
                return STRING(CHART_LAYOUT);
            else if (col == 1) {
                snprintf (aBuffer, 99, "%s_%d", STRING(PREF_CHART_LAYOUT), aView);
                return aChoiceLayout[DBFactory::GetPrefIntValue (aBuffer, XTRADER_DEFAULT_LAYOUT, XTRADER_LAST_LAYOUT, XTRADER_DEFAULT_LAYOUT)];
            }
        }
        else if (row == 1) {
            if (col == 0)
                snprintf (aBuffer, 99, "%s (0 - 20)", STRING(CHART_VERTICAL_SPACE));
            else if (col == 1) {
                snprintf (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_CHART_VSPACE), aView, XTRADER_MIN_VSPACE, XTRADER_MAX_VSPACE, XTRADER_DEFAULT_VSPACE));
            }
        }
        else if (row == 2) {
            if (col == 0)
                snprintf (aBuffer, 99, "%s (0 - 20)", STRING(CHART_BORDER_SPACE));
            else if (col == 1) {
                snprintf (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_CHART_BSPACE), aView, XTRADER_MIN_BSPACE, XTRADER_MAX_BSPACE, XTRADER_DEFAULT_BSPACE));
            }
        }
        else if (row == 3) {
            if (col == 0)
                return STRING(INFO_TICK_WIDTH);
            else if (col == 1)
                snprintf (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_CHART_LINEWIDTH), aView, XTRADER_MIN_LINE_WIDTH, XTRADER_MAX_LINE_WIDTH, XTRADER_DEFAULT_LINE_WIDTH));
        }
        else if (row == 4) {
            if (col == 0)
                snprintf (aBuffer, 999, "%s", STRING(CANVAS_COLOR));
            else if (col == 1)
                snprintf (aBuffer, 999, "%d", DBFactory::GetPrefIntValue (STRING(PREF_CHART_BGCOLOR), aView, 0, 255, 0));
        }
        else if (row >= START_MODELS && row < (START_MODELS + 6)) {
            if (col == 0) {
                setLabel (row);
                return aBuffer;
            }
            else {
                setPreference (row, col);
                int model = DBFactory::GetPrefIntValue (aBuffer, PriceChart::NO_CHART, PriceChart::LAST_CHART, PriceChart::NO_CHART);
                for (int f = 0; f < aModels.aChoice.aNames.Size(); f++) {
                    int newModel = ((MHString*)aModels.aChoice.aTypes[f])->ToInt();
                    if (model == newModel) {
                        MHString* s = (MHString*) aModels.aChoice.aNames[f];
                        return s->Get();
                    }
                }
            }
        }
        else if (row == START_MODELS + 6) {
            if (col == 0)
                snprintf (aBuffer, 99, "%s %s (2 - 100)", STRING(CHARTINDICATOR_SMA), STRING(SHORT));
            else if (col == 1)
                snprintf (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_IND_SMA_SHORT), aView, 2, 100, MHPrice::MVG_SHORT));
        }
        else if (row == START_MODELS + 7) {
            if (col == 0)
                snprintf (aBuffer, 99, "%s %s (2 - 100)", STRING(CHARTINDICATOR_SMA), STRING(LONG));
            else if (col == 1)
                snprintf (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_IND_SMA_LONG), aView, 2, 100, MHPrice::MVG_LONG));
        }
        else if (row == START_MODELS + 8) {
            if (col == 0)
                snprintf (aBuffer, 99, "%s %s (2 - 100)", STRING(CHARTINDICATOR_ESMA), STRING(SHORT));
            else if (col == 1)
                snprintf (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_IND_ESMA_SHORT), aView, 2, 100, MHPrice::MVG_ESHORT));
        }
        else if (row == START_MODELS + 9) {
            if (col == 0)
                snprintf (aBuffer, 99, "%s %s (2 - 100)", STRING(CHARTINDICATOR_ESMA), STRING(LONG));
            else if (col == 1)
                snprintf (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_IND_ESMA_LONG), aView, 2, 100, MHPrice::MVG_ELONG));
        }
        else if (row == START_MODELS + 10) {
            if (col == 0)
                snprintf (aBuffer, 99, "%s %s (2 - 100)", STRING(CHARTINDICATOR_MACD), STRING(SHORT));
            else if (col == 1)
                snprintf (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_IND_MACD_SHORT), aView, 2, 100, MHPrice::MVG_MACD_SHORT));
        }
        else if (row == START_MODELS + 11) {
            if (col == 0)
                snprintf (aBuffer, 99, "%s %s (2 - 100)", STRING(CHARTINDICATOR_MACD), STRING(LONG));
            else if (col == 1)
                snprintf (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_IND_MACD_LONG), aView, 2, 100, MHPrice::MVG_MACD_LONG));
        }
        else if (row == START_MODELS + 12) {
            if (col == 0)
                snprintf (aBuffer, 99, "%s %s (2 - 100)", STRING(CHARTINDICATOR_MACD), STRING(CHARTINDICATOR_ESMA));
            else if (col == 1)
                snprintf (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_IND_MACD_ESMA), aView, 2, 100, MHPrice::MVG_MACD_ESMA));
        }
        else if (row == START_MODELS + 13) {
            if (col == 0)
                snprintf (aBuffer, 99, "%s (2 - 100)", STRING(CHARTINDICATOR_ATR));
            else if (col == 1)
                snprintf (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_IND_ATR), aView, 2, 100, MHPrice::MVG_ATR));
        }
        else if (row == START_MODELS + 14) {
            if (col == 0)
                snprintf (aBuffer, 99, "%s (2 - 100)", STRING(CHARTINDICATOR_BOLLINGER));
            else if (col == 1)
                snprintf (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_IND_BOLLINGER), aView, 2, 100, MHPrice::MVG_BOLLINGER));
        }
        else if (row == START_MODELS + 15) {
            if (col == 0)
                snprintf (aBuffer, 99, "%s (2 - 100)", STRING(CHARTINDICATOR_MOMENTUM));
            else if (col == 1)
                snprintf (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_IND_MOMENTUM), aView, 2, 100, MHPrice::MVG_MOMENTUM));
        }
        else if (row == START_MODELS + 16) {
            if (col == 0)
                snprintf (aBuffer, 99, "%s (2 - 100)", STRING(CHARTINDICATOR_RSI));
            else if (col == 1)
                snprintf (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_IND_RSI), aView, 2, 100, MHPrice::MVG_RSI));
        }
        else if (row == START_MODELS + 17) {
            if (col == 0)
                snprintf (aBuffer, 99, "%s (2 - 100)", STRING(CHARTINDICATOR_STDDEV));
            else if (col == 1)
                snprintf (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_IND_STDDEV), aView, 2, 100, MHPrice::MVG_STD));
        }
        else if (row == START_MODELS + 18) {
            if (col == 0)
                snprintf (aBuffer, 99, "%s (2 - 100)", STRING(CHARTINDICATOR_STOCHASTICS));
            else if (col == 1)
                snprintf (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_IND_STOCHASTIC_K), aView, 2, 100, MHPrice::MVG_STOCHASTICS_K));
        }
        else if (row == START_MODELS + 19) {
            if (col == 0)
                snprintf (aBuffer, 99, "%s %%K (1 - 100)", STRING(CHARTINDICATOR_STOCHASTICS));
            else if (col == 1)
                snprintf (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_IND_STOCHASTIC_SMA_K), aView, 1, 100, MHPrice::MVG_STOCHASTICS_SMA_K));
        }
        else if (row == START_MODELS + 20) {
            if (col == 0)
                snprintf (aBuffer, 99, "%s %%D (2 - 100)", STRING(CHARTINDICATOR_STOCHASTICS));
            else if (col == 1)
                snprintf (aBuffer, 99, "%d", DBFactory::GetPrefIntValue (STRING(PREF_IND_STOCHASTIC_SMA_D), aView, 2, 100, MHPrice::MVG_STOCHASTICS_SMA_D));
        }
        else if (row == START_MODELS + 21) {
            if (col == 0)
                return STRING(CHARTINDICATOR_RELATIVE);
            else if (col == 1) {
                int sec = DBFactory::GetPrefIntValue (STRING(PREF_CHART_RELATIVE), aView, 0, 999999999, 0);

                for (int f = 0; f < aVectorSecurity->Size(); f++) {
                    Security* s = (Security*) (*aVectorSecurity)[f];
                    if (s->GetID() == sec) return aChoiceSecurity[f];
                }
                return "";
            }
        }
    }

    return aBuffer;
}



/**
*
*/
int PrefPriceChartData::width (int col) {
    if (col == 0)
        return 230;
    else
        return 160;
}
