/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <table/PrefToolbarData.h>
#include <db/DBFactory.h>
#include <db/Pref.h>
#include <report/PriceReport.h>
#include <Enums.h>
#include <Resource.h>
#include <FL/Fl.h>



/**
*
*/
PrefToolbarData::PrefToolbarData () :
BaseData () {
    aButtons = MHUtil::Allocate (PrefToolbarData::BUTTONS, 25);

    STRNCPY (aButtons[0], " ", 24)
    STRNCPY (aButtons[1], STRING(TOOL_DATABASE), 24)
    STRNCPY (aButtons[2], STRING(TOOL_DOWNLOAD), 24)
    STRNCPY (aButtons[3], STRING(TOOL_EXPORT), 24)
    STRNCPY (aButtons[4], STRING(TOOL_IMPORT), 24)
    STRNCPY (aButtons[5], STRING(TOOL_PREFERENCE), 24)
    STRNCPY (aButtons[6], STRING(TOOL_PORTFOLIO_CHART), 24)
    STRNCPY (aButtons[7], STRING(TOOL_PORTFOLIO_REPORT), 24)
    STRNCPY (aButtons[8], STRING(TOOL_PRICE_CHART), 24)
    STRNCPY (aButtons[9], STRING(TOOL_PRICE_REPORT), 24)
    STRNCPY (aButtons[10], STRING(TOOL_PRICE_WATCH), 24)
    STRNCPY (aButtons[11], STRING(TOOL_PRICE_WATCH_REPORT), 24)
    STRNCPY (aButtons[12], STRING(TOOL_QUIT), 24)
    //STRNCPY (aButtons[13], STRING(TOOL_SPLIT), 24)
    STRNCPY (aButtons[13], STRING(TOOL_VIEW_LOG), 24)
    STRNCPY (aButtons[14], STRING(TOOL_WORKSPACE), 24)

    MHString tool (DBFactory::GetPrefStringValue (STRING(PREF_TOOLBAR), ""));

    tool.Split (':', &aSelected);
    for (int f = aSelected.Size(); f <= BUTTONS; f++)
        aSelected.Push (new MHString (" "));
}



/**
*
*/
PrefToolbarData::~PrefToolbarData () {
    MHUtil::Delete (aButtons, BUTTONS);
    aSelected.Erase ();
}



/**
*
*/
Fl_Align PrefToolbarData::align (int row, int col) {
    return FL_ALIGN_LEFT;
}



/**
*
*/
const char** PrefToolbarData::choice (int row, int col, int& size, int& selected, char* label) {
    size = selected = 0;

    if (col == 1) {
        size     = BUTTONS;
        selected = 0;
        return (const char**) aButtons;
    }
    return 0;
}



/**
*
*/
FL_TABLE_EDITOR PrefToolbarData::editor_type (int row, int col, bool force_custom) {
    if (col == 1) {
        return FL_LIST_EDITOR;
    }
    return (FL_TABLE_EDITOR)0;
}



/**
*
*/
bool PrefToolbarData::value (const char* data, int row, int col) {
    if (row >= 0 && row < BUTTONS) {
        MHString  s;
        MHString* curr = (MHString*) aSelected.Get (row);

        if (curr)
            *curr = data;
        for (int f = 0; f < BUTTONS; f++) {
            MHString* s2 = (MHString*) aSelected[f];
            if (s2) {
                if (s.Size() > 0)
                    s += ":";
                s += *s2;
            }
            else
                s += ":";
        }
        DBFactory::SetPrefStringValue (STRING(TOOLBAR), s());
    }
    return true;
}



/**
*
*/
const char* PrefToolbarData::value (int row, int col) {
    if (row == FL_LABEL_ROW)
        return "";
    else if (row == FL_HEADER_ROW) {
        switch (col) {
            case 0:
                return STRING(PROPERTY);

            case 1:
                return STRING(VALUE);
        }
    }
    else {
        if (col == 0) {
            snprintf (aBuffer, 999, "%s %d", STRING(BUTTON), row + 1);
            return aBuffer;
        }
        else {
            MHString* s = (MHString*) aSelected.Get (row);

            if (s) {
                return s->Get();
            }
            return "";
        }
    }

    return "";
}



/**
*
*/
int PrefToolbarData::width (int col) {
    if (col == 0)
        return 230;
    else
        return 320;
}
