/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <table/PriceData.h>
#include <db/DBFactory.h>
#include <db/Price.h>
#include <db/Security.h>
#include <gui/XTrader.h>
#include <Resource.h>
#include <MHDate.h>
#include <MHString.h>
#include <FL/Fl.h>



/**
*  @param Security* - The security object to load price data for.
*/
PriceData::PriceData (Security* s) :
BaseData () {
    aVector   = new MHVector (500);
    aSecurity = s;
    aChoice   = MHUtil::Allocate (3, 30);

    STRNCPY (aChoice[0], STRING(PRICE), 20)
    STRNCPY (aChoice[1], STRING(CLOSED), 20)
    STRNCPY (aChoice[2], STRING(SPLIT), 20)

    Price::LoadAll (aSecurity, aVector);
}



/**
*
*/
PriceData::~PriceData () {
    MHUtil::Delete (aChoice, 3);
    aVector->Erase ();
    delete aVector;
    delete aSecurity;
}



/**
*
*/
void PriceData::add () {
    insert (aVector->Size());
}



/**
*
*/
Fl_Align PriceData::align (int row, int col) {
    switch (col) {
        case 0:
        case 1:
            return FL_ALIGN_LEFT;

        default:
            return FL_ALIGN_RIGHT;
    }
}



/**
*
*/
const char** PriceData::choice (int row, int col, int& size, int& selected, char* label) {
    Price* p = (Price*) aVector->Get (row);

    size = 3;

    if (p) {
        if (p->aType == Price::SPLIT)
            selected = 2;
        else if (p->aType == Price::CLOSED)
            selected = 1;
        else
            selected = 0;

    }

    return (char const**) aChoice;
}



/**
*
*/
FL_TABLE_EDITOR PriceData::editor_type (int row, int col, bool force_custom) {
    switch (col) {
        case 0:
            return FL_TEXT_EDITOR;

        case 1:
            return FL_LIST_EDITOR;

        default:
            return FL_FLOAT_EDITOR;
    }
}



/**
*
*/
void PriceData::erase (int row) {
    if (row >= 0 && row < aVector->Size()) {
        aVector->Erase (row);
        dirty (true);
    }
}



/**
*
*/
void PriceData::insert (int row) {
    MHDate now;
    Price* p1 = new Price ("", Price::PRICE, -1, -1, -1, -1, -1);
    Price* p2 = (Price*) aVector->Search (p1, MHVector::FIND_EXACT, Price::SORT_DATE_TYPE, 0);

    if (p2) {
        delete p1;
        XTrader::SetError (STRING(ERROR_DATA_ERROR));
        return;
    }
    aVector->Insert (p1, row);
    dirty (true);
}



/**
*
*/
void PriceData::reload (const char* message) {
}


/**
*
*/
int PriceData::save () {
    for (int f = 0; f < aVector->Size(); f++) {
        Price* p = (Price*) (*aVector)[f];
        if (MHDate::ValidDate(p->aDate) == false)
            return f;
        switch (p->aType) {
            case Price::PRICE:
                if (p->aClose > p->aHigh ||
                    p->aClose < p->aLow)
                    return f;

                if (p->aOpen > 0.00000001) {
                    if (p->aOpen > p->aHigh ||
                        p->aOpen < p->aLow)
                        return f;
                }
                break;

            case Price::SPLIT:
                if (p->aHigh < 0.0000001)
                    return f;
                break;

            default:
                p->aHigh  = -1.0;
                p->aLow   = -1.0;
                p->aClose = -1.0;
                p->aOpen  = -1.0;
                p->aVol   = -1.0;
                break;
        }
    }

    Price::Save (aSecurity, aVector);
    DBFactory::ClearPrice();
    dirty (false);
    XTrader::SetInfo (STRING(DATA_SAVED));
    XTrader::SendMessage (0, "Price", "changed", "", aSecurity);
    return -1;
}



/**
*
*/
void PriceData::sort (int col, bool ascending) {
}



/**
*
*/
bool PriceData::value (const char* data, int row, int col) {
    if (row < 0 || row >= aVector->Size()) return true;

    Price* s  = (Price*) (*aVector)[row];
    ASS (s)

    switch (col) {
        case 0: {
            if (strcmp (s->aDate, data) == 0) return true;
            Price p(data, s->aType, 0);
            if (MHDate::ValidDate (data) == false || aVector->Search (&p, MHVector::SEARCH_ALL, Price::SORT_DATE_TYPE, 0) != 0)
                return false;
            s->SetDate (data);
            break;
        }

        case 1: {
            Price::TYPE type;
            if (strcmp(STRING(PRICE), data) == 0)
                type = Price::PRICE;
            else if (strcmp(STRING(SPLIT), data) == 0)
                type = Price::SPLIT;
            else
                type = Price::CLOSED;
            
            if (type == s->aType) return true;
            
            Price p(s->aDate, type, 0.0);
            if (aVector->Search (&p, MHVector::SEARCH_ALL, Price::SORT_DATE_TYPE, 0))
                return false;
            
            s->aType = type;
            break;
        }

        case 2:
            s->aHigh = atof(data);
            break;

        case 3:
            s->aLow = atof(data);
            break;

        case 4:
            s->aClose = atof(data);
            break;

        case 5:
            s->aOpen = atof(data);
            break;

        case 6:
            s->aVol = atof(data);
            break;
    }

    s->IsChanged (true);
    dirty (true);
    return true;
}



/**
*
*/
const char* PriceData::value (int row, int col) {
    if (row == FL_LABEL_ROW)
        return aSecurity ? aSecurity->GetName()->Get() : "PriceData::value !!!";
    else if (row == FL_HEADER_ROW) {
        switch (col) {
            case 0:
                return STRING(DATE);

            case 1:
                return STRING(TYPE);

            case 2:
                return STRING(HIGH);

            case 3:
                return STRING(LOW);

            case 4:
                return STRING(CLOSE_);

            case 5:
                return STRING(OPEN);

            case 6:
                return STRING(VOLUME);

            default:
                return "";
        }
    }
    else {
        Price* s = (Price*) aVector->Get (row);
        if (!s) return "";

        switch (col) {
            case 0:
                return s->aDate;

            case 1:
                if (s->aType == Price::CLOSED)
                    STRNCPY (aBuff, STRING(CLOSED), 20)
                else if (s->aType == Price::SPLIT)
                    STRNCPY (aBuff, STRING(SPLIT), 20)
                else
                    STRNCPY (aBuff, STRING(PRICE), 20)
                return aBuff;

            case 2:
                snprintf (aBuff, 30, "%.2f", s->aHigh);
                return aBuff;

            case 3:
                snprintf (aBuff, 30, "%.2f", s->aLow);
                return aBuff;

            case 4:
                snprintf (aBuff, 30, "%.2f", s->aClose);
                return aBuff;

            case 5:
                snprintf (aBuff, 30, "%.2f", s->aOpen);
                return aBuff;

            case 6:
                snprintf (aBuff, 30, "%.0f", s->aVol);
                return aBuff;

            default:
                return "";
        }
    }
}



/**
*
*/
int PriceData::width (int col) {
    return XTrader::FontSize() * 6;
}
