/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <table/ReportData.h>
#include <report/Report.h>
#include <Resource.h>
#include <FL/Fl.h>



/**
*  Caller is responsible for deleting report object
*/
ReportData::ReportData (Report* report) :
BaseData () {
    aReport = report;
}



/**
*
*/
ReportData::~ReportData () {
}



/**
*
*/
Fl_Align ReportData::align (int row, int col) {
    MHVector* cols = aReport->GetAlign ();

    if (col < cols->Size()) {
        const char* string = ((MHString*)(*cols)[col])->Get();
        int         align  = *string;

        if (align & Report::LEFT)
            return FL_ALIGN_LEFT;
        else if (align & Report::RIGHT)
            return FL_ALIGN_RIGHT;
        else
            return FL_ALIGN_CENTER;
    }
    return FL_ALIGN_LEFT;
}



/**
*
*/
const char** ReportData::choice (int row, int col, int& size, int& selected, char* label) {
    return 0;
}



/**
*
*/
FL_TABLE_EDITOR ReportData::editor_type (int row, int col, bool force_custom) {
    return (FL_TABLE_EDITOR)0;
}



/**
*
*/
void ReportData::reload (const char* message) {
}



/**
*
*/
int ReportData::save () {
    return -1;
}



/**
*
*/
void ReportData::sort (int col, bool ascending) {
    MHString*   str  = (MHString*) aReport->aAlign.Get(col);
    int         type = str->GetChar(0);

    if (type & Report::NUMBER)
        aReport->aRows.Sort (MHString::SORT_NUM, col);
    else
        aReport->aRows.Sort (MHString::SORT_STRING, col);
    if (ascending == false) aReport->aRows.Reverse ();
}



/**
*
*/
Fl_Color ReportData::textcolor (int row, int col) {
    if (row >= FL_ROW) {
        MHVector*   prop   = aReport->GetAlign ();
        MHString*   string = (MHString*) prop->Get (col);
        const char* buffer = (string) ? string->Get() : "";
        int         align  = *buffer;

        if (align & Report::NUMBER) {
            MHVector*   rows   = aReport->GetRows ();
            MHVector*   cols   = (MHVector*) rows->Get (row);
            MHString*   val    = (MHString*) cols->Get (col);

            if (val) {
                double value  = val->ToDouble ();

                if (value < 0.001 && value > -0.001)
                    return FL_BLACK;
                else if (value < 0.000001)
                    return FL_RED;
                else
                    return FL_BLUE;
            }
        }
    }
    return Fl_Table_Data::textcolor(row, col);
}



/**
*
*/
bool ReportData::value (const char* data, int row, int col) {
    return false;
}



/**
*
*/
const char* ReportData::value (int row, int col) {
    if (row == FL_LABEL_ROW)
        return aReport->GetTitle();
    else if (row == FL_HEADER_ROW) {
        MHVector* cols = aReport->GetHeader ();

        if (col < cols->Size()) {
            return ((MHString*)(*cols)[col])->Get();
        }
        return "";
    }
    else {
        MHVector* rows = aReport->GetRows ();

        if (row < rows->Size()) {
            MHVector* cols = ((MHVector*)(*rows)[row]);

            if (col < cols->Size()) {
                return ((MHString*)(*cols)[col])->Get();
            }
        }
        return "";
    }
}



/**
*
*/
int ReportData::width (int col) {
    switch (col) {
        case 0:
            return XTrader::FontSize() * 12;

        default:
            return XTrader::FontSize() * 7;
    }
}
