/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <table/SecurityData.h>
#include <db/DBFactory.h>
#include <db/Security.h>
#include <Resource.h>
#include <FL/Fl.h>



/**
*
*/
SecurityData::SecurityData () :
BaseData () {
    show_label(false);
    aVector = new MHVector (500);
    Security::Load (aVector);
    aVector->Sort (Security::SORT_NAME);
    aCurrSort = Security::SORT_NAME;
}



/**
*
*/
SecurityData::~SecurityData () {
    aVector->Erase ();
    delete aVector;
}



/**
*
*/
void SecurityData::add () {
    insert (aVector->Size());
}



/**
*
*/
Fl_Align SecurityData::align (int row, int col) {
    switch (col) {
        case 4:
            return FL_ALIGN_RIGHT;

        default:
            return FL_ALIGN_LEFT;
    }
}



/**
*
*/
const char** SecurityData::choice (int row, int col, int& size, int& selected, char* label) {
    return 0;
}



/**
*
*/
FL_TABLE_EDITOR SecurityData::editor_type (int row, int col, bool force_custom) {
    switch (col) {
        case 4:
            return FL_INTEGER_EDITOR;

        default:
            return FL_TEXT_EDITOR;
    }
}



/**
*
*/
void SecurityData::erase (int row) {
    if (row >= 0 && row < aVector->Size()) {
        aVector->Erase (row);
        dirty (true);
    }
}



/**
*
*/
void SecurityData::insert (int row) {
    Security* s1 = new Security(DBFactory::GetUniqueID(), "", "");
    Security* s2 = (Security*) aVector->Search (s1, MHVector::FIND_EXACT, Security::SORT_CODE, 0);

    if (!s2)
        s2 = (Security*) aVector->Search (s1, MHVector::FIND_EXACT, Security::SORT_NAME, 0);

    if (s2) {
        delete s1;
        XTrader::SetError (STRING(ERROR_DATA_ERROR));
        return;
    }
    aVector->Insert (s1, row);
    dirty (true);
}



/**
*
*/
int SecurityData::save () {
    Security::Save (aVector);
    DBFactory::ClearSecurity();
    aVector->Sort (aCurrSort);
    dirty (false);
    XTrader::SetInfo (STRING(DATA_SAVED));
    XTrader::SendMessage (0, "Security", "changed");
    return -1;
}



/**
*
*/
void SecurityData::sort (int col, bool ascending) {
    switch (col) {
        case 0:
            aVector->Sort (Security::SORT_CODE);
            if (ascending == false) aVector->Reverse ();
            aCurrSort = Security::SORT_CODE;
            break;

        case 1:
            aVector->Sort (Security::SORT_NAME);
            if (ascending == false) aVector->Reverse ();
            aCurrSort = Security::SORT_NAME;
            break;
    }
}



/**
*
*/
bool SecurityData::value (const char* data, int row, int col) {
    if (row < 0 || row >= aVector->Size()) return true;

    Security* s  = (Security*) (*aVector)[row];
    Security  s2 = *s;

    try {
        switch (col) {
            case 0: {
                if (strcmp (s->GetCode()->Get(), data) == 0) return true;
                Security s3(0, data, "");
                if (aVector->Search (&s3, MHVector::FIND_EXACT, Security::SORT_CODE, 0) != 0)
                    return false;
                s->SetCode (data);
                break;
            }

            case 1: {
                if (strcmp (s->GetName()->Get(), data) == 0) return true;
                //Security se(0, "", data);
                //if (aVector->Search (&se, MHVector::FIND_EXACT, Security::SORT_NAME, 0) != 0)
                //    return false;
                s->SetName (data);
                break;
            }

            case 2:
                s->SetMarket (data);
                break;

            case 3:
                s->SetSector (data);
                break;

            case 4:
                if (atof (data) >= 0.0)
                    s->SetNoShares (atof (data));
                else
                    return false;
                break;
        }
    }
    catch (const char*) {
        return false;
    }

    dirty (true);
    return true;
}



/**
*
*/
const char* SecurityData::value (int row, int col) {
    if (row == FL_LABEL_ROW)
        return "";
    else if (row == FL_HEADER_ROW) {
        switch (col) {
            case 0:
                return STRING(CODE);

            case 1:
                return STRING(NAME);

            case 2:
                return STRING(MARKET);

            case 3:
                return STRING(SECTOR);

            case 4:
                return STRING(NO_SHARES);
        }
    }
    else {
        Security* s = (Security*) aVector->Get (row);
        if (!s) return "";

        switch (col) {
            case 0:
                return s->GetCode()->Get();

            case 1:
                return s->GetName()->Get();

            case 2:
                return s->GetMarket()->Get();

            case 3:
                return s->GetSector()->Get();

            case 4:
                snprintf (aBuff, 30, "%d", int(s->GetNoShares()));
                return aBuff;
        }
    }

    return "";
}



/**
*
*/
int SecurityData::width (int col) {
    switch (col) {
        case 1:
            return XTrader::FontSize() * 12;

        case 0:
        case 2:
        case 3:
            return XTrader::FontSize() * 8;

        default:
            return XTrader::FontSize() * 10;
    }
}
