/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <table/SetData.h>
#include <db/DBFactory.h>
#include <db/Set.h>
#include <Resource.h>
#include <FL/Fl.h>



/**
*
*/
SetData::SetData () :
BaseData () {
    show_label(false);
    aVector = new MHVector ();
    Set::Load (aVector);
}



/**
*
*/
SetData::~SetData () {
    aVector->Erase ();
    delete aVector;
}



/**
*
*/
void SetData::add () {
    insert (aVector->Size());
}



/**
*
*/
Fl_Align SetData::align (int row, int col) {
    return FL_ALIGN_LEFT;
}



/**
*
*/
const char** SetData::choice (int row, int col, int& size, int& selected, char* label) {
    return 0;
}



/**
*
*/
FL_TABLE_EDITOR SetData::editor_type (int row, int col, bool force_custom) {
    return FL_TEXT_EDITOR;
}



/**
*
*/
void SetData::erase (int row) {
    if (row >= 0 && row < aVector->Size()) {
        aVector->Erase (row);
        dirty (true);
    }
}



/**
*
*/
void SetData::insert (int row) {
    Set* s1 = new Set(DBFactory::GetUniqueID(), "");
    Set* s2 = (Set*) aVector->Search (s1, MHVector::FIND_EXACT, 0, 0);

    if (s2) {
        delete s1;
        XTrader::SetError (STRING(ERROR_DATA_ERROR));
        return;
    }
    aVector->Insert (s1, row);
    dirty (true);
}



/**
*
*/
int SetData::save () {
    Set::Save (aVector);
    DBFactory::ClearSet();
    dirty (false);
    XTrader::SetInfo (STRING(DATA_SAVED));
    XTrader::SendMessage (0, "Set", "changed");
    return -1;
}



/**
*
*/
void SetData::sort (int col, bool ascending) {
}



/**
*
*/
bool SetData::value (const char* data, int row, int col) {
    if (row < 0 || row >= aVector->Size()) return true;

    Set* s = (Set*) (*aVector)[row];
    ASS (s)

    switch (col) {
    case 0:
        if (strcmp (s->GetSet()->Get(), data) == 0) return true;
        Set s2(0, data);
        if (aVector->Search (&s2, MHVector::FIND_EXACT, 0, 0) != 0)
            return false;
        s->SetSet (data);
        break;
    }

    dirty (true);
    return true;
}



/**
*
*/
const char* SetData::value (int row, int col) {
    if (row == FL_LABEL_ROW)
        return "";
    else if (row == FL_HEADER_ROW) {
        switch (col) {
        case 0:
            return STRING(SECURITY_LIST);
        }
    }
    else {
        Set* s = (Set*) aVector->Get (row);
        if (!s) return "";

        switch (col) {
        case 0:
            return s->GetSet()->Get();
        }
    }

    return "";
}



/**
*
*/
int SetData::width (int col) {
    return XTrader::FontSize() * 12;
}
