/**
*  Copyright Mikael Hgdahl - triyana@users.sourceforge.net
*
*  This source is distributed under the terms of the Q Public License version 1.0,
*  created by Trolltech (www.trolltech.com).
*/

#include <table/SetItemData.h>
#include <db/Security.h>
#include <db/SetItem.h>
#include <Resource.h>
#include <FL/Fl.h>



/**
*
*/
SetItemData::SetItemData (Set* set) :
BaseData () {
    show_label(false);
    aVector         = new MHVector ();
    aVectorSecurity = new MHVector ();

    aSet    = *set;
    aChoice = 0;
    SetItem::Load (&aSet, aVector);
    SetItem::LoadSecurities (aVector);
    aVector->Sort (SetItem::SORT_SECURITY);
    reload ("Security");
}



/**
*
*/
SetItemData::~SetItemData () {
    if (aChoice) MHUtil::Delete (aChoice, aVectorSecurity->Size() + 1);
    aVector->Erase ();
    aVectorSecurity->Erase ();
    delete aVector;
    delete aVectorSecurity;
}



/**
*
*/
void SetItemData::add () {
    insert (aVector->Size());
}



/**
*
*/
Fl_Align SetItemData::align (int row, int col) {
    return FL_ALIGN_LEFT;
}



/**
*
*/
const char** SetItemData::choice (int row, int col, int& size, int& selected, char* label) {
    SetItem* s = (SetItem*) aVector->Get (row);
    size       = aVectorSecurity->Size();
    selected   = 0;

    for (int f = 0; f < aVectorSecurity->Size(); f++) {
        Security* s2 = (Security*) (*aVectorSecurity)[f];
        if (s->GetSecurityID() == s2->GetID()) {
            selected = f;
            break;
        }
    }

    strncpy (label, STRING(SELECT_SECURITY), 99);
    return (char const**) aChoice;
}



/**
*
*/
FL_TABLE_EDITOR SetItemData::editor_type (int row, int col, bool force_custom) {
    return FL_DLG_LIST_EDITOR;
}



/**
*
*/
void SetItemData::erase (int row) {
    if (row >= 0 && row < aVector->Size()) {
        aVector->Erase (row);
        dirty (true);
    }
}



/**
*
*/
void SetItemData::insert (int row) {
    Security* s = (Security*) aVectorSecurity->Get(0);

    if (s) {
        SetItem* i = new SetItem (s->GetID());
        i->LoadSecurity();
        aVector->Insert (i, row);
        dirty (true);
    }
}



/**
*
*/
void SetItemData::reload (const char* message) {
    if (strcmp (message, "Security") == 0) {
        SetItem::LoadSecurities (aVector);
        aVector->Sort (SetItem::SORT_SECURITY);

        if (aChoice) MHUtil::Delete (aChoice, aVectorSecurity->Size() + 1);
        aVectorSecurity->Erase ();
        Security::Load (aVectorSecurity);
        aVectorSecurity->Sort (Security::SORT_NAME);

        aChoice = MHUtil::Allocate (aVectorSecurity->Size() + 1, 31);
        for (int f = 0; f < aVectorSecurity->Size(); f++) {
            Security* s = (Security*) (*aVectorSecurity)[f];
            STRNCPY (aChoice[f], s->GetName()->Get(), 30)
        }
    }
}


/**
*
*/
int SetItemData::save () {
    SetItem::Save (&aSet, aVector);
    aVector->Sort (SetItem::SORT_SECURITY);
    dirty (false);
    XTrader::SetInfo (STRING(DATA_SAVED));
    XTrader::SendMessage (0, "SetItem", "changed");
    return -1;
}



/**
*
*/
void SetItemData::sort (int col, bool ascending) {
}



/**
*
*/
bool SetItemData::value (const char* data, int row, int col) {
    SetItem* i = (SetItem*) aVector->Get (row);

    if (i) {
        switch (col) {
        case 0:
            for (int f = 0; f < aVectorSecurity->Size(); f++) {
                Security* s = (Security*) (*aVectorSecurity)[f];

                if (strcmp (s->GetName()->Get(), data) == 0) {
                    i->SetSecurityID (s->GetID());
                    i->LoadSecurity ();
                    break;
                }
            }
            break;
        }
    }
    dirty (true);
    return true;
}



/**
*
*/
const char* SetItemData::value (int row, int col) {
    if (row == FL_LABEL_ROW)
        return "";
    else if (row == FL_HEADER_ROW) {
        switch (col) {
        case 0:
            return STRING(SECURITY);
        }
    }
    else {
        SetItem* s = (SetItem*) aVector->Get (row);
        if (!s) return "";

        switch (col) {
            case 0:
                return s->GetSecurity()->GetName()->Get();
        }
    }

    return "";
}



/**
*
*/
int SetItemData::width (int col) {
    return XTrader::FontSize() * 12;
}
